@@ -52,7 +52,7 @@ define(["jquery", "app/common", "stickytabs", "visibility",
     function cleanCredentials() {
         sessionStorage.clear();
         $("#statWidgets").empty();
-        $("#listMaps").empty();
+        $("#listMaps").children("tbody").empty();
         $("#modalBody").empty();
     }
 
@@ -92,7 +92,7 @@ define(["jquery", "app/common", "stickytabs", "visibility",
             tab_id = "#" + $(".nav-link.active").attr("id");
         }
 
-        $("#autoRefresh").hide();
+        $("#autoRefresh").addClass("invisible");
         $("#refresh").addClass("radius-right");
 
         function setAutoRefresh(refreshInterval, timer, callback) {
@@ -113,7 +113,7 @@ define(["jquery", "app/common", "stickytabs", "visibility",
             }
 
             $("#refresh").removeClass("radius-right");
-            $("#autoRefresh").show();
+            $("#autoRefresh").removeClass("invisible");
 
             countdown(refreshInterval);
             if (!refreshInterval) return;
@@ -126,9 +126,9 @@ define(["jquery", "app/common", "stickytabs", "visibility",
         }
 
         if (["#scan_nav", "#selectors_nav", "#disconnect"].indexOf(tab_id) !== -1) {
-            $("#refresh").hide();
+            $("#refresh").addClass("invisible");
         } else {
-            $("#refresh").show();
+            $("#refresh").removeClass("invisible");
         }
 
         switch (tab_id) {
@@ -168,12 +168,12 @@ define(["jquery", "app/common", "stickytabs", "visibility",
                 break;
             case "#configuration_nav":
                 require(["app/config"], (module) => {
-                    module.getActions(checked_server);
-                    module.getMaps(checked_server);
+                    module.getActions();
+                    module.getMaps();
                 });
                 break;
             case "#symbols_nav":
-                require(["app/symbols"], (module) => module.getSymbols(checked_server));
+                require(["app/symbols"], (module) => module.getSymbols());
                 break;
             case "#scan_nav":
                 require(["app/upload"]);
@@ -465,9 +465,9 @@ define(["jquery", "app/common", "stickytabs", "visibility",
         checked_server = this.value;
         $("#selSrv [value=\"" + checked_server + "\"]").prop("checked", true);
         if (checked_server === "All SERVERS") {
-            $("#learnServers").show();
+            $("#learnServers").removeClass("invisible");
         } else {
-            $("#learnServers").hide();
+            $("#learnServers").addClass("invisible");
         }
         tabClick("#" + $("#tablist > .nav-item > .nav-link.active").attr("id"));
     });
