@@ -24,11 +24,12 @@ local exports = {}
 local E = {}
 local N = "lua_redis"
 
+local db_schema = (ts.number / tostring + ts.string):is_optional():describe("Database number")
 local common_schema = {
-  timeout = (ts.number + ts.string / lutil.parse_time_interval):is_optional():describe("Connection timeout"),
-  db = ts.string:is_optional():describe("Database number"),
-  database = ts.string:is_optional():describe("Database number"),
-  dbname = ts.string:is_optional():describe("Database number"),
+  timeout = (ts.number + ts.string / lutil.parse_time_interval):is_optional():describe("Connection timeout (seconds)"),
+  db = db_schema,
+  database = db_schema,
+  dbname = db_schema,
   prefix = ts.string:is_optional():describe("Key prefix"),
   username = ts.string:is_optional():describe("Username"),
   password = ts.string:is_optional():describe("Password"),
@@ -39,6 +40,7 @@ local common_schema = {
   sentinel_master_maxerrors = (ts.number + ts.string / tonumber):is_optional():describe("Sentinel master max errors"),
   sentinel_username = ts.string:is_optional():describe("Sentinel username"),
   sentinel_password = ts.string:is_optional():describe("Sentinel password"),
+  redis_version = (ts.number + ts.string / tonumber):is_optional():describe("Redis server version (6 or 7)"),
 }
 
 local read_schema = lutil.table_merge({
@@ -64,7 +66,8 @@ local server_schema = lutil.table_merge({
 
 local enrich_schema = function(external)
   return ts.one_of {
-    ts.shape(external), -- no specific redis parameters
+    ts.shape(lutil.table_merge(common_schema,
+        external)), -- no specific redis servers (e.g when global settings are used)
     ts.shape(lutil.table_merge(read_schema, external)), -- read_servers specified
     ts.shape(lutil.table_merge(write_schema, external)), -- write_servers specified
     ts.shape(lutil.table_merge(rw_schema, external)), -- both read and write servers defined
@@ -355,6 +358,10 @@ local function process_redis_opts(options, redis_params)
     redis_params['prefix'] = options['prefix']
   end
 
+  if options['redis_version'] and not redis_params['redis_version'] then
+    redis_params['redis_version'] = tonumber(options['redis_version'])
+  end
+
   if type(options['expand_keys']) == 'boolean' then
     redis_params['expand_keys'] = options['expand_keys']
   else
@@ -1122,9 +1129,9 @@ local function redis_make_request_taskless(ev_base, cfg, redis_params, key,
 end
 
 --[[[
--- @function lua_redis.redis_make_request_taskless(ev_base, redis_params, key, is_write, callback, command, args)
+-- @function lua_redis.redis_make_request_taskless(ev_base, cfg, redis_params, key, is_write, callback, command, args)
 -- Sends a request to Redis in context where `task` is not available for some specific use-cases
--- Identical to redis_make_request() except in that first parameter is an `event base` object
+-- Identical to redis_make_request() except in that first parameter is an `event base` object and the second one is the 'config' object
 --]]
 
 exports.rspamd_redis_make_request_taskless = redis_make_request_taskless
@@ -1138,6 +1145,8 @@ local function script_set_loaded(script)
     script.loaded = true
   end
 
+  script.pending_upload = false -- Allow further reload
+
   local wait_table = {}
   for _, s in ipairs(script.waitq) do
     table.insert(wait_table, s)
@@ -1162,8 +1171,11 @@ local function prepare_redis_call(script)
   end
 
   -- Call load script on each server, set loaded flag
-  script.in_flight = #servers
-  for _, s in ipairs(servers) do
+  if not script.servers_ready then
+    script.servers_ready = { } -- possible values for each server: unsent, tempfail, failed, done
+  end
+
+  for idx, s in ipairs(servers) do
     local cur_opts = {
       host = s:get_addr(),
       timeout = script.redis_params['timeout'],
@@ -1172,6 +1184,11 @@ local function prepare_redis_call(script)
       upstream = s
     }
 
+    -- By default we start from unsent status
+    if not script.servers_ready[idx] then
+      script.servers_ready[idx] = "unsent"
+    end
+
     if script.redis_params['username'] then
       cur_opts['username'] = script.redis_params['username']
     end
@@ -1190,46 +1207,89 @@ local function prepare_redis_call(script)
   return options
 end
 
+local function is_any_server_ready(script)
+  for _, s in ipairs(script.servers_ready) do
+    if s == "done" then
+      return true
+    end
+  end
+  return false
+end
+
+local function is_all_servers_failed(script)
+  for _, s in ipairs(script.servers_ready) do
+    if s == "unsent" or s == "tempfail" or s == "done" then
+      return false
+    end
+  end
+
+  return true
+end
+
+local function script_description(script)
+  return script.filename and ("from file: " .. script.filename)
+      or ("with id: " .. script.id)
+end
+
 local function load_script_task(script, task, is_write)
   local rspamd_redis = require "rspamd_redis"
   local opts = prepare_redis_call(script)
 
-  for _, opt in ipairs(opts) do
-    opt.task = task
-    opt.is_write = is_write
-    opt.callback = function(err, data)
-      if err then
-        logger.errx(task, 'cannot upload script to %s: %s; registered from: %s:%s',
-            opt.upstream:get_addr():to_string(true),
-            err, script.caller.short_src, script.caller.currentline)
+  for idx, opt in ipairs(opts) do
+    if script.servers_ready[idx] ~= 'done' then
+      opt.task = task
+      opt.is_write = is_write
+      opt.callback = function(err, data)
+        if err then
+          if string.match(err, 'ERR') then
+            logger.errx(task, 'cannot upload script %s to %s: %s; registered from: %s:%s',
+                script_description(script),
+                opt.upstream:get_addr():to_string(true),
+                err, script.caller.short_src, script.caller.currentline)
+            opt.upstream:fail()
+            script.servers_ready[idx] = "failed"
+            return
+          else
+            -- Assume temporary error
+            logger.infox(task, 'temporary error uploading script %s to %s: %s; registered from: %s:%s',
+                script_description(script),
+                opt.upstream:get_addr():to_string(true),
+                err, script.caller.short_src, script.caller.currentline)
+            script.servers_ready[idx] = "tempfail"
+            return
+          end
+        else
+          opt.upstream:ok()
+          logger.infox(task,
+              "uploaded redis script to %s %s, sha: %s",
+              opt.upstream:get_addr():to_string(true),
+              script_description(script), data)
+          script.sha = data -- We assume that sha is the same on all servers
+          script.servers_ready[idx] = "done"
+        end
+        if is_any_server_ready(script) then
+          script_set_loaded(script)
+        elseif is_all_servers_failed(script) then
+          script.pending_upload = false
+          script.fatal_error = "cannot upload script to any server"
+        end
+      end -- callback
+
+      local ret = rspamd_redis.make_request(opt)
+
+      if not ret then
+        logger.errx('cannot execute redis request to load script on %s',
+            opt.upstream:get_addr())
+        script.servers_ready[idx] = "failed"
         opt.upstream:fail()
-        script.fatal_error = err
-      else
-        opt.upstream:ok()
-        logger.infox(task,
-            "uploaded redis script to %s with id %s, sha: %s",
-            opt.upstream:get_addr():to_string(true),
-            script.id, data)
-        script.sha = data -- We assume that sha is the same on all servers
-      end
-      script.in_flight = script.in_flight - 1
-
-      if script.in_flight == 0 then
-        script_set_loaded(script)
       end
     end
 
-    local ret = rspamd_redis.make_request(opt)
-
-    if not ret then
-      logger.errx('cannot execute redis request to load script on %s',
-          opt.upstream:get_addr())
-      script.in_flight = script.in_flight - 1
-      opt.upstream:fail()
-    end
-
-    if script.in_flight == 0 then
+    if is_any_server_ready(script) then
       script_set_loaded(script)
+    elseif is_all_servers_failed(script) then
+      script.pending_upload = false
+      script.fatal_error = "cannot upload script to any server"
     end
   end
 end
@@ -1238,42 +1298,62 @@ local function load_script_taskless(script, cfg, ev_base, is_write)
   local rspamd_redis = require "rspamd_redis"
   local opts = prepare_redis_call(script)
 
-  for _, opt in ipairs(opts) do
-    opt.config = cfg
-    opt.ev_base = ev_base
-    opt.is_write = is_write
-    opt.callback = function(err, data)
-      if err then
-        logger.errx(cfg, 'cannot upload script to %s: %s; registered from: %s:%s',
-            opt.upstream:get_addr():to_string(true),
-            err, script.caller.short_src, script.caller.currentline)
+  for idx, opt in ipairs(opts) do
+    if script.servers_ready[idx] ~= 'done' then
+      opt.config = cfg
+      opt.ev_base = ev_base
+      opt.is_write = is_write
+      opt.callback = function(err, data)
+        if err then
+          if string.match(err, 'ERR') then
+            logger.errx(cfg, 'cannot upload script %s to %s: %s; registered from: %s:%s',
+                script_description(script),
+                opt.upstream:get_addr():to_string(true),
+                err, script.caller.short_src, script.caller.currentline)
+            opt.upstream:fail()
+            script.servers_ready[idx] = "failed"
+          else
+            -- Assume temporary error
+            logger.infox(cfg, 'temporary error uploading script %s to %s: %s; registered from: %s:%s',
+                script_description(script),
+                opt.upstream:get_addr():to_string(true),
+                err, script.caller.short_src, script.caller.currentline)
+            script.servers_ready[idx] = "tempfail"
+          end
+        else
+          opt.upstream:ok()
+          logger.infox(cfg,
+              "uploaded redis script %s to %s, sha: %s",
+              script_description(script),
+              opt.upstream:get_addr():to_string(true),
+              data)
+          script.sha = data -- We assume that sha is the same on all servers
+          script.servers_ready[idx] = "done"
+        end
+
+        if is_any_server_ready(script) then
+          script_set_loaded(script)
+        elseif is_all_servers_failed(script) then
+          script.pending_upload = false
+          script.fatal_error = "cannot upload script to any server"
+        end
+      end
+      local ret = rspamd_redis.make_request(opt)
+
+      if not ret then
+        logger.errx('cannot execute redis request to load script %s on %s',
+            script_description(script),
+            opt.upstream:get_addr())
+        script.servers_ready[idx] = "failed"
         opt.upstream:fail()
-        script.fatal_error = err
-      else
-        opt.upstream:ok()
-        logger.infox(cfg,
-            "uploaded redis script to %s with id %s, sha: %s",
-            opt.upstream:get_addr():to_string(true), script.id, data)
-        script.sha = data -- We assume that sha is the same on all servers
-        script.fatal_error = nil
-      end
-      script.in_flight = script.in_flight - 1
-
-      if script.in_flight == 0 then
-        script_set_loaded(script)
       end
     end
-    local ret = rspamd_redis.make_request(opt)
 
-    if not ret then
-      logger.errx('cannot execute redis request to load script on %s',
-          opt.upstream:get_addr())
-      script.in_flight = script.in_flight - 1
-      opt.upstream:fail()
-    end
-
-    if script.in_flight == 0 then
+    if is_any_server_ready(script) then
       script_set_loaded(script)
+    elseif is_all_servers_failed(script) then
+      script.pending_upload = false
+      script.fatal_error = "cannot upload script " .. script_description(script) .. " to any server"
     end
   end
 end
@@ -1284,11 +1364,11 @@ local function load_redis_script(script, cfg, ev_base, _)
   end
 end
 
-local function add_redis_script(script, redis_params, caller_level)
+local function add_redis_script(script, redis_params, caller_level, maybe_filename)
   if not caller_level then
     caller_level = 2
   end
-  local caller = debug.getinfo(caller_level)
+  local caller = debug.getinfo(caller_level) or debug.getinfo(caller_level - 1) or E
 
   local new_script = {
     caller = caller,
@@ -1296,19 +1376,25 @@ local function add_redis_script(script, redis_params, caller_level)
     redis_params = redis_params,
     script = script,
     waitq = {}, -- callbacks pending for script being loaded
-    id = #redis_scripts + 1
+    id = #redis_scripts + 1,
+    filename = maybe_filename,
   }
 
   -- Register on load function
   rspamd_config:add_on_load(function(cfg, ev_base, worker)
     local mult = 0.0
     rspamd_config:add_periodic(ev_base, 0.0, function()
-      if not new_script.sha then
+      if not new_script.sha and not new_script.fatal_error then
         load_redis_script(new_script, cfg, ev_base, worker)
-        mult = mult + 1
+
+        -- Do not wait for more than 10 seconds to retry
+        if mult < 10 then
+          mult = mult + 1
+        end
         return 1.0 * mult -- Check one more time in one second
       end
 
+      -- All loaded, we are good to go
       return false
     end, false)
   end)
@@ -1333,25 +1419,26 @@ local function load_redis_script_from_file(filename, redis_params, dir)
   if not dir then
     dir = rspamd_paths.LUALIBDIR
   end
+  local path = filename
   if filename:sub(1, 1) ~= package.config:sub(1, 1) then
     -- Relative path
-    filename = lua_util.join_path(dir, "redis_scripts", filename)
+    path = lua_util.join_path(dir, "redis_scripts", filename)
   end
   -- Read file contents
-  local file = io.open(filename, "r")
+  local file = io.open(path, "r")
   if not file then
-    rspamd_logger.errx("failed to open Redis script file: %s", filename)
+    rspamd_logger.errx("failed to open Redis script file: %s", path)
     return nil
   end
   local script = file:read("*all")
   if not script then
-    rspamd_logger.errx("failed to load Redis script file: %s", filename)
+    rspamd_logger.errx("failed to load Redis script file: %s", path)
     return nil
   end
   file:close()
   script = lua_util.strip_lua_comments(script)
 
-  return add_redis_script(script, redis_params, 3)
+  return add_redis_script(script, redis_params, 3, filename)
 end
 
 exports.load_redis_script_from_file = load_redis_script_from_file
@@ -1381,17 +1468,30 @@ local function exec_redis_script(id, params, callback, keys, args)
       if not err then
         callback(err, data)
       elseif string.match(err, 'NOSCRIPT') then
-        -- Schedule restart
-        script.sha = nil
+        -- Schedule restart if possible
         if can_reload then
           table.insert(script.waitq, do_call)
-          if script.in_flight == 0 then
+          if not script.pending_upload then
+            logger.infox(params.task or rspamd_config,
+                'redis script %s is not loaded (NOSCRIPT returned), scheduling reload',
+                script_description(script))
+            script.sha = nil
+            script.loaded = nil
+            script.pending_upload = true
+	    -- We must initialize all servers as we don't know here which one failed
+	    for i, _ in ipairs(script.servers_ready) do
+	      script.servers_ready[i] = "unsent"
+	    end
             -- Reload scripts if this has not been initiated yet
             if params.task then
               load_script_task(script, params.task)
             else
               load_script_taskless(script, rspamd_config, params.ev_base)
             end
+          else
+            logger.infox(params.task or rspamd_config,
+                'redis script %s is not ready (NOSCRIPT returned), waiting it to be loaded',
+                script_description(script))
           end
         else
           callback(err, data)
@@ -1415,36 +1515,43 @@ local function exec_redis_script(id, params, callback, keys, args)
       end
     end
 
+    local redis_command = 'EVALSHA'
+    if not params.is_write and script.redis_params.redis_version and
+        script.redis_params.redis_version >= 7 then
+      redis_command = 'EVALSHA_RO'
+    end
     if params.task then
       if not rspamd_redis_make_request(params.task, script.redis_params,
-          params.key, params.is_write, redis_cb, 'EVALSHA', redis_args) then
+          params.key, params.is_write, redis_cb, redis_command, redis_args) then
         callback('Cannot make redis request', nil)
       end
     else
       if not redis_make_request_taskless(params.ev_base, rspamd_config,
           script.redis_params,
-          params.key, params.is_write, redis_cb, 'EVALSHA', redis_args) then
+          params.key, params.is_write, redis_cb, redis_command, redis_args) then
         callback('Cannot make redis request', nil)
       end
     end
   end
 
-  if script.loaded then
+  if script.loaded and script.sha then
     do_call(true)
   else
-    -- Delayed until scripts are loaded
-    if not params.task then
-      table.insert(script.waitq, do_call)
+    table.insert(script.waitq, do_call)
+    if not script.servers_ready then
+      -- Delayed until scripts are loaded
+      logger.infox(params.task or rspamd_config, 'redis script %s is not loaded, trying to load',
+          script_description(script))
+      -- Reload scripts if this has not been initiated yet
+      if params.task then
+        load_script_task(script, params.task)
+      else
+        load_script_taskless(script, rspamd_config, params.ev_base)
+      end
     else
-      -- TODO: fix taskfull requests
-      table.insert(script.waitq, function()
-        if script.loaded then
-          do_call(false)
-        else
-          callback('NOSCRIPT', nil)
-        end
-      end)
-      load_script_task(script, params.task, params.is_write)
+      -- Already loaded, but not yet ready
+      logger.infox(params.task or rspamd_config, 'redis script %s is not ready, waiting it to be loaded',
+          script_description(script))
     end
   end
 
@@ -1641,11 +1748,10 @@ exports.request = function(redis_params, attrs, req)
     opts.dbname = redis_params.db
   end
 
-  lutil.debugm(N, 'perform generic request to redis server' ..
-      ' (host=%s, timeout=%s): cmd: %s, arguments: %s', addr,
-      opts.timeout, opts.cmd, opts.args)
-
   if opts.callback then
+    lutil.debugm(N, 'perform generic async request to redis server' ..
+        ' (host=%s, timeout=%s): cmd: %s, arguments: %s', addr,
+        opts.timeout, opts.cmd, opts.args)
     local ret, conn = rspamd_redis.make_request(opts)
     if not ret then
       logger.errx(log_obj, 'cannot execute redis request')
@@ -1655,6 +1761,9 @@ exports.request = function(redis_params, attrs, req)
     return ret, conn, addr
   else
     -- Coroutines version
+    lutil.debugm(N, 'perform generic coroutine request to redis server' ..
+        ' (host=%s, timeout=%s): cmd: %s, arguments: %s', addr,
+        opts.timeout, opts.cmd, opts.args)
     local ret, conn = rspamd_redis.connect_sync(opts)
     if not ret then
       logger.errx(log_obj, 'cannot execute redis request')
