@@ -24,7 +24,7 @@
 #include "libutil/util.h"
 #include <unicode/utf8.h>
 
-KHASH_INIT(rspamd_mime_headers_htb, gchar *,
+KHASH_INIT(rspamd_mime_headers_htb, char *,
 		   struct rspamd_mime_header *, 1,
 		   rspamd_strcase_hash, rspamd_strcase_equal);
 
@@ -37,10 +37,10 @@ static void
 rspamd_mime_header_check_special(struct rspamd_task *task,
 								 struct rspamd_mime_header *rh)
 {
-	guint64 h;
-	const gchar *p, *end;
-	gchar *id;
-	gint max_recipients = -1, len;
+	uint64_t h;
+	const char *p, *end;
+	char *id;
+	int max_recipients = -1, len;
 
 	if (task->cfg) {
 		max_recipients = task->cfg->max_recipients;
@@ -93,7 +93,7 @@ rspamd_mime_header_check_special(struct rspamd_task *task,
 		}
 
 		if (end > p) {
-			gchar *d;
+			char *d;
 
 			if (*(end - 1) == '>') {
 				end--;
@@ -186,16 +186,16 @@ rspamd_mime_header_add(struct rspamd_task *task,
 void rspamd_mime_headers_process(struct rspamd_task *task,
 								 struct rspamd_mime_headers_table *target,
 								 struct rspamd_mime_header **order_ptr,
-								 const gchar *in, gsize len,
+								 const char *in, gsize len,
 								 gboolean check_newlines)
 {
 	struct rspamd_mime_header *nh = NULL;
-	const gchar *p, *c, *end;
-	gchar *tmp, *tp;
-	gint state = 0, l, next_state = 100, err_state = 100, t_state;
+	const char *p, *c, *end;
+	char *tmp, *tp;
+	int state = 0, l, next_state = 100, err_state = 100, t_state;
 	gboolean valid_folding = FALSE, shift_by_one = FALSE;
-	guint nlines_count[RSPAMD_TASK_NEWLINES_MAX];
-	guint norder = 0;
+	unsigned int nlines_count[RSPAMD_TASK_NEWLINES_MAX];
+	unsigned int norder = 0;
 
 	p = in;
 	end = p + len;
@@ -485,12 +485,12 @@ void rspamd_mime_headers_process(struct rspamd_task *task,
 	LL_REVERSE(*order_ptr);
 
 	if (check_newlines) {
-		guint max_cnt = 0;
-		gint sel = 0;
+		unsigned int max_cnt = 0;
+		int sel = 0;
 		rspamd_cryptobox_hash_state_t hs;
-		guchar hout[rspamd_cryptobox_HASHBYTES], *hexout;
+		unsigned char hout[rspamd_cryptobox_HASHBYTES], *hexout;
 
-		for (gint i = RSPAMD_TASK_NEWLINES_CR; i < RSPAMD_TASK_NEWLINES_MAX; i++) {
+		for (int i = RSPAMD_TASK_NEWLINES_CR; i < RSPAMD_TASK_NEWLINES_MAX; i++) {
 			if (nlines_count[i] > max_cnt) {
 				max_cnt = nlines_count[i];
 				sel = i;
@@ -572,7 +572,7 @@ static void
 rspamd_mime_header_sanity_check(GString *str)
 {
 	gsize i;
-	gchar t;
+	char t;
 
 	for (i = 0; i < str->len; i++) {
 		t = str->str[i];
@@ -588,20 +588,20 @@ rspamd_mime_header_sanity_check(GString *str)
 	}
 }
 
-gchar *
-rspamd_mime_header_decode(rspamd_mempool_t *pool, const gchar *in,
+char *
+rspamd_mime_header_decode(rspamd_mempool_t *pool, const char *in,
 						  gsize inlen, gboolean *invalid_utf)
 {
 	GString *out;
-	const guchar *c, *p, *end;
-	const gchar *tok_start = NULL;
+	const unsigned char *c, *p, *end;
+	const char *tok_start = NULL;
 	gsize tok_len = 0, pos;
 	GByteArray *token = NULL, *decoded;
 	rspamd_ftok_t cur_charset = {0, NULL}, old_charset = {0, NULL};
-	gint encoding;
+	int encoding;
 	gssize r;
-	guint qmarks = 0;
-	gchar *ret;
+	unsigned int qmarks = 0;
+	char *ret;
 	enum {
 		parse_normal = 0,
 		got_eqsign,
@@ -628,7 +628,7 @@ rspamd_mime_header_decode(rspamd_mempool_t *pool, const gchar *in,
 				state = got_eqsign;
 			}
 			else if (*p >= 128) {
-				gint off = 0;
+				int off = 0;
 				UChar32 uc;
 				/* Unencoded character */
 				g_string_append_len(out, c, p - c);
@@ -754,7 +754,7 @@ rspamd_mime_header_decode(rspamd_mempool_t *pool, const gchar *in,
 						state = parse_normal;
 					}
 				} /* qmarks >= 3 */
-			}     /* p == '=' */
+			} /* p == '=' */
 			else {
 				state = got_encoded_start;
 			}
@@ -815,102 +815,107 @@ rspamd_mime_header_decode(rspamd_mempool_t *pool, const gchar *in,
 	return ret;
 }
 
-gchar *
-rspamd_mime_header_encode(const gchar *in, gsize len)
+char *
+rspamd_mime_header_encode(const char *in, gsize len, bool is_structured)
 {
-	const gchar *p = in, *end = in + len;
-	gchar *out, encode_buf[80 * sizeof(guint32)];
-	GString *res;
-	gboolean need_encoding = FALSE;
+	static const size_t max_token_size = 76 - 12; /* 12 is the length of "=?UTF-8?Q??="; */
+	GString *outbuf = g_string_sized_new(len);
+	char *encode_buf = g_alloca(max_token_size + 3);
+	const char *p = in;
+	const char *end = in + len;
 
-	/* Check if we need to encode */
 	while (p < end) {
-		if ((((guchar) *p) & 0x80) != 0) {
-			need_encoding = TRUE;
-			break;
+		if (*p == ' ' || *p == '\r' || *p == '\n' || *p == '(' || *p == ')') {
+			/* Append the separator as is */
+			g_string_append_c(outbuf, *p);
+			p++;
 		}
-		p++;
-	}
+		else {
+			const char *q = end;
+			size_t piece_len = q - p, encoded_len = 0;
 
-	if (!need_encoding) {
-		out = g_malloc(len + 1);
-		rspamd_strlcpy(out, in, len + 1);
-	}
-	else {
-		/* Need encode */
-		gsize ulen, pos;
-		gint r;
-		const gchar *prev;
-		/* Choose step: =?UTF-8?Q?<qp>?= should be less than 76 chars */
-		guint step = (76 - 12) / 3 + 1;
+			/* Check if the piece contains non-ASCII characters */
+			gboolean need_encoding = FALSE;
+			size_t unencoded_prefix = 0, unencoded_suffix = 0;
+			for (size_t i = 0; i < piece_len; i++) {
+				unsigned char c = p[i];
+				if (c >= 128 || (is_structured && !g_ascii_isalnum(c))) {
+					need_encoding = TRUE;
+					unencoded_suffix = 0;
+					encoded_len += 3;
 
-		ulen = g_utf8_strlen(in, len);
-		res = g_string_sized_new(len * 2 + 1);
-		pos = 0;
-		prev = in;
-		/* Adjust chunk size for unicode average length */
-		step *= 1.0 * ulen / (gdouble) len;
-
-		while (pos < ulen) {
-			p = g_utf8_offset_to_pointer(in, pos);
-
-			if (p > prev) {
-				/* Encode and print */
-				r = rspamd_encode_qp2047_buf(prev, p - prev,
-											 encode_buf, sizeof(encode_buf));
-
-				if (r != -1) {
-					if (res->len > 0) {
-						rspamd_printf_gstring(res, " =?UTF-8?Q?%*s?=", r,
-											  encode_buf);
+					if (encoded_len > max_token_size) {
+						piece_len = i;
+						q = p + piece_len;
+						/* No more space */
+						break;
 					}
-					else {
-						rspamd_printf_gstring(res, "=?UTF-8?Q?%*s?=", r,
-											  encode_buf);
-					}
-				}
-			}
-
-			pos += MIN(step, ulen - pos);
-			prev = p;
-		}
-
-		/* Leftover */
-		if (prev < end) {
-			r = rspamd_encode_qp2047_buf(prev, end - prev,
-										 encode_buf, sizeof(encode_buf));
-
-			if (r != -1) {
-				if (res->len > 0) {
-					rspamd_printf_gstring(res, " =?UTF-8?Q?%*s?=", r,
-										  encode_buf);
 				}
 				else {
-					rspamd_printf_gstring(res, "=?UTF-8?Q?%*s?=", r,
-										  encode_buf);
+					encoded_len++;
+
+					if (encoded_len > max_token_size) {
+						piece_len = i;
+						q = p + piece_len;
+						/* No more space */
+						break;
+					}
+
+					if (need_encoding && (c == '(' || c == ')')) {
+						/* If we need to encode, we must stop on comments characters */
+						piece_len = i;
+						q = p + piece_len;
+						/* No more space */
+						break;
+					}
+
+					if (!need_encoding) {
+						unencoded_prefix++;
+					}
+					else {
+						unencoded_suffix++;
+					}
 				}
 			}
-		}
 
-		out = g_string_free(res, FALSE);
+			if (need_encoding) {
+				g_string_append_len(outbuf, p, unencoded_prefix);
+				p += unencoded_prefix;
+				g_string_append(outbuf, "=?UTF-8?Q?");
+				/* Do encode */
+				encoded_len = rspamd_encode_qp2047_buf(p, piece_len - unencoded_prefix - unencoded_suffix,
+													   encode_buf, max_token_size + 3);
+				p += piece_len - unencoded_prefix - unencoded_suffix;
+				g_string_append_len(outbuf, encode_buf, encoded_len);
+				g_string_append(outbuf, "?=");
+				g_string_append_len(outbuf, p, unencoded_suffix);
+			}
+			else {
+				/* No transformation */
+				g_string_append_len(outbuf, p, piece_len);
+			}
+			p = q;
+		}
 	}
 
-	return out;
+	/* return the allocated string and free the GString struct */
+	return g_string_free(outbuf, FALSE);
 }
 
-gchar *
-rspamd_mime_message_id_generate(const gchar *fqdn)
+
+char *
+rspamd_mime_message_id_generate(const char *fqdn)
 {
 	GString *out;
-	guint64 rnd, clk;
+	uint64_t rnd, clk;
 
 	out = g_string_sized_new(strlen(fqdn) + 22);
 	rnd = ottery_rand_uint64();
 	clk = rspamd_get_calendar_ticks() * 1e6;
 
 	rspamd_printf_gstring(out, "%*bs.%*bs@%s",
-						  (gint) sizeof(guint64) - 3, (guchar *) &clk,
-						  (gint) sizeof(guint64), (gchar *) &rnd,
+						  (int) sizeof(uint64_t) - 3, (unsigned char *) &clk,
+						  (int) sizeof(uint64_t), (char *) &rnd,
 						  fqdn);
 
 	return g_string_free(out, FALSE);
@@ -918,7 +923,7 @@ rspamd_mime_message_id_generate(const gchar *fqdn)
 
 struct rspamd_mime_header *
 rspamd_message_get_header_from_hash(struct rspamd_mime_headers_table *hdrs,
-									const gchar *field,
+									const char *field,
 									gboolean need_modified)
 {
 	if (hdrs == NULL) {
@@ -930,7 +935,7 @@ rspamd_message_get_header_from_hash(struct rspamd_mime_headers_table *hdrs,
 	struct rspamd_mime_header *hdr;
 
 	if (htb) {
-		k = kh_get(rspamd_mime_headers_htb, htb, (gchar *) field);
+		k = kh_get(rspamd_mime_headers_htb, htb, (char *) field);
 
 		if (k == kh_end(htb)) {
 			return NULL;
@@ -958,7 +963,7 @@ rspamd_message_get_header_from_hash(struct rspamd_mime_headers_table *hdrs,
 }
 
 struct rspamd_mime_header *
-rspamd_message_get_header_array(struct rspamd_task *task, const gchar *field,
+rspamd_message_get_header_array(struct rspamd_task *task, const char *field,
 								gboolean need_modified)
 {
 	return rspamd_message_get_header_from_hash(
@@ -978,7 +983,7 @@ gsize rspamd_mime_headers_count(struct rspamd_mime_headers_table *hdrs)
 bool rspamd_mime_headers_foreach(const struct rspamd_mime_headers_table *hdrs,
 								 rspamd_hdr_traverse_func_t func, void *ud)
 {
-	const gchar *name;
+	const char *name;
 	struct rspamd_mime_header *hdr;
 
 	kh_foreach(&hdrs->htb, name, hdr, {
@@ -1097,7 +1102,7 @@ gsize rspamd_message_header_unfold_inplace(char *hdr, gsize len)
 
 void rspamd_message_set_modified_header(struct rspamd_task *task,
 										struct rspamd_mime_headers_table *hdrs,
-										const gchar *hdr_name,
+										const char *hdr_name,
 										const ucl_object_t *obj,
 										struct rspamd_mime_header **order_ptr)
 {
@@ -1107,7 +1112,7 @@ void rspamd_message_set_modified_header(struct rspamd_task *task,
 	int i;
 
 	if (htb) {
-		k = kh_get(rspamd_mime_headers_htb, htb, (gchar *) hdr_name);
+		k = kh_get(rspamd_mime_headers_htb, htb, (char *) hdr_name);
 
 		if (k == kh_end(htb)) {
 			hdr_elt = rspamd_mempool_alloc0(task->task_pool, sizeof(*hdr_elt));
@@ -1353,7 +1358,7 @@ void rspamd_message_set_modified_header(struct rspamd_task *task,
 	}
 }
 
-gsize rspamd_strip_smtp_comments_inplace(gchar *input, gsize len)
+gsize rspamd_strip_smtp_comments_inplace(char *input, gsize len)
 {
 	enum parser_state {
 		parse_normal,
@@ -1363,8 +1368,8 @@ gsize rspamd_strip_smtp_comments_inplace(gchar *input, gsize len)
 		parse_quoted_ignore,
 	} state = parse_normal,
 	  next_state = parse_normal;
-	gchar *d = input, *end = input + len, *start = input;
-	gchar t;
+	char *d = input, *end = input + len, *start = input;
+	char t;
 	int obraces = 0, ebraces = 0;
 
 	while (input < end) {
