@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -35,7 +35,7 @@
 #include "cryptobox.h"
 #include "ssl_util.h"
 #include "contrib/libottery/ottery.h"
-#include "contrib/fastutf8/fastutf8.h"
+#include "rspamd_simdutf.h"
 
 #ifdef SYS_ZSTD
 #include "zstd.h"
@@ -57,7 +57,7 @@
 #ifdef HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>
 #endif
-#include <math.h>
+#include <cmath>
 #include "libserver/composites/composites.h"
 
 #include "blas-config.h"
@@ -65,10 +65,11 @@
 #include <string>
 #include <string_view>
 #include <vector>
-#include "fmt/core.h"
+#include "contrib/fmt/include/fmt/base.h"
 #include "cxx/util.hxx"
 #include "frozen/unordered_map.h"
 #include "frozen/string.h"
+#include "contrib/expected/expected.hpp"
 #include "contrib/ankerl/unordered_dense.h"
 
 #define DEFAULT_SCORE 10.0
@@ -101,14 +102,14 @@ struct rspamd_ucl_map_cbdata {
 	{
 	}
 };
-static gchar *rspamd_ucl_read_cb(gchar *chunk,
-								 gint len,
-								 struct map_cb_data *data,
-								 gboolean final);
+static char *rspamd_ucl_read_cb(char *chunk,
+								int len,
+								struct map_cb_data *data,
+								gboolean final);
 static void rspamd_ucl_fin_cb(struct map_cb_data *data, void **target);
 static void rspamd_ucl_dtor_cb(struct map_cb_data *data);
 
-guint rspamd_config_log_id = (guint) -1;
+unsigned int rspamd_config_log_id = (unsigned int) -1;
 RSPAMD_CONSTRUCTOR(rspamd_config_log_init)
 {
 	rspamd_config_log_id = rspamd_logger_add_debug_module("config");
@@ -135,14 +136,14 @@ struct rspamd_actions_list {
 	void sort()
 	{
 		std::sort(actions.begin(), actions.end(), [](const action_ptr &a1, const action_ptr &a2) -> bool {
-			if (!isnan(a1->threshold) && !isnan(a2->threshold)) {
+			if (!std::isnan(a1->threshold) && !std::isnan(a2->threshold)) {
 				return a1->threshold < a2->threshold;
 			}
 
-			if (isnan(a1->threshold) && isnan(a2->threshold)) {
+			if (std::isnan(a1->threshold) && std::isnan(a2->threshold)) {
 				return false;
 			}
-			else if (isnan(a1->threshold)) {
+			else if (std::isnan(a1->threshold)) {
 				return true;
 			}
 
@@ -162,10 +163,10 @@ struct rspamd_actions_list {
 gboolean
 rspamd_parse_bind_line(struct rspamd_config *cfg,
 					   struct rspamd_worker_conf *cf,
-					   const gchar *str)
+					   const char *str)
 {
 	struct rspamd_worker_bind_conf *cnf;
-	const gchar *fdname;
+	const char *fdname;
 	gboolean ret = TRUE;
 
 	if (str == nullptr) {
@@ -340,6 +341,7 @@ rspamd_config_new(enum rspamd_config_init_flags flags)
 	cfg->heartbeat_interval = 10.0;
 
 	cfg->enable_css_parser = true;
+	cfg->enable_mime_utf = false;
 	cfg->script_modules = g_ptr_array_new();
 
 	REF_INIT_RETAIN(cfg, rspamd_config_free);
@@ -437,8 +439,8 @@ void rspamd_config_free(struct rspamd_config *cfg)
 
 const ucl_object_t *
 rspamd_config_get_module_opt(struct rspamd_config *cfg,
-							 const gchar *module_name,
-							 const gchar *opt_name)
+							 const char *module_name,
+							 const char *opt_name)
 {
 	const ucl_object_t *res = nullptr, *sec;
 
@@ -450,9 +452,9 @@ rspamd_config_get_module_opt(struct rspamd_config *cfg,
 	return res;
 }
 
-gint rspamd_config_parse_flag(const gchar *str, guint len)
+int rspamd_config_parse_flag(const char *str, unsigned int len)
 {
-	gint c;
+	int c;
 
 	if (!str || !*str) {
 		return -1;
@@ -627,8 +629,8 @@ rspamd_config_process_var(struct rspamd_config *cfg, const rspamd_ftok_t *var,
 static gboolean
 rspamd_config_parse_log_format(struct rspamd_config *cfg)
 {
-	const gchar *p, *c, *end, *s;
-	gchar *d;
+	const char *p, *c, *end, *s;
+	char *d;
 	struct rspamd_log_format *lf = nullptr;
 	rspamd_ftok_t var, var_content;
 	enum {
@@ -637,7 +639,7 @@ rspamd_config_parse_log_format(struct rspamd_config *cfg)
 		parse_var_name,
 		parse_var_content,
 	} state = parse_str;
-	gint braces = 0;
+	int braces = 0;
 
 	g_assert(cfg != nullptr);
 	c = cfg->log_format_str;
@@ -826,8 +828,7 @@ gboolean
 rspamd_config_post_load(struct rspamd_config *cfg,
 						enum rspamd_post_load_options opts)
 {
-
-	auto ret = TRUE;
+	auto ret = tl::expected<void, std::string>{};
 
 	rspamd_adjust_clocks_resolution(cfg);
 	rspamd_logger_configure_modules(cfg->debug_modules);
@@ -867,14 +868,15 @@ rspamd_config_post_load(struct rspamd_config *cfg,
 			else {
 				if (opts & RSPAMD_CONFIG_INIT_VALIDATE) {
 					msg_err_config("no url_tld option has been specified");
-					ret = FALSE;
+					ret = tl::make_unexpected(std::string{"no url_tld option has been specified"});
 				}
 			}
 		}
 		else {
 			if (access(cfg->tld_file, R_OK) == -1) {
 				if (opts & RSPAMD_CONFIG_INIT_VALIDATE) {
-					ret = FALSE;
+					ret = tl::make_unexpected(fmt::format("cannot access tld file {}: {}",
+														  cfg->tld_file, strerror(errno)));
 					msg_err_config("cannot access tld file %s: %s", cfg->tld_file,
 								   strerror(errno));
 				}
@@ -905,13 +907,17 @@ rspamd_config_post_load(struct rspamd_config *cfg,
 	if (!rspamd_config_parse_log_format(cfg)) {
 		msg_err_config("cannot parse log format, task logging will not be available");
 		if (opts & RSPAMD_CONFIG_INIT_VALIDATE) {
-			ret = FALSE;
+			ret = tl::make_unexpected(std::string{"cannot parse log format"});
 		}
 	}
 
 	if (opts & RSPAMD_CONFIG_INIT_SYMCACHE) {
 		/* Init config cache */
-		ret = rspamd_symcache_init(cfg->cache) && ret;
+		auto symcache_ret = rspamd_symcache_init(cfg->cache);
+
+		if (!symcache_ret) {
+			ret = tl::make_unexpected(std::string{"symcache init failed"});
+		}
 
 		/* Init re cache */
 		rspamd_re_cache_init(cfg->re_cache, cfg);
@@ -928,9 +934,9 @@ rspamd_config_post_load(struct rspamd_config *cfg,
 
 	if (opts & RSPAMD_CONFIG_INIT_LIBS) {
 		/* Config other libraries */
-		ret = rspamd_config_libs(cfg->libs_ctx, cfg) && ret;
+		auto libs_ret = rspamd_config_libs(cfg->libs_ctx, cfg);
 
-		if (!ret) {
+		if (!libs_ret) {
 			msg_err_config("cannot configure libraries, fatal error");
 			return FALSE;
 		}
@@ -959,7 +965,11 @@ rspamd_config_post_load(struct rspamd_config *cfg,
 							" Rspamd features will be broken");
 		}
 
-		ret = rspamd_symcache_validate(cfg->cache, cfg, FALSE) && ret;
+		auto val_ret = rspamd_symcache_validate(cfg->cache, cfg, FALSE);
+
+		if (!val_ret) {
+			ret = tl::make_unexpected(std::string{"symcache validation failed"});
+		}
 	}
 
 	if (opts & RSPAMD_CONFIG_INIT_POST_LOAD_LUA) {
@@ -970,7 +980,14 @@ rspamd_config_post_load(struct rspamd_config *cfg,
 		rspamd_map_preload(cfg);
 	}
 
-	return ret;
+	if (ret) {
+		return true;
+	}
+	else {
+		msg_err_config("error on post-init stage: %s", ret.error().c_str());
+
+		return false;
+	}
 }
 
 struct rspamd_classifier_config *
@@ -1026,7 +1043,7 @@ void rspamd_config_init_metric(struct rspamd_config *cfg)
 }
 
 struct rspamd_symbols_group *
-rspamd_config_new_group(struct rspamd_config *cfg, const gchar *name)
+rspamd_config_new_group(struct rspamd_config *cfg, const char *name)
 {
 	struct rspamd_symbols_group *gr;
 
@@ -1036,6 +1053,8 @@ rspamd_config_new_group(struct rspamd_config *cfg, const gchar *name)
 	rspamd_mempool_add_destructor(cfg->cfg_pool,
 								  (rspamd_mempool_destruct_t) g_hash_table_unref, gr->symbols);
 	gr->name = rspamd_mempool_strdup(cfg->cfg_pool, name);
+	gr->max_score = NAN;
+	gr->min_score = NAN;
 
 	if (strcmp(gr->name, "ungrouped") == 0) {
 		gr->flags |= RSPAMD_SYMBOL_GROUP_UNGROUPED;
@@ -1093,7 +1112,7 @@ rspamd_config_new_worker(struct rspamd_config *cfg,
 
 
 static bool
-rspamd_include_map_handler(const guchar *data, gsize len,
+rspamd_include_map_handler(const unsigned char *data, gsize len,
 						   const ucl_object_t *args, void *ud)
 {
 	auto *cfg = (struct rspamd_config *) ud;
@@ -1236,7 +1255,7 @@ void rspamd_config_insert_classify_symbols(struct rspamd_config *cfg)
 }
 
 struct rspamd_classifier_config *
-rspamd_config_find_classifier(struct rspamd_config *cfg, const gchar *name)
+rspamd_config_find_classifier(struct rspamd_config *cfg, const char *name)
 {
 	if (name == nullptr) {
 		return nullptr;
@@ -1312,9 +1331,9 @@ rspamd_config_check_statfiles(struct rspamd_classifier_config *cf)
 	return res;
 }
 
-static gchar *
-rspamd_ucl_read_cb(gchar *chunk,
-				   gint len,
+static char *
+rspamd_ucl_read_cb(char *chunk,
+				   int len,
 				   struct map_cb_data *data,
 				   gboolean final)
 {
@@ -1344,7 +1363,7 @@ rspamd_ucl_fin_cb(struct map_cb_data *data, void **target)
 	}
 
 	/* New data available */
-	auto *parser = ucl_parser_new(0);
+	auto *parser = ucl_parser_new(UCL_PARSER_SAFE_FLAGS);
 	if (!ucl_parser_add_chunk(parser, (unsigned char *) cbdata->buf.data(),
 							  cbdata->buf.size())) {
 		msg_err_config("cannot parse map %s: %s",
@@ -1388,7 +1407,7 @@ rspamd_check_module(struct rspamd_config *cfg, module_t *mod)
 	if (mod != nullptr) {
 		if (mod->module_version != RSPAMD_CUR_MODULE_VERSION) {
 			msg_err_config("module %s has incorrect version %xd (%xd expected)",
-						   mod->name, (gint) mod->module_version, RSPAMD_CUR_MODULE_VERSION);
+						   mod->name, (int) mod->module_version, RSPAMD_CUR_MODULE_VERSION);
 			ret = FALSE;
 		}
 		if (ret && mod->rspamd_version != RSPAMD_VERSION_NUM) {
@@ -1443,7 +1462,7 @@ rspamd_init_filters(struct rspamd_config *cfg, bool reconfig, bool strict)
 {
 	GList *cur;
 	module_t *mod, **pmod;
-	guint i = 0;
+	unsigned int i = 0;
 	struct module_ctx *mod_ctx, *cur_ctx;
 	gboolean ret = TRUE;
 
@@ -1470,7 +1489,7 @@ rspamd_init_filters(struct rspamd_config *cfg, bool reconfig, bool strict)
 		PTR_ARRAY_FOREACH(cfg->c_modules, i, cur_ctx)
 		{
 			if (g_ascii_strcasecmp(cur_ctx->mod->name,
-								   (const gchar *) cur->data) == 0) {
+								   (const char *) cur->data) == 0) {
 				mod_ctx = cur_ctx;
 				break;
 			}
@@ -1513,9 +1532,9 @@ rspamd_init_filters(struct rspamd_config *cfg, bool reconfig, bool strict)
 }
 
 static void
-rspamd_config_new_symbol(struct rspamd_config *cfg, const gchar *symbol,
-						 gdouble score, const gchar *description, const gchar *group,
-						 guint flags, guint priority, gint nshots)
+rspamd_config_new_symbol(struct rspamd_config *cfg, const char *symbol,
+						 double score, const char *description, const char *group,
+						 unsigned int flags, unsigned int priority, int nshots)
 {
 	struct rspamd_symbols_group *sym_group;
 	struct rspamd_symbol *sym_def;
@@ -1525,7 +1544,7 @@ rspamd_config_new_symbol(struct rspamd_config *cfg, const gchar *symbol,
 		rspamd_mempool_alloc0_type(cfg->cfg_pool, struct rspamd_symbol);
 	score_ptr = rspamd_mempool_alloc_type(cfg->cfg_pool, double);
 
-	if (isnan(score)) {
+	if (std::isnan(score)) {
 		/* In fact, it could be defined later */
 		msg_debug_config("score is not defined for symbol %s, set it to zero",
 						 symbol);
@@ -1585,17 +1604,17 @@ rspamd_config_new_symbol(struct rspamd_config *cfg, const gchar *symbol,
 
 gboolean
 rspamd_config_add_symbol(struct rspamd_config *cfg,
-						 const gchar *symbol,
-						 gdouble score,
-						 const gchar *description,
-						 const gchar *group,
-						 guint flags,
-						 guint priority,
-						 gint nshots)
+						 const char *symbol,
+						 double score,
+						 const char *description,
+						 const char *group,
+						 unsigned int flags,
+						 unsigned int priority,
+						 int nshots)
 {
 	struct rspamd_symbol *sym_def;
 	struct rspamd_symbols_group *sym_group;
-	guint i;
+	unsigned int i;
 
 	g_assert(cfg != nullptr);
 	g_assert(symbol != nullptr);
@@ -1636,7 +1655,7 @@ rspamd_config_add_symbol(struct rspamd_config *cfg,
 		}
 
 		if (sym_def->priority > priority &&
-			(isnan(score) || !(sym_def->flags & RSPAMD_SYMBOL_FLAG_UNSCORED))) {
+			(std::isnan(score) || !(sym_def->flags & RSPAMD_SYMBOL_FLAG_UNSCORED))) {
 			msg_debug_config("symbol %s has been already registered with "
 							 "priority %ud, do not override (new priority: %ud)",
 							 symbol,
@@ -1657,7 +1676,7 @@ rspamd_config_add_symbol(struct rspamd_config *cfg,
 		}
 		else {
 
-			if (!isnan(score)) {
+			if (!std::isnan(score)) {
 				msg_debug_config("symbol %s has been already registered with "
 								 "priority %ud, override it with new priority: %ud, "
 								 "old score: %.2f, new score: %.2f",
@@ -1724,12 +1743,12 @@ rspamd_config_add_symbol(struct rspamd_config *cfg,
 
 gboolean
 rspamd_config_add_symbol_group(struct rspamd_config *cfg,
-							   const gchar *symbol,
-							   const gchar *group)
+							   const char *symbol,
+							   const char *group)
 {
 	struct rspamd_symbol *sym_def;
 	struct rspamd_symbols_group *sym_group;
-	guint i;
+	unsigned int i;
 
 	g_assert(cfg != nullptr);
 	g_assert(symbol != nullptr);
@@ -1787,7 +1806,7 @@ rspamd_config_is_enabled_from_ucl(rspamd_mempool_t *pool,
 			return ucl_object_toboolean(enabled);
 		}
 		else if (ucl_object_type(enabled) == UCL_STRING) {
-			gint ret = rspamd_config_parse_flag(ucl_object_tostring(enabled), 0);
+			int ret = rspamd_config_parse_flag(ucl_object_tostring(enabled), 0);
 
 			if (ret == 0) {
 				return FALSE;
@@ -1811,7 +1830,7 @@ rspamd_config_is_enabled_from_ucl(rspamd_mempool_t *pool,
 			return !ucl_object_toboolean(disabled);
 		}
 		else if (ucl_object_type(disabled) == UCL_STRING) {
-			gint ret = rspamd_config_parse_flag(ucl_object_tostring(disabled), 0);
+			int ret = rspamd_config_parse_flag(ucl_object_tostring(disabled), 0);
 
 			if (ret == 0) {
 				return TRUE;
@@ -1831,7 +1850,7 @@ rspamd_config_is_enabled_from_ucl(rspamd_mempool_t *pool,
 
 gboolean
 rspamd_config_is_module_enabled(struct rspamd_config *cfg,
-								const gchar *module_name)
+								const char *module_name)
 {
 	gboolean is_c = FALSE, enabled;
 	const ucl_object_t *conf;
@@ -1839,7 +1858,7 @@ rspamd_config_is_module_enabled(struct rspamd_config *cfg,
 	struct rspamd_symbols_group *gr;
 	lua_State *L = RSPAMD_LUA_CFG_STATE(cfg);
 	struct module_ctx *cur_ctx;
-	guint i;
+	unsigned int i;
 
 	PTR_ARRAY_FOREACH(cfg->c_modules, i, cur_ctx)
 	{
@@ -1931,7 +1950,7 @@ static gboolean
 rspamd_config_action_from_ucl(struct rspamd_config *cfg,
 							  struct rspamd_action *act,
 							  const ucl_object_t *obj,
-							  guint priority)
+							  unsigned int priority)
 {
 	auto threshold = NAN;
 	int flags = 0;
@@ -1955,7 +1974,7 @@ rspamd_config_action_from_ucl(struct rspamd_config *cfg,
 
 			while ((cur = ucl_object_iterate(elt, &it, true)) != nullptr) {
 				if (ucl_object_type(cur) == UCL_STRING) {
-					const gchar *fl_str = ucl_object_tostring(cur);
+					const char *fl_str = ucl_object_tostring(cur);
 
 					if (g_ascii_strcasecmp(fl_str, "no_threshold") == 0) {
 						flags |= RSPAMD_ACTION_NO_THRESHOLD;
@@ -1976,7 +1995,7 @@ rspamd_config_action_from_ucl(struct rspamd_config *cfg,
 		elt = ucl_object_lookup(obj, "milter");
 
 		if (elt) {
-			const gchar *milter_action = ucl_object_tostring(elt);
+			const char *milter_action = ucl_object_tostring(elt);
 
 			if (strcmp(milter_action, "discard") == 0) {
 				flags |= RSPAMD_ACTION_MILTER;
@@ -1997,7 +2016,7 @@ rspamd_config_action_from_ucl(struct rspamd_config *cfg,
 
 	/* TODO: add lua references support */
 
-	if (isnan(threshold) && !(flags & RSPAMD_ACTION_NO_THRESHOLD)) {
+	if (std::isnan(threshold) && !(flags & RSPAMD_ACTION_NO_THRESHOLD)) {
 		msg_err_config("action %s has no threshold being set and it is not"
 					   " a no threshold action",
 					   act->name);
@@ -2024,12 +2043,12 @@ rspamd_config_action_from_ucl(struct rspamd_config *cfg,
 
 gboolean
 rspamd_config_set_action_score(struct rspamd_config *cfg,
-							   const gchar *action_name,
+							   const char *action_name,
 							   const ucl_object_t *obj)
 {
 	enum rspamd_action_type std_act;
 	const ucl_object_t *elt;
-	guint priority = ucl_object_get_priority(obj), obj_type;
+	unsigned int priority = ucl_object_get_priority(obj), obj_type;
 
 	g_assert(cfg != nullptr);
 	g_assert(action_name != nullptr);
@@ -2107,8 +2126,8 @@ rspamd_config_set_action_score(struct rspamd_config *cfg,
 
 gboolean
 rspamd_config_maybe_disable_action(struct rspamd_config *cfg,
-								   const gchar *action_name,
-								   guint priority)
+								   const char *action_name,
+								   unsigned int priority)
 {
 	auto actions = RSPAMD_CFG_ACTIONS(cfg);
 	auto maybe_act = rspamd::find_map(actions->actions_by_name, action_name);
@@ -2140,7 +2159,7 @@ rspamd_config_maybe_disable_action(struct rspamd_config *cfg,
 }
 
 struct rspamd_action *
-rspamd_config_get_action(struct rspamd_config *cfg, const gchar *name)
+rspamd_config_get_action(struct rspamd_config *cfg, const char *name)
 {
 	auto actions = RSPAMD_CFG_ACTIONS(cfg);
 	auto maybe_act = rspamd::find_map(actions->actions_by_name, name);
@@ -2189,14 +2208,14 @@ gsize rspamd_config_actions_size(struct rspamd_config *cfg)
 }
 
 gboolean
-rspamd_config_radix_from_ucl(struct rspamd_config *cfg, const ucl_object_t *obj, const gchar *description,
+rspamd_config_radix_from_ucl(struct rspamd_config *cfg, const ucl_object_t *obj, const char *description,
 							 struct rspamd_radix_map_helper **target, GError **err,
-							 struct rspamd_worker *worker, const gchar *map_name)
+							 struct rspamd_worker *worker, const char *map_name)
 {
 	ucl_type_t type;
 	ucl_object_iter_t it = nullptr;
 	const ucl_object_t *cur, *cur_elt;
-	const gchar *str;
+	const char *str;
 
 	/* Cleanup */
 	*target = nullptr;
@@ -2317,7 +2336,7 @@ constexpr const auto action_types = frozen::make_unordered_map<frozen::string, e
 });
 
 gboolean
-rspamd_action_from_str(const gchar *data, enum rspamd_action_type *result)
+rspamd_action_from_str(const char *data, enum rspamd_action_type *result)
 {
 	auto maybe_action = rspamd::find_map(action_types, std::string_view{data});
 
@@ -2330,7 +2349,7 @@ rspamd_action_from_str(const gchar *data, enum rspamd_action_type *result)
 	}
 }
 
-const gchar *
+const char *
 rspamd_action_to_str(enum rspamd_action_type action)
 {
 	switch (action) {
@@ -2359,7 +2378,7 @@ rspamd_action_to_str(enum rspamd_action_type action)
 	return "unknown action";
 }
 
-const gchar *
+const char *
 rspamd_action_to_str_alt(enum rspamd_action_type action)
 {
 	switch (action) {
@@ -2399,20 +2418,20 @@ rspamd_config_settings_elt_dtor(struct rspamd_config_settings_elt *e)
 	}
 }
 
-guint32
-rspamd_config_name_to_id(const gchar *name, gsize namelen)
+uint32_t
+rspamd_config_name_to_id(const char *name, gsize namelen)
 {
-	guint64 h;
+	uint64_t h;
 
 	h = rspamd_cryptobox_fast_hash_specific(RSPAMD_CRYPTOBOX_XXHASH64,
 											name, namelen, 0x0);
 	/* Take the lower part of hash as LE number */
-	return ((guint32) GUINT64_TO_LE(h));
+	return ((uint32_t) GUINT64_TO_LE(h));
 }
 
 struct rspamd_config_settings_elt *
 rspamd_config_find_settings_id_ref(struct rspamd_config *cfg,
-								   guint32 id)
+								   uint32_t id)
 {
 	struct rspamd_config_settings_elt *cur;
 
@@ -2429,9 +2448,9 @@ rspamd_config_find_settings_id_ref(struct rspamd_config *cfg,
 
 struct rspamd_config_settings_elt *rspamd_config_find_settings_name_ref(
 	struct rspamd_config *cfg,
-	const gchar *name, gsize namelen)
+	const char *name, gsize namelen)
 {
-	guint32 id;
+	uint32_t id;
 
 	id = rspamd_config_name_to_id(name, namelen);
 
@@ -2439,13 +2458,13 @@ struct rspamd_config_settings_elt *rspamd_config_find_settings_name_ref(
 }
 
 void rspamd_config_register_settings_id(struct rspamd_config *cfg,
-										const gchar *name,
+										const char *name,
 										ucl_object_t *symbols_enabled,
 										ucl_object_t *symbols_disabled,
 										enum rspamd_config_settings_policy policy)
 {
 	struct rspamd_config_settings_elt *elt;
-	guint32 id;
+	uint32_t id;
 
 	id = rspamd_config_name_to_id(name, strlen(name));
 	elt = rspamd_config_find_settings_id_ref(cfg, id);
@@ -2561,7 +2580,7 @@ int rspamd_config_ev_backend_get(struct rspamd_config *cfg)
 	return AUTO_BACKEND;
 }
 
-const gchar *
+const char *
 rspamd_config_ev_backend_to_string(int ev_backend, gboolean *effective)
 {
 #define SET_EFFECTIVE(b)                              \
@@ -2638,15 +2657,7 @@ rspamd_init_libs(void)
 #endif
 
 	/* Configure utf8 library */
-	guint utf8_flags = 0;
-
-	if ((ctx->crypto_ctx->cpu_config & CPUID_SSE41)) {
-		utf8_flags |= RSPAMD_FAST_UTF8_FLAG_SSE41;
-	}
-	if ((ctx->crypto_ctx->cpu_config & CPUID_AVX2)) {
-		utf8_flags |= RSPAMD_FAST_UTF8_FLAG_AVX2;
-	}
-
+	unsigned int utf8_flags = 0;
 	rspamd_fast_utf8_library_init(utf8_flags);
 
 #ifdef HAVE_LOCALE_H
