@@ -23,7 +23,7 @@
 
 #include "contrib/libev/ev.h"
 
-#ifdef  __cplusplus
+#ifdef __cplusplus
 extern "C" {
 #endif
 
@@ -33,14 +33,14 @@ struct rspamd_http_message;
 struct upstream_ctx;
 
 struct rspamd_http_context_cfg {
-	guint kp_cache_size_client;
-	guint kp_cache_size_server;
-	guint ssl_cache_size;
-	gdouble keepalive_interval;
-	gdouble client_key_rotate_time;
-	const gchar *user_agent;
-	const gchar *http_proxy;
-	const gchar *server_hdr;
+	unsigned int kp_cache_size_client;
+	unsigned int kp_cache_size_server;
+	unsigned int ssl_cache_size;
+	double keepalive_interval;
+	double client_key_rotate_time;
+	const char *user_agent;
+	const char *http_proxy;
+	const char *server_hdr;
 };
 
 /**
@@ -49,22 +49,22 @@ struct rspamd_http_context_cfg {
  * @param ev_base event base
  * @return new context used for both client and server HTTP connections
  */
-struct rspamd_http_context *rspamd_http_context_create (struct rspamd_config *cfg,
-														struct ev_loop *ev_base,
-														struct upstream_ctx *ctx);
+struct rspamd_http_context *rspamd_http_context_create(struct rspamd_config *cfg,
+													   struct ev_loop *ev_base,
+													   struct upstream_ctx *ctx);
 
-struct rspamd_http_context *rspamd_http_context_create_config (
-		struct rspamd_http_context_cfg *cfg,
-		struct ev_loop *ev_base,
-		struct upstream_ctx *ctx);
+struct rspamd_http_context *rspamd_http_context_create_config(
+	struct rspamd_http_context_cfg *cfg,
+	struct ev_loop *ev_base,
+	struct upstream_ctx *ctx);
 
 /**
  * Destroys context
  * @param ctx
  */
-void rspamd_http_context_free (struct rspamd_http_context *ctx);
+void rspamd_http_context_free(struct rspamd_http_context *ctx);
 
-struct rspamd_http_context *rspamd_http_context_default (void);
+struct rspamd_http_context *rspamd_http_context_default(void);
 
 /**
  * Returns preserved keepalive connection if it's available.
@@ -74,10 +74,10 @@ struct rspamd_http_context *rspamd_http_context_default (void);
  * @param host
  * @return
  */
-struct rspamd_http_connection * rspamd_http_context_check_keepalive(struct rspamd_http_context *ctx,
-		const rspamd_inet_addr_t *addr,
-		const gchar *host,
-		bool is_ssl);
+struct rspamd_http_connection *rspamd_http_context_check_keepalive(struct rspamd_http_context *ctx,
+																   const rspamd_inet_addr_t *addr,
+																   const char *host,
+																   bool is_ssl);
 
 /**
  * Checks if there is a valid keepalive connection
@@ -88,9 +88,9 @@ struct rspamd_http_connection * rspamd_http_context_check_keepalive(struct rspam
  * @return
  */
 const rspamd_inet_addr_t *rspamd_http_context_has_keepalive(struct rspamd_http_context *ctx,
-									   const gchar *host,
-									   unsigned port,
-									   bool is_ssl);
+															const char *host,
+															unsigned port,
+															bool is_ssl);
 
 /**
  * Prepares keepalive key for a connection by creating a new entry or by reusing existent
@@ -101,7 +101,7 @@ const rspamd_inet_addr_t *rspamd_http_context_has_keepalive(struct rspamd_http_c
  * @param host
  */
 void rspamd_http_context_prepare_keepalive(struct rspamd_http_context *ctx, struct rspamd_http_connection *conn,
-										   const rspamd_inet_addr_t *addr, const gchar *host, bool is_ssl);
+										   const rspamd_inet_addr_t *addr, const char *host, bool is_ssl);
 
 /**
  * Pushes a connection to keepalive pool after client request is finished,
@@ -110,12 +110,12 @@ void rspamd_http_context_prepare_keepalive(struct rspamd_http_context *ctx, stru
  * @param conn
  * @param msg
  */
-void rspamd_http_context_push_keepalive (struct rspamd_http_context *ctx,
-										 struct rspamd_http_connection *conn,
-										 struct rspamd_http_message *msg,
-										 struct ev_loop *ev_base);
+void rspamd_http_context_push_keepalive(struct rspamd_http_context *ctx,
+										struct rspamd_http_connection *conn,
+										struct rspamd_http_message *msg,
+										struct ev_loop *ev_base);
 
-#ifdef  __cplusplus
+#ifdef __cplusplus
 }
 #endif
 
