@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -65,31 +65,32 @@ static void rspamd_map_process_periodic(struct map_periodic_cbdata *cbd);
 static void rspamd_map_schedule_periodic(struct rspamd_map *map, int how);
 static gboolean read_map_file_chunks(struct rspamd_map *map,
 									 struct map_cb_data *cbdata,
-									 const gchar *fname,
+									 const char *fname,
 									 gsize len,
 									 goffset off);
 static gboolean rspamd_map_save_http_cached_file(struct rspamd_map *map,
 												 struct rspamd_map_backend *bk,
 												 struct http_map_data *htdata,
-												 const guchar *data,
+												 const unsigned char *data,
 												 gsize len);
 static gboolean rspamd_map_update_http_cached_file(struct rspamd_map *map,
 												   struct rspamd_map_backend *bk,
 												   struct http_map_data *htdata);
 
-guint rspamd_map_log_id = (guint) -1;
+unsigned int rspamd_map_log_id = (unsigned int) -1;
 RSPAMD_CONSTRUCTOR(rspamd_map_log_init)
 {
 	rspamd_map_log_id = rspamd_logger_add_debug_module("map");
 }
 
 /**
- * Write HTTP request
+ * Write HTTP request with proper cache validation headers
+ * Uses ETags (If-None-Match) and Last-Modified (If-Modified-Since) for conditional requests
  */
 static void
 write_http_request(struct http_callback_data *cbd)
 {
-	gchar datebuf[128];
+	char datebuf[128];
 	struct rspamd_http_message *msg;
 
 	msg = rspamd_http_new_message(HTTP_REQUEST);
@@ -109,7 +110,8 @@ write_http_request(struct http_callback_data *cbd)
 		}
 		if (cbd->data->etag) {
 			rspamd_http_message_add_header_len(msg, "If-None-Match",
-											   cbd->data->etag->str, cbd->data->etag->len);
+											   cbd->data->etag->str,
+											   cbd->data->etag->len);
 		}
 	}
 
@@ -153,7 +155,7 @@ free_http_cbdata_common(struct http_callback_data *cbd, gboolean plan_new)
 
 	if (cbd->addrs) {
 		rspamd_inet_addr_t *addr;
-		guint i;
+		unsigned int i;
 
 		PTR_ARRAY_FOREACH(cbd->addrs, i, addr)
 		{
@@ -295,6 +297,101 @@ rspamd_map_cache_cb(struct ev_loop *loop, ev_timer *w, int revents)
 	}
 }
 
+/**
+ * Calculate next check time with proper priority for different cache validation mechanisms
+ * Priority: ETags > Last-Modified > Cache expiration headers
+ * @param now current time
+ * @param expires time from cache expiration header
+ * @param map_check_interval base polling interval
+ * @param has_etag whether we have ETag for conditional requests
+ * @param has_last_modified whether we have Last-Modified for conditional requests
+ * @return next check time
+ */
+static inline time_t
+rspamd_http_map_process_next_check(struct rspamd_map *map,
+								   struct rspamd_map_backend *bk,
+								   time_t now,
+								   time_t expires,
+								   time_t map_check_interval,
+								   gboolean has_etag,
+								   gboolean has_last_modified)
+{
+	static const time_t interval_mult = 4; /* Reduced from 16 to be more responsive */
+	static const time_t min_respectful_interval = 5;
+	time_t next_check;
+	time_t effective_interval = map_check_interval;
+
+	/*
+	 * Priority order for cache validation:
+	 * 1. ETags (most reliable)
+	 * 2. Last-Modified dates
+	 * 3. Cache expiration headers (least reliable)
+	 */
+
+	if (has_etag || has_last_modified) {
+		/*
+		 * If we have ETags or Last-Modified, we can use conditional requests
+		 * to avoid unnecessary downloads. However, we still need to be respectful
+		 * to servers and not DoS them with overly aggressive polling.
+		 */
+		if (map_check_interval < min_respectful_interval) {
+			/*
+			 * User configured very aggressive polling, but server provides cache validation.
+			 * Enforce minimum respectful interval to avoid DoS'ing the server.
+			 */
+			effective_interval = min_respectful_interval * interval_mult;
+			msg_info_map("map polling interval %d too aggressive with server cache support for %s, "
+						 "using %d seconds minimum",
+						 (int) map_check_interval, bk->uri, (int) effective_interval);
+		}
+
+		if (expires > now && (expires - now) <= effective_interval * interval_mult) {
+			/* Use expires header if it's reasonable (within interval_mult x poll interval) */
+			next_check = expires;
+		}
+		else {
+			/* Use effective interval, don't extend too much */
+			next_check = now + effective_interval;
+		}
+	}
+	else if (expires > now) {
+		/*
+		 * No ETags or Last-Modified available, rely on cache expiration.
+		 * But still cap the interval to avoid too long delays.
+		 * No need for respectful interval protection here since no conditional requests.
+		 */
+		if (expires - now > map_check_interval * interval_mult) {
+			next_check = now + map_check_interval * interval_mult;
+		}
+		else {
+			next_check = expires;
+		}
+	}
+	else {
+		/* No valid cache information, check immediately */
+		next_check = now;
+	}
+
+	return next_check;
+}
+
+/**
+ * Calculate respectful polling interval to avoid DoS'ing servers with cache validation
+ * @param map_check_interval user configured interval
+ * @return effective interval that respects server resources
+ */
+static inline time_t
+rspamd_map_get_respectful_interval(time_t map_check_interval)
+{
+	static const time_t min_respectful_interval = 5; /* Minimum 5 seconds to be respectful */
+	static const time_t interval_mult = 4;           /* Multiplier for respectful minimum */
+
+	if (map_check_interval < min_respectful_interval) {
+		return min_respectful_interval * interval_mult;
+	}
+	return map_check_interval;
+}
+
 static int
 http_map_finish(struct rspamd_http_connection *conn,
 				struct rspamd_http_message *msg)
@@ -306,7 +403,7 @@ http_map_finish(struct rspamd_http_connection *conn,
 	struct rspamd_http_map_cached_cbdata *cache_cbd;
 	const rspamd_ftok_t *expires_hdr, *etag_hdr;
 	char next_check_date[128];
-	guchar *in = NULL;
+	unsigned char *in = NULL;
 	gsize dlen = 0;
 
 	map = cbd->map;
@@ -316,12 +413,15 @@ http_map_finish(struct rspamd_http_connection *conn,
 	if (msg->code == 200) {
 
 		if (cbd->check) {
-			msg_info_map("need to reread map from %s", cbd->bk->uri);
+			msg_info_map("need to reread map from %s (reply code 200); "
+						 "date timestamp: %z, last modified: %z",
+						 cbd->bk->uri, (size_t) msg->date, (size_t) msg->last_modified);
 			cbd->periodic->need_modify = TRUE;
 			/* Reset the whole chain */
 			cbd->periodic->cur_backend = 0;
 			/* Reset cache, old cached data will be cleaned on timeout */
 			g_atomic_int_set(&data->cache->available, 0);
+			g_atomic_int_set(&map->shared->loaded, 0);
 			data->cur_cache_cbd = NULL;
 
 			rspamd_map_process_periodic(cbd->periodic);
@@ -330,6 +430,7 @@ http_map_finish(struct rspamd_http_connection *conn,
 			return 0;
 		}
 
+		/* This code is executed when we are actually reading a map */
 		cbd->data->last_checked = msg->date;
 
 		if (msg->last_modified) {
@@ -360,10 +461,11 @@ http_map_finish(struct rspamd_http_connection *conn,
 			goto err;
 		}
 
-		/* Check for expires */
+		/* Check for expires + etag */
 		double cached_timeout = map->poll_timeout * 2;
 
 		expires_hdr = rspamd_http_message_find_header(msg, "Expires");
+		etag_hdr = rspamd_http_message_find_header(msg, "ETag");
 
 		if (expires_hdr) {
 			time_t hdate;
@@ -371,19 +473,27 @@ http_map_finish(struct rspamd_http_connection *conn,
 			hdate = rspamd_http_parse_date(expires_hdr->begin, expires_hdr->len);
 
 			if (hdate != (time_t) -1 && hdate > msg->date) {
-				cached_timeout = map->next_check - msg->date +
-								 map->poll_timeout * 2;
-
-				map->next_check = hdate;
+				map->next_check = rspamd_http_map_process_next_check(map, bk, msg->date, hdate,
+																	 (time_t) map->poll_timeout,
+																	 etag_hdr != NULL,
+																	 msg->last_modified != 0);
+				cached_timeout = map->next_check - msg->date;
 			}
 			else {
 				msg_info_map("invalid expires header: %T, ignore it", expires_hdr);
 				map->next_check = 0;
 			}
 		}
-
-		/* Check for etag */
-		etag_hdr = rspamd_http_message_find_header(msg, "ETag");
+		else if (etag_hdr != NULL || msg->last_modified != 0) {
+			/* No expires header, but we have ETag or Last-Modified - use respectful interval */
+			time_t effective_interval = rspamd_map_get_respectful_interval(map->poll_timeout);
+			if (effective_interval != map->poll_timeout) {
+				msg_info_map("map polling interval %d too aggressive with server cache support, "
+							 "using %d seconds minimum",
+							 (int) map->poll_timeout, (int) effective_interval);
+			}
+			map->next_check = msg->date + effective_interval;
+		}
 
 		if (etag_hdr) {
 			if (cbd->data->etag) {
@@ -404,10 +514,7 @@ http_map_finish(struct rspamd_http_connection *conn,
 
 		MAP_RETAIN(cbd->shmem_data, "shmem_data");
 		cbd->data->gen++;
-		/*
-		 * We know that a map is in the locked state
-		 */
-		g_atomic_int_set(&data->cache->available, 1);
+
 		/* Store cached data */
 		rspamd_strlcpy(data->cache->shmem_name, cbd->shmem_data->shm_name,
 					   sizeof(data->cache->shmem_name));
@@ -444,7 +551,7 @@ http_map_finish(struct rspamd_http_connection *conn,
 			ZSTD_DStream *zstream;
 			ZSTD_inBuffer zin;
 			ZSTD_outBuffer zout;
-			guchar *out;
+			unsigned char *out;
 			gsize outlen, r;
 
 			zstream = ZSTD_createDStream();
@@ -509,6 +616,12 @@ http_map_finish(struct rspamd_http_connection *conn,
 
 		cbd->periodic->cur_backend++;
 		munmap(in, dlen);
+
+		/* Announce for other processes */
+		g_atomic_int_set(&data->cache->available, 1);
+		g_atomic_int_set(&map->shared->loaded, 1);
+		g_atomic_int_set(&map->shared->cached, 1);
+
 		rspamd_map_process_periodic(cbd->periodic);
 	}
 	else if (msg->code == 304 && cbd->check) {
@@ -522,19 +635,34 @@ http_map_finish(struct rspamd_http_connection *conn,
 		}
 
 		expires_hdr = rspamd_http_message_find_header(msg, "Expires");
+		bool has_expires = (expires_hdr != NULL);
 
 		if (expires_hdr) {
 			time_t hdate;
 
 			hdate = rspamd_http_parse_date(expires_hdr->begin, expires_hdr->len);
 			if (hdate != (time_t) -1 && hdate > msg->date) {
-				map->next_check = hdate;
+				map->next_check = rspamd_http_map_process_next_check(map, bk, msg->date, hdate,
+																	 (time_t) map->poll_timeout,
+																	 cbd->data->etag != NULL,
+																	 msg->last_modified != 0);
 			}
 			else {
 				msg_info_map("invalid expires header: %T, ignore it", expires_hdr);
 				map->next_check = 0;
+				has_expires = false;
 			}
 		}
+		else if (cbd->data->etag != NULL || msg->last_modified != 0) {
+			/* No expires header, but we have ETag or Last-Modified - use respectful interval */
+			time_t effective_interval = rspamd_map_get_respectful_interval(map->poll_timeout);
+			if (effective_interval != map->poll_timeout) {
+				msg_info_map("map polling interval %d too aggressive with server cache support, "
+							 "using %d seconds minimum",
+							 (int) map->poll_timeout, (int) effective_interval);
+			}
+			map->next_check = msg->date + effective_interval;
+		}
 
 		etag_hdr = rspamd_http_message_find_header(msg, "ETag");
 
@@ -547,19 +675,24 @@ http_map_finish(struct rspamd_http_connection *conn,
 			}
 		}
 
-		if (map->next_check) {
+		if (has_expires) {
 			rspamd_http_date_format(next_check_date, sizeof(next_check_date),
 									map->next_check);
-			msg_info_map("data is not modified for server %s, next check at %s "
+			msg_info_map("data is not modified for server %s (%s), next check at %s "
 						 "(http cache based: %T)",
-						 cbd->data->host, next_check_date, expires_hdr);
+						 cbd->data->host,
+						 bk->uri,
+						 next_check_date,
+						 expires_hdr);
 		}
 		else {
 			rspamd_http_date_format(next_check_date, sizeof(next_check_date),
-									rspamd_get_calendar_ticks() + map->poll_timeout);
-			msg_info_map("data is not modified for server %s, next check at %s "
+									map->next_check);
+			msg_info_map("data is not modified for server %s (%s), next check at %s "
 						 "(timer based)",
-						 cbd->data->host, next_check_date);
+						 cbd->data->host,
+						 bk->uri,
+						 next_check_date);
 		}
 
 		rspamd_map_update_http_cached_file(map, bk, cbd->data);
@@ -585,12 +718,12 @@ err:
 
 static gboolean
 read_map_file_chunks(struct rspamd_map *map, struct map_cb_data *cbdata,
-					 const gchar *fname, gsize len, goffset off)
+					 const char *fname, gsize len, goffset off)
 {
-	gint fd;
+	int fd;
 	gssize r, avail;
 	gsize buflen = 1024 * 1024;
-	gchar *pos, *bytes;
+	char *pos, *bytes;
 
 	fd = rspamd_file_xopen(fname, O_RDONLY, 0, TRUE);
 
@@ -602,7 +735,7 @@ read_map_file_chunks(struct rspamd_map *map, struct map_cb_data *cbdata,
 
 	if (lseek(fd, off, SEEK_SET) == -1) {
 		msg_err_map("can't seek in map to pos %d for buffered reading %s: %s",
-					(gint) off, fname, strerror(errno));
+					(int) off, fname, strerror(errno));
 		close(fd);
 
 		return FALSE;
@@ -614,13 +747,13 @@ read_map_file_chunks(struct rspamd_map *map, struct map_cb_data *cbdata,
 	pos = bytes;
 
 	while ((r = read(fd, pos, avail)) > 0) {
-		gchar *end = bytes + (pos - bytes) + r;
+		char *end = bytes + (pos - bytes) + r;
 		msg_debug_map("%s: read map chunk, %z bytes", fname,
 					  r);
 		pos = map->read_callback(bytes, end - bytes, cbdata, r == len);
 
 		if (pos && pos > bytes && pos < end) {
-			guint remain = end - pos;
+			unsigned int remain = end - pos;
 
 			memmove(bytes, pos, remain);
 			pos = bytes + remain;
@@ -660,24 +793,24 @@ read_map_file_chunks(struct rspamd_map *map, struct map_cb_data *cbdata,
 }
 
 static gboolean
-rspamd_map_check_sig_pk_mem(const guchar *sig,
+rspamd_map_check_sig_pk_mem(const unsigned char *sig,
 							gsize siglen,
 							struct rspamd_map *map,
-							const guchar *input,
+							const unsigned char *input,
 							gsize inlen,
 							struct rspamd_cryptobox_pubkey *pk)
 {
 	GString *b32_key;
 	gboolean ret = TRUE;
 
-	if (siglen != rspamd_cryptobox_signature_bytes(RSPAMD_CRYPTOBOX_MODE_25519)) {
+	if (siglen != crypto_sign_bytes()) {
 		msg_err_map("can't open signature for %s: invalid size: %z", map->name, siglen);
 
 		ret = FALSE;
 	}
 
 	if (ret && !rspamd_cryptobox_verify(sig, siglen, input, inlen,
-										rspamd_pubkey_get_pk(pk, NULL), RSPAMD_CRYPTOBOX_MODE_25519)) {
+										rspamd_pubkey_get_pk(pk, NULL))) {
 		msg_err_map("can't verify signature for %s: incorrect signature", map->name);
 
 		ret = FALSE;
@@ -685,7 +818,8 @@ rspamd_map_check_sig_pk_mem(const guchar *sig,
 
 	if (ret) {
 		b32_key = rspamd_pubkey_print(pk,
-									  RSPAMD_KEYPAIR_BASE32 | RSPAMD_KEYPAIR_PUBKEY);
+									  RSPAMD_KEYPAIR_ENCODING_DEFAULT,
+									  RSPAMD_KEYPAIR_PUBKEY);
 		msg_info_map("verified signature for %s using trusted key %v",
 					 map->name, b32_key);
 		g_string_free(b32_key, TRUE);
@@ -698,15 +832,15 @@ static gboolean
 rspamd_map_check_file_sig(const char *fname,
 						  struct rspamd_map *map,
 						  struct rspamd_map_backend *bk,
-						  const guchar *input,
+						  const unsigned char *input,
 						  gsize inlen)
 {
-	guchar *data;
+	unsigned char *data;
 	struct rspamd_cryptobox_pubkey *pk = NULL;
 	GString *b32_key;
 	gboolean ret = TRUE;
 	gsize len = 0;
-	gchar fpath[PATH_MAX];
+	char fpath[PATH_MAX];
 
 	if (bk->trusted_pubkey == NULL) {
 		/* Try to load and check pubkey */
@@ -718,8 +852,7 @@ rspamd_map_check_file_sig(const char *fname,
 			return FALSE;
 		}
 
-		pk = rspamd_pubkey_from_base32(data, len, RSPAMD_KEYPAIR_SIGN,
-									   RSPAMD_CRYPTOBOX_MODE_25519);
+		pk = rspamd_pubkey_from_base32(data, len, RSPAMD_KEYPAIR_SIGN);
 		munmap(data, len);
 
 		if (pk == NULL) {
@@ -729,7 +862,8 @@ rspamd_map_check_file_sig(const char *fname,
 
 		/* We just check pk against the trusted db of keys */
 		b32_key = rspamd_pubkey_print(pk,
-									  RSPAMD_KEYPAIR_BASE32 | RSPAMD_KEYPAIR_PUBKEY);
+									  RSPAMD_KEYPAIR_ENCODING_DEFAULT,
+									  RSPAMD_KEYPAIR_PUBKEY);
 		g_assert(b32_key != NULL);
 
 		if (g_hash_table_lookup(map->cfg->trusted_keys, b32_key->str) == NULL) {
@@ -772,7 +906,7 @@ static gboolean
 read_map_file(struct rspamd_map *map, struct file_map_data *data,
 			  struct rspamd_map_backend *bk, struct map_periodic_cbdata *periodic)
 {
-	gchar *bytes;
+	char *bytes;
 	gsize len;
 	struct stat st;
 
@@ -836,7 +970,7 @@ read_map_file(struct rspamd_map *map, struct file_map_data *data,
 				ZSTD_DStream *zstream;
 				ZSTD_inBuffer zin;
 				ZSTD_outBuffer zout;
-				guchar *out;
+				unsigned char *out;
 				gsize outlen, r;
 
 				zstream = ZSTD_createDStream();
@@ -901,6 +1035,8 @@ read_map_file(struct rspamd_map *map, struct file_map_data *data,
 		map->read_callback(NULL, 0, &periodic->cbdata, TRUE);
 	}
 
+	g_atomic_int_set(&map->shared->loaded, 1);
+
 	return TRUE;
 }
 
@@ -908,7 +1044,7 @@ static gboolean
 read_map_static(struct rspamd_map *map, struct static_map_data *data,
 				struct rspamd_map_backend *bk, struct map_periodic_cbdata *periodic)
 {
-	guchar *bytes;
+	unsigned char *bytes;
 	gsize len;
 
 	if (map->read_callback == NULL || map->fin_callback == NULL) {
@@ -925,7 +1061,7 @@ read_map_static(struct rspamd_map *map, struct static_map_data *data,
 			ZSTD_DStream *zstream;
 			ZSTD_inBuffer zin;
 			ZSTD_outBuffer zout;
-			guchar *out;
+			unsigned char *out;
 			gsize outlen, r;
 
 			zstream = ZSTD_createDStream();
@@ -985,6 +1121,7 @@ read_map_static(struct rspamd_map *map, struct static_map_data *data,
 	}
 
 	data->processed = TRUE;
+	g_atomic_int_set(&map->shared->loaded, 1);
 
 	return TRUE;
 }
@@ -992,9 +1129,7 @@ read_map_static(struct rspamd_map *map, struct static_map_data *data,
 static void
 rspamd_map_periodic_dtor(struct map_periodic_cbdata *periodic)
 {
-	struct rspamd_map *map;
-
-	map = periodic->map;
+	struct rspamd_map *map = periodic->map;
 	msg_debug_map("periodic dtor %p; need_modify=%d", periodic, periodic->need_modify);
 
 	if (periodic->need_modify || periodic->cbdata.errored) {
@@ -1009,18 +1144,13 @@ rspamd_map_periodic_dtor(struct map_periodic_cbdata *periodic)
 		/* Not modified */
 	}
 
-	if (periodic->locked) {
-		g_atomic_int_set(periodic->map->locked, 0);
-		msg_debug_map("unlocked map %s", periodic->map->name);
-
-		if (periodic->map->wrk->state == rspamd_worker_state_running) {
-			rspamd_map_schedule_periodic(periodic->map,
-										 RSPAMD_SYMBOL_RESULT_NORMAL);
-		}
-		else {
-			msg_debug_map("stop scheduling periodics for %s; terminating state",
-						  periodic->map->name);
-		}
+	if (periodic->map->wrk->state == rspamd_worker_state_running) {
+		rspamd_map_schedule_periodic(periodic->map,
+									 RSPAMD_MAP_SCHEDULE_NORMAL);
+	}
+	else {
+		msg_debug_map("stop scheduling periodics for %s; terminating state",
+					  periodic->map->name);
 	}
 
 	g_free(periodic);
@@ -1041,11 +1171,11 @@ rspamd_map_periodic_callback(struct ev_loop *loop, ev_timer *w, int revents)
 static void
 rspamd_map_schedule_periodic(struct rspamd_map *map, int how)
 {
-	const gdouble error_mult = 20.0, lock_mult = 0.1;
-	static const gdouble min_timer_interval = 2.0;
-	const gchar *reason = "unknown reason";
-	gdouble jittered_sec;
-	gdouble timeout;
+	const double error_mult = 20.0, lock_mult = 0.1;
+	static const double min_timer_interval = 2.0;
+	const char *reason = "unknown reason";
+	double jittered_sec;
+	double timeout;
 	struct map_periodic_cbdata *cbd;
 
 	if (map->scheduled_check || (map->wrk &&
@@ -1070,7 +1200,7 @@ rspamd_map_schedule_periodic(struct rspamd_map *map, int how)
 
 		if (timeout > 0 && timeout < map->poll_timeout) {
 			/* Early check case, jitter */
-			gdouble poll_timeout = map->poll_timeout;
+			double poll_timeout = map->poll_timeout;
 
 			if (how & RSPAMD_MAP_SCHEDULE_ERROR) {
 				poll_timeout = map->poll_timeout * error_mult;
@@ -1172,7 +1302,7 @@ rspamd_map_schedule_periodic(struct rspamd_map *map, int how)
 				  cbd, jittered_sec, map->name, reason);
 }
 
-static gint
+static int
 rspamd_map_af_to_weight(const rspamd_inet_addr_t *addr)
 {
 	int ret;
@@ -1192,12 +1322,12 @@ rspamd_map_af_to_weight(const rspamd_inet_addr_t *addr)
 	return ret;
 }
 
-static gint
+static int
 rspamd_map_dns_address_sort_func(gconstpointer a, gconstpointer b)
 {
 	const rspamd_inet_addr_t *ip1 = *(const rspamd_inet_addr_t **) a,
 							 *ip2 = *(const rspamd_inet_addr_t **) b;
-	gint w1, w2;
+	int w1, w2;
 
 	w1 = rspamd_map_af_to_weight(ip1);
 	w2 = rspamd_map_af_to_weight(ip2);
@@ -1212,7 +1342,7 @@ rspamd_map_dns_callback(struct rdns_reply *reply, void *arg)
 	struct http_callback_data *cbd = arg;
 	struct rdns_reply_entry *cur_rep;
 	struct rspamd_map *map;
-	guint flags = RSPAMD_HTTP_CLIENT_SIMPLE | RSPAMD_HTTP_CLIENT_SHARED;
+	unsigned int flags = RSPAMD_HTTP_CLIENT_SIMPLE | RSPAMD_HTTP_CLIENT_SHARED;
 
 	map = cbd->map;
 
@@ -1264,7 +1394,7 @@ rspamd_map_dns_callback(struct rdns_reply *reply, void *arg)
 
 	if (cbd->stage == http_map_http_conn && cbd->addrs->len > 0) {
 		rspamd_ptr_array_shuffle(cbd->addrs);
-		gint idx = 0;
+		int idx = 0;
 		/*
 		 * For the existing addr we can just select any address as we have
 		 * data available
@@ -1328,7 +1458,7 @@ rspamd_map_dns_callback(struct rdns_reply *reply, void *arg)
 
 static gboolean
 rspamd_map_read_cached(struct rspamd_map *map, struct rspamd_map_backend *bk,
-					   struct map_periodic_cbdata *periodic, const gchar *host)
+					   struct map_periodic_cbdata *periodic, const char *host)
 {
 	gsize mmap_len, len;
 	gpointer in;
@@ -1366,7 +1496,7 @@ rspamd_map_read_cached(struct rspamd_map *map, struct rspamd_map_backend *bk,
 		ZSTD_DStream *zstream;
 		ZSTD_inBuffer zin;
 		ZSTD_outBuffer zout;
-		guchar *out;
+		unsigned char *out;
 		gsize outlen, r;
 
 		zstream = ZSTD_createDStream();
@@ -1429,8 +1559,8 @@ static gboolean
 rspamd_map_has_http_cached_file(struct rspamd_map *map,
 								struct rspamd_map_backend *bk)
 {
-	gchar path[PATH_MAX];
-	guchar digest[rspamd_cryptobox_HASHBYTES];
+	char path[PATH_MAX];
+	unsigned char digest[rspamd_cryptobox_HASHBYTES];
 	struct rspamd_config *cfg = map->cfg;
 	struct stat st;
 
@@ -1454,13 +1584,13 @@ static gboolean
 rspamd_map_save_http_cached_file(struct rspamd_map *map,
 								 struct rspamd_map_backend *bk,
 								 struct http_map_data *htdata,
-								 const guchar *data,
+								 const unsigned char *data,
 								 gsize len)
 {
-	gchar path[PATH_MAX];
-	guchar digest[rspamd_cryptobox_HASHBYTES];
+	char path[PATH_MAX], temp_path[PATH_MAX];
+	unsigned char digest[rspamd_cryptobox_HASHBYTES];
 	struct rspamd_config *cfg = map->cfg;
-	gint fd;
+	int fd;
 	struct rspamd_http_file_data header;
 
 	if (cfg->maps_cache_dir == NULL || cfg->maps_cache_dir[0] == '\0') {
@@ -1470,8 +1600,10 @@ rspamd_map_save_http_cached_file(struct rspamd_map *map,
 	rspamd_cryptobox_hash(digest, bk->uri, strlen(bk->uri), NULL, 0);
 	rspamd_snprintf(path, sizeof(path), "%s%c%*xs.map", cfg->maps_cache_dir,
 					G_DIR_SEPARATOR, 20, digest);
+	rspamd_snprintf(temp_path, sizeof(temp_path), "%s.tmp.%d.%d", path,
+					(int) getpid(), (int) rspamd_get_calendar_ticks());
 
-	fd = rspamd_file_xopen(path, O_WRONLY | O_TRUNC | O_CREAT,
+	fd = rspamd_file_xopen(temp_path, O_WRONLY | O_TRUNC | O_CREAT,
 						   00600, FALSE);
 
 	if (fd == -1) {
@@ -1479,8 +1611,9 @@ rspamd_map_save_http_cached_file(struct rspamd_map *map,
 	}
 
 	if (!rspamd_file_lock(fd, FALSE)) {
-		msg_err_map("cannot lock file %s: %s", path, strerror(errno));
+		msg_err_map("cannot lock file %s: %s", temp_path, strerror(errno));
 		close(fd);
+		unlink(temp_path);
 
 		return FALSE;
 	}
@@ -1499,9 +1632,10 @@ rspamd_map_save_http_cached_file(struct rspamd_map *map,
 	}
 
 	if (write(fd, &header, sizeof(header)) != sizeof(header)) {
-		msg_err_map("cannot write file %s (header stage): %s", path, strerror(errno));
+		msg_err_map("cannot write file %s (header stage): %s", temp_path, strerror(errno));
 		rspamd_file_unlock(fd, FALSE);
 		close(fd);
+		unlink(temp_path);
 
 		return FALSE;
 	}
@@ -1509,9 +1643,10 @@ rspamd_map_save_http_cached_file(struct rspamd_map *map,
 	if (header.etag_len > 0) {
 		if (write(fd, RSPAMD_FSTRING_DATA(htdata->etag), header.etag_len) !=
 			header.etag_len) {
-			msg_err_map("cannot write file %s (etag stage): %s", path, strerror(errno));
+			msg_err_map("cannot write file %s (etag stage): %s", temp_path, strerror(errno));
 			rspamd_file_unlock(fd, FALSE);
 			close(fd);
+			unlink(temp_path);
 
 			return FALSE;
 		}
@@ -1519,9 +1654,10 @@ rspamd_map_save_http_cached_file(struct rspamd_map *map,
 
 	/* Now write the rest */
 	if (write(fd, data, len) != len) {
-		msg_err_map("cannot write file %s (data stage): %s", path, strerror(errno));
+		msg_err_map("cannot write file %s (data stage): %s", temp_path, strerror(errno));
 		rspamd_file_unlock(fd, FALSE);
 		close(fd);
+		unlink(temp_path);
 
 		return FALSE;
 	}
@@ -1529,6 +1665,13 @@ rspamd_map_save_http_cached_file(struct rspamd_map *map,
 	rspamd_file_unlock(fd, FALSE);
 	close(fd);
 
+	/* Atomically move temp file to final location */
+	if (rename(temp_path, path) != 0) {
+		msg_err_map("cannot rename %s to %s: %s", temp_path, path, strerror(errno));
+		unlink(temp_path);
+		return FALSE;
+	}
+
 	msg_info_map("saved data from %s in %s, %uz bytes", bk->uri, path, len + sizeof(header) + header.etag_len);
 
 	return TRUE;
@@ -1539,10 +1682,10 @@ rspamd_map_update_http_cached_file(struct rspamd_map *map,
 								   struct rspamd_map_backend *bk,
 								   struct http_map_data *htdata)
 {
-	gchar path[PATH_MAX];
-	guchar digest[rspamd_cryptobox_HASHBYTES];
+	char path[PATH_MAX];
+	unsigned char digest[rspamd_cryptobox_HASHBYTES];
 	struct rspamd_config *cfg = map->cfg;
-	gint fd;
+	int fd;
 	struct rspamd_http_file_data header;
 
 	if (!rspamd_map_has_http_cached_file(map, bk)) {
@@ -1612,10 +1755,10 @@ rspamd_map_read_http_cached_file(struct rspamd_map *map,
 								 struct http_map_data *htdata,
 								 struct map_cb_data *cbdata)
 {
-	gchar path[PATH_MAX];
-	guchar digest[rspamd_cryptobox_HASHBYTES];
+	char path[PATH_MAX];
+	unsigned char digest[rspamd_cryptobox_HASHBYTES];
 	struct rspamd_config *cfg = map->cfg;
-	gint fd;
+	int fd;
 	struct stat st;
 	struct rspamd_http_file_data header;
 
@@ -1662,7 +1805,11 @@ rspamd_map_read_http_cached_file(struct rspamd_map *map,
 	double now = rspamd_get_calendar_ticks();
 
 	if (header.next_check > now) {
-		map->next_check = header.next_check;
+		/* We assume that we have this data inside the cached file */
+		map->next_check = rspamd_http_map_process_next_check(map, bk, now, header.next_check,
+															 map->poll_timeout,
+															 header.etag_len > 0,
+															 true);
 	}
 	else {
 		map->next_check = now;
@@ -1707,8 +1854,10 @@ rspamd_map_read_http_cached_file(struct rspamd_map *map,
 	}
 
 	struct tm tm;
-	gchar ncheck_buf[32], lm_buf[32];
+	char ncheck_buf[32], lm_buf[32];
 
+	g_atomic_int_set(&map->shared->loaded, 1);
+	g_atomic_int_set(&map->shared->cached, 1);
 	rspamd_localtime(map->next_check, &tm);
 	strftime(ncheck_buf, sizeof(ncheck_buf) - 1, "%Y-%m-%d %H:%M:%S", &tm);
 	rspamd_localtime(htdata->last_modified, &tm);
@@ -1737,7 +1886,7 @@ rspamd_map_common_http_callback(struct rspamd_map *map,
 {
 	struct http_map_data *data;
 	struct http_callback_data *cbd;
-	guint flags = RSPAMD_HTTP_CLIENT_SIMPLE | RSPAMD_HTTP_CLIENT_SHARED;
+	unsigned int flags = RSPAMD_HTTP_CLIENT_SIMPLE | RSPAMD_HTTP_CLIENT_SHARED;
 
 	data = bk->data.hd;
 
@@ -1751,7 +1900,6 @@ rspamd_map_common_http_callback(struct rspamd_map *map,
 							 (int) data->last_modified,
 							 (int) data->cache->last_modified);
 				periodic->need_modify = TRUE;
-				/* Reset the whole chain */
 				periodic->cur_backend = 0;
 				rspamd_map_process_periodic(periodic);
 			}
@@ -1846,7 +1994,7 @@ check:
 	}
 	else if (map->r->r) {
 		/* Send both A and AAAA requests */
-		guint nreq = 0;
+		unsigned int nreq = 0;
 
 		if (rdns_make_request_full(map->r->r, rspamd_map_dns_callback, cbd,
 								   map->cfg->dns_timeout, map->cfg->dns_retransmits, 1,
@@ -2009,33 +2157,22 @@ rspamd_map_process_periodic(struct map_periodic_cbdata *cbd)
 	map = cbd->map;
 	map->scheduled_check = NULL;
 
-	if (!map->file_only && !cbd->locked) {
-		if (!g_atomic_int_compare_and_exchange(cbd->map->locked,
-											   0, 1)) {
-			msg_debug_map(
-				"don't try to reread map %s as it is locked by other process, "
-				"will reread it later",
-				cbd->map->name);
-			rspamd_map_schedule_periodic(map, RSPAMD_MAP_SCHEDULE_LOCKED);
-			MAP_RELEASE(cbd, "periodic");
+	/* For each backend we need to check for modifications */
+	if (cbd->cur_backend >= cbd->map->backends->len) {
+		/* Last backend */
+		msg_debug_map("finished map: %d of %d", cbd->cur_backend,
+					  cbd->map->backends->len);
+		MAP_RELEASE(cbd, "periodic");
 
-			return;
-		}
-		else {
-			msg_debug_map("locked map %s", cbd->map->name);
-			cbd->locked = TRUE;
-		}
+		return;
 	}
 
+	bk = g_ptr_array_index(map->backends, cbd->cur_backend);
+
 	if (cbd->errored) {
 		/* We should not check other backends if some backend has failed*/
 		rspamd_map_schedule_periodic(cbd->map, RSPAMD_MAP_SCHEDULE_ERROR);
 
-		if (cbd->locked) {
-			g_atomic_int_set(cbd->map->locked, 0);
-			cbd->locked = FALSE;
-		}
-
 		/* Also set error flag for the map consumer */
 		cbd->cbdata.errored = true;
 
@@ -2046,19 +2183,7 @@ rspamd_map_process_periodic(struct map_periodic_cbdata *cbd)
 		return;
 	}
 
-	/* For each backend we need to check for modifications */
-	if (cbd->cur_backend >= cbd->map->backends->len) {
-		/* Last backend */
-		msg_debug_map("finished map: %d of %d", cbd->cur_backend,
-					  cbd->map->backends->len);
-		MAP_RELEASE(cbd, "periodic");
-
-		return;
-	}
-
 	if (cbd->map->wrk && cbd->map->wrk->state == rspamd_worker_state_running) {
-		bk = g_ptr_array_index(cbd->map->backends, cbd->cur_backend);
-		g_assert(bk != NULL);
 
 		if (cbd->need_modify) {
 			/* Load data from the next backend */
@@ -2107,7 +2232,7 @@ rspamd_map_on_stat(struct ev_loop *loop, ev_stat *w, int revents)
 
 		/* Fire need modify flag */
 		struct rspamd_map_backend *bk;
-		guint i;
+		unsigned int i;
 
 		PTR_ARRAY_FOREACH(map->backends, i, bk)
 		{
@@ -2138,7 +2263,7 @@ void rspamd_map_watch(struct rspamd_config *cfg,
 	GList *cur = cfg->maps;
 	struct rspamd_map *map;
 	struct rspamd_map_backend *bk;
-	guint i;
+	unsigned int i;
 
 	g_assert(how > RSPAMD_MAP_WATCH_MIN && how < RSPAMD_MAP_WATCH_MAX);
 
@@ -2228,7 +2353,7 @@ void rspamd_map_preload(struct rspamd_config *cfg)
 	GList *cur = cfg->maps;
 	struct rspamd_map *map;
 	struct rspamd_map_backend *bk;
-	guint i;
+	unsigned int i;
 	gboolean map_ok;
 
 	/* First of all do synced read of data */
@@ -2337,7 +2462,7 @@ void rspamd_map_remove_all(struct rspamd_config *cfg)
 	GList *cur;
 	struct rspamd_map_backend *bk;
 	struct map_cb_data cbdata;
-	guint i;
+	unsigned int i;
 
 	for (cur = cfg->maps; cur != NULL; cur = g_list_next(cur)) {
 		map = cur->data;
@@ -2374,11 +2499,11 @@ void rspamd_map_remove_all(struct rspamd_config *cfg)
 	cfg->maps = NULL;
 }
 
-static const gchar *
+static const char *
 rspamd_map_check_proto(struct rspamd_config *cfg,
-					   const gchar *map_line, struct rspamd_map_backend *bk)
+					   const char *map_line, struct rspamd_map_backend *bk)
 {
-	const gchar *pos = map_line, *end, *end_key;
+	const char *pos = map_line, *end, *end_key;
 
 	g_assert(bk != NULL);
 	g_assert(pos != NULL);
@@ -2414,8 +2539,7 @@ rspamd_map_check_proto(struct rspamd_config *cfg,
 			end_key = memchr(pos, '+', end - pos);
 
 			if (end_key != NULL) {
-				bk->trusted_pubkey = rspamd_pubkey_from_base32(pos, end_key - pos,
-															   RSPAMD_KEYPAIR_SIGN, RSPAMD_CRYPTOBOX_MODE_25519);
+				bk->trusted_pubkey = rspamd_pubkey_from_base32(pos, end_key - pos, RSPAMD_KEYPAIR_SIGN);
 
 				if (bk->trusted_pubkey == NULL) {
 					msg_err_config("cannot read pubkey from map: %s",
@@ -2426,8 +2550,7 @@ rspamd_map_check_proto(struct rspamd_config *cfg,
 			}
 			else if (end - pos > 64) {
 				/* Try hex encoding */
-				bk->trusted_pubkey = rspamd_pubkey_from_hex(pos, 64,
-															RSPAMD_KEYPAIR_SIGN, RSPAMD_CRYPTOBOX_MODE_25519);
+				bk->trusted_pubkey = rspamd_pubkey_from_hex(pos, 64, RSPAMD_KEYPAIR_SIGN);
 
 				if (bk->trusted_pubkey == NULL) {
 					msg_err_config("cannot read pubkey from map: %s",
@@ -2489,7 +2612,7 @@ rspamd_map_check_proto(struct rspamd_config *cfg,
 }
 
 gboolean
-rspamd_map_is_map(const gchar *map_line)
+rspamd_map_is_map(const char *map_line)
 {
 	gboolean ret = FALSE;
 
@@ -2589,14 +2712,14 @@ rspamd_map_backend_dtor(struct rspamd_map_backend *bk)
 }
 
 static struct rspamd_map_backend *
-rspamd_map_parse_backend(struct rspamd_config *cfg, const gchar *map_line)
+rspamd_map_parse_backend(struct rspamd_config *cfg, const char *map_line)
 {
 	struct rspamd_map_backend *bk;
 	struct file_map_data *fdata = NULL;
 	struct http_map_data *hdata = NULL;
 	struct static_map_data *sdata = NULL;
 	struct http_parser_url up;
-	const gchar *end, *p;
+	const char *end, *p;
 	rspamd_ftok_t tok;
 
 	bk = g_malloc0(sizeof(*bk));
@@ -2693,8 +2816,8 @@ rspamd_map_parse_backend(struct rspamd_config *cfg, const gchar *map_line)
 
 			if (up.field_set & (1u << UF_USERINFO)) {
 				/* Create authorisation header for basic auth */
-				guint len = sizeof("Basic ") +
-							up.field_data[UF_USERINFO].len * 8 / 5 + 4;
+				unsigned int len = sizeof("Basic ") +
+								   up.field_data[UF_USERINFO].len * 8 / 5 + 4;
 				hdata->userinfo = g_malloc(len);
 				rspamd_snprintf(hdata->userinfo, len, "Basic %*Bs",
 								(int) up.field_data[UF_USERINFO].len,
@@ -2731,7 +2854,7 @@ rspamd_map_parse_backend(struct rspamd_config *cfg, const gchar *map_line)
 								ucl_object_type(user_obj) == UCL_STRING &&
 								ucl_object_type(password_obj) == UCL_STRING) {
 
-								gchar *tmpbuf;
+								char *tmpbuf;
 								unsigned tlen;
 
 								/* User + password + ':' */
@@ -2765,10 +2888,6 @@ rspamd_map_parse_backend(struct rspamd_config *cfg, const gchar *map_line)
 		bk->data.sd = sdata;
 	}
 
-	bk->id = rspamd_cryptobox_fast_hash_specific(RSPAMD_CRYPTOBOX_T1HA,
-												 bk->uri, strlen(bk->uri),
-												 0xdeadbabe);
-
 	return bk;
 
 err:
@@ -2793,12 +2912,19 @@ static void
 rspamd_map_calculate_hash(struct rspamd_map *map)
 {
 	struct rspamd_map_backend *bk;
-	guint i;
+	unsigned int i;
 	rspamd_cryptobox_hash_state_t st;
-	gchar *cksum_encoded, cksum[rspamd_cryptobox_HASHBYTES];
+	char *cksum_encoded, cksum[rspamd_cryptobox_HASHBYTES];
 
 	rspamd_cryptobox_hash_init(&st, NULL, 0);
 
+	if (map->name) {
+		rspamd_cryptobox_hash_update(&st, map->name, strlen(map->name));
+	}
+	if (map->description) {
+		rspamd_cryptobox_hash_update(&st, map->description, strlen(map->description));
+	}
+
 	for (i = 0; i < map->backends->len; i++) {
 		bk = g_ptr_array_index(map->backends, i);
 		rspamd_cryptobox_hash_update(&st, bk->uri, strlen(bk->uri));
@@ -2807,6 +2933,26 @@ rspamd_map_calculate_hash(struct rspamd_map *map)
 	rspamd_cryptobox_hash_final(&st, cksum);
 	cksum_encoded = rspamd_encode_base32(cksum, sizeof(cksum), RSPAMD_BASE32_DEFAULT);
 	rspamd_strlcpy(map->tag, cksum_encoded, sizeof(map->tag));
+
+	for (i = 0; i < map->backends->len; i++) {
+		bk = g_ptr_array_index(map->backends, i);
+
+		/* Also update each backend */
+		rspamd_cryptobox_fast_hash_state_t hst;
+		rspamd_cryptobox_fast_hash_init(&hst, 0);
+		rspamd_cryptobox_fast_hash_update(&hst, bk->uri, strlen(bk->uri));
+		rspamd_cryptobox_fast_hash_update(&hst, map->tag, sizeof(map->tag));
+
+		if (bk->protocol == MAP_PROTO_STATIC) {
+			/* Static maps content is pre-defined */
+			rspamd_cryptobox_fast_hash_update(&hst, bk->data.sd->data,
+											  bk->data.sd->len);
+		}
+
+		/* We use only 52 bits to be compatible with other numbers representation */
+		bk->id = rspamd_cryptobox_fast_hash_final(&hst) & ~(0xFFFULL << 52);
+	}
+
 	g_free(cksum_encoded);
 }
 
@@ -2816,7 +2962,7 @@ rspamd_map_add_static_string(struct rspamd_config *cfg,
 							 GString *target)
 {
 	gsize sz;
-	const gchar *dline;
+	const char *dline;
 
 	if (ucl_object_type(elt) != UCL_STRING) {
 		msg_err_config("map has static backend but `data` is "
@@ -2841,8 +2987,8 @@ rspamd_map_add_static_string(struct rspamd_config *cfg,
 
 struct rspamd_map *
 rspamd_map_add(struct rspamd_config *cfg,
-			   const gchar *map_line,
-			   const gchar *description,
+			   const char *map_line,
+			   const char *description,
 			   map_cb_t read_callback,
 			   map_fin_cb_t fin_callback,
 			   map_dtor_t dtor,
@@ -2872,8 +3018,8 @@ rspamd_map_add(struct rspamd_config *cfg,
 	map->user_data = user_data;
 	map->cfg = cfg;
 	map->id = rspamd_random_uint64_fast();
-	map->locked =
-		rspamd_mempool_alloc0_shared(cfg->cfg_pool, sizeof(gint));
+	map->shared =
+		rspamd_mempool_alloc0_shared(cfg->cfg_pool, sizeof(struct rspamd_map_shared_data));
 	map->backends = g_ptr_array_sized_new(1);
 	map->wrk = worker;
 	rspamd_mempool_add_destructor(cfg->cfg_pool, rspamd_ptr_array_free_hard,
@@ -2904,8 +3050,8 @@ rspamd_map_add(struct rspamd_config *cfg,
 
 struct rspamd_map *
 rspamd_map_add_fake(struct rspamd_config *cfg,
-					const gchar *description,
-					const gchar *name)
+					const char *description,
+					const char *name)
 {
 	struct rspamd_map *map;
 
@@ -2943,19 +3089,19 @@ rspamd_map_add_backend(struct rspamd_map *map, struct rspamd_map_backend *bk)
 struct rspamd_map *
 rspamd_map_add_from_ucl(struct rspamd_config *cfg,
 						const ucl_object_t *obj,
-						const gchar *description,
+						const char *description,
 						map_cb_t read_callback,
 						map_fin_cb_t fin_callback,
 						map_dtor_t dtor,
 						void **user_data,
 						struct rspamd_worker *worker,
-						gint flags)
+						int flags)
 {
 	ucl_object_iter_t it = NULL;
 	const ucl_object_t *cur, *elt;
 	struct rspamd_map *map;
 	struct rspamd_map_backend *bk;
-	guint i;
+	unsigned int i;
 
 	g_assert(obj != NULL);
 
@@ -2972,8 +3118,8 @@ rspamd_map_add_from_ucl(struct rspamd_config *cfg,
 	map->user_data = user_data;
 	map->cfg = cfg;
 	map->id = rspamd_random_uint64_fast();
-	map->locked =
-		rspamd_mempool_alloc0_shared(cfg->cfg_pool, sizeof(gint));
+	map->shared =
+		rspamd_mempool_alloc0_shared(cfg->cfg_pool, sizeof(struct rspamd_map_shared_data));
 	map->backends = g_ptr_array_new();
 	map->wrk = worker;
 	map->no_file_read = (flags & RSPAMD_MAP_FILE_NO_READ);
@@ -3092,7 +3238,7 @@ rspamd_map_add_from_ucl(struct rspamd_config *cfg,
 		goto err;
 	}
 
-	gboolean all_local = TRUE;
+	gboolean all_local = TRUE, all_loaded = TRUE;
 
 	PTR_ARRAY_FOREACH(map->backends, i, bk)
 	{
@@ -3111,9 +3257,8 @@ rspamd_map_add_from_ucl(struct rspamd_config *cfg,
 				map_data = g_string_sized_new(32);
 
 				if (rspamd_map_add_static_string(cfg, elt, map_data)) {
-					bk->data.sd->data = map_data->str;
 					bk->data.sd->len = map_data->len;
-					g_string_free(map_data, FALSE);
+					bk->data.sd->data = (unsigned char *) g_string_free(map_data, FALSE);
 				}
 				else {
 					g_string_free(map_data, TRUE);
@@ -3136,13 +3281,16 @@ rspamd_map_add_from_ucl(struct rspamd_config *cfg,
 				}
 
 				ucl_object_iterate_free(it);
-				bk->data.sd->data = map_data->str;
 				bk->data.sd->len = map_data->len;
-				g_string_free(map_data, FALSE);
+				bk->data.sd->data = (unsigned char *) g_string_free(map_data, FALSE);
 			}
 		}
 		else if (bk->protocol != MAP_PROTO_FILE) {
 			all_local = FALSE;
+			all_loaded = FALSE; /* Will be loaded later */
+		}
+		else {
+			all_loaded = FALSE; /* Will be loaded later (even for files) */
 		}
 	}
 
@@ -3151,6 +3299,11 @@ rspamd_map_add_from_ucl(struct rspamd_config *cfg,
 							 cfg->map_file_watch_multiplier);
 	}
 
+	if (all_loaded) {
+		/* Static map */
+		g_atomic_int_set(&map->shared->loaded, 1);
+	}
+
 	rspamd_map_calculate_hash(map);
 	msg_debug_map("added map from ucl");
 
