@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -43,22 +43,22 @@
 
 struct spf_resolved_element {
 	GPtrArray *elts;
-	gchar *cur_domain;
+	char *cur_domain;
 	gboolean redirected; /* Ignore level, it's redirected */
 };
 
 struct spf_record {
-	gint nested;
-	gint dns_requests;
-	gint requests_inflight;
+	int nested;
+	int dns_requests;
+	int requests_inflight;
 
-	guint ttl;
+	unsigned int ttl;
 	GPtrArray *resolved;
 	/* Array of struct spf_resolved_element */
-	const gchar *sender;
-	const gchar *sender_domain;
-	const gchar *top_record;
-	gchar *local_part;
+	const char *sender;
+	const char *sender_domain;
+	const char *top_record;
+	char *local_part;
 	struct rspamd_task *task;
 	spf_cb_t callback;
 	gpointer cbdata;
@@ -66,9 +66,9 @@ struct spf_record {
 };
 
 struct rspamd_spf_library_ctx {
-	guint max_dns_nesting;
-	guint max_dns_requests;
-	guint min_cache_ttl;
+	unsigned int max_dns_nesting;
+	unsigned int max_dns_requests;
+	unsigned int min_cache_ttl;
 	gboolean disable_ipv6;
 	rspamd_lru_hash_t *spf_hash;
 };
@@ -125,31 +125,42 @@ struct rspamd_spf_library_ctx *spf_lib_ctx = NULL;
 
 INIT_LOG_MODULE(spf)
 
+
 struct spf_dns_cb {
 	struct spf_record *rec;
 	struct spf_addr *addr;
 	struct spf_resolved_element *resolved;
-	const gchar *ptr_host;
-	spf_action_t cur_action;
-	gboolean in_include;
+	const char *initiated_dns_name;
+	spf_action_t initiated_by;
+	unsigned eyeballs_sent;     /* number of DNS subrequests sent */
+	unsigned eyeballs_received; /* number of DNS subrequests received */
+	unsigned eyeballs_errors;   /* number of DNS subrequests errored */
 };
 
-#define CHECK_REC(rec)                                                             \
-	do {                                                                           \
-		if (spf_lib_ctx->max_dns_nesting > 0 &&                                    \
-			(rec)->nested > spf_lib_ctx->max_dns_nesting) {                        \
-			msg_warn_spf("spf nesting limit: %d > %d is reached, domain: %s",      \
-						 (rec)->nested, spf_lib_ctx->max_dns_nesting,              \
-						 (rec)->sender_domain);                                    \
-			return FALSE;                                                          \
-		}                                                                          \
-		if (spf_lib_ctx->max_dns_requests > 0 &&                                   \
-			(rec)->dns_requests > spf_lib_ctx->max_dns_requests) {                 \
-			msg_warn_spf("spf dns requests limit: %d > %d is reached, domain: %s", \
-						 (rec)->dns_requests, spf_lib_ctx->max_dns_requests,       \
-						 (rec)->sender_domain);                                    \
-			return FALSE;                                                          \
-		}                                                                          \
+static inline bool
+spf_record_can_dns(const struct spf_record *rec)
+{
+	if (spf_lib_ctx->max_dns_nesting > 0 &&
+		rec->nested > spf_lib_ctx->max_dns_nesting) {
+		msg_warn_spf("spf nesting limit: %d > %d is reached, domain: %s",
+					 rec->nested, spf_lib_ctx->max_dns_nesting,
+					 rec->sender_domain);
+		return false;
+	}
+	if (spf_lib_ctx->max_dns_requests > 0 &&
+		rec->dns_requests > spf_lib_ctx->max_dns_requests) {
+		msg_warn_spf("spf dns requests limit: %d > %d is reached, domain: %s",
+					 rec->dns_requests, spf_lib_ctx->max_dns_requests,
+					 rec->sender_domain);
+		return false;
+	}
+
+	return true;
+}
+
+#define CHECK_REC(rec)                              \
+	do {                                            \
+		if (!spf_record_can_dns(rec)) return FALSE; \
 	} while (0)
 
 RSPAMD_CONSTRUCTOR(rspamd_spf_lib_ctx_ctor)
@@ -175,13 +186,13 @@ spf_record_cached_unref_dtor(gpointer p)
 {
 	struct spf_resolved *flat = (struct spf_resolved *) p;
 
-	_spf_record_unref(flat, "LRU cache");
+	spf_record_unref_internal(flat, "LRU cache");
 }
 
 void spf_library_config(const ucl_object_t *obj)
 {
 	const ucl_object_t *value;
-	gint64 ival;
+	int64_t ival;
 	bool bval;
 
 	if (obj == NULL) {
@@ -235,11 +246,11 @@ void spf_library_config(const ucl_object_t *obj)
 }
 
 static gboolean start_spf_parse(struct spf_record *rec,
-								struct spf_resolved_element *resolved, gchar *begin);
+								struct spf_resolved_element *resolved, char *begin);
 
 /* Determine spf mech */
 static spf_mech_t
-check_spf_mech(const gchar *elt, gboolean *need_shift)
+check_spf_mech(const char *elt, gboolean *need_shift)
 {
 	g_assert(elt != NULL);
 
@@ -260,7 +271,7 @@ check_spf_mech(const gchar *elt, gboolean *need_shift)
 	}
 }
 
-static const gchar *
+static const char *
 rspamd_spf_dns_action_to_str(spf_action_t act)
 {
 	const char *ret = "unknown";
@@ -297,7 +308,7 @@ rspamd_spf_dns_action_to_str(spf_action_t act)
 
 static struct spf_addr *
 rspamd_spf_new_addr(struct spf_record *rec,
-					struct spf_resolved_element *resolved, const gchar *elt)
+					struct spf_resolved_element *resolved, const char *elt)
 {
 	gboolean need_shift = FALSE;
 	struct spf_addr *naddr;
@@ -334,7 +345,7 @@ rspamd_spf_free_addr(gpointer a)
 }
 
 static struct spf_resolved_element *
-rspamd_spf_new_addr_list(struct spf_record *rec, const gchar *domain)
+rspamd_spf_new_addr_list(struct spf_record *rec, const char *domain)
 {
 	struct spf_resolved_element *resolved;
 
@@ -356,7 +367,7 @@ spf_record_destructor(gpointer r)
 {
 	struct spf_record *rec = r;
 	struct spf_resolved_element *elt;
-	guint i;
+	unsigned int i;
 
 	if (rec) {
 		for (i = 0; i < rec->resolved->len; i++) {
@@ -374,7 +385,7 @@ static void
 rspamd_flatten_record_dtor(struct spf_resolved *r)
 {
 	struct spf_addr *addr;
-	guint i;
+	unsigned int i;
 
 	for (i = 0; i < r->elts->len; i++) {
 		addr = &g_array_index(r->elts, struct spf_addr, i);
@@ -393,7 +404,7 @@ rspamd_spf_process_reference(struct spf_resolved *target,
 {
 	struct spf_resolved_element *elt, *relt;
 	struct spf_addr *cur = NULL, taddr, *cur_addr;
-	guint i;
+	unsigned int i;
 
 	if (addr) {
 		g_assert(addr->m.idx < rec->resolved->len);
@@ -451,6 +462,9 @@ rspamd_spf_process_reference(struct spf_resolved *target,
 			target->flags |= RSPAMD_SPF_RESOLVED_NA;
 			continue;
 		}
+		if (cur->flags & RSPAMD_SPF_FLAG_PLUSALL) {
+			target->flags |= RSPAMD_SPF_RESOLVED_PLUSALL;
+		}
 		if (cur->flags & RSPAMD_SPF_FLAG_INVALID) {
 			/* Ignore invalid elements */
 			continue;
@@ -521,7 +535,7 @@ rspamd_spf_record_flatten(struct spf_record *rec)
 	return res;
 }
 
-static gint
+static int
 rspamd_spf_elts_cmp(gconstpointer a, gconstpointer b)
 {
 	struct spf_addr *addr_a, *addr_b;
@@ -565,30 +579,30 @@ rspamd_spf_record_postprocess(struct spf_resolved *rec, struct rspamd_task *task
 {
 	g_array_sort(rec->elts, rspamd_spf_elts_cmp);
 
-	for (guint i = 0; i < rec->elts->len; i++) {
+	for (unsigned int i = 0; i < rec->elts->len; i++) {
 		struct spf_addr *cur_addr = &g_array_index(rec->elts, struct spf_addr, i);
 
 		if (cur_addr->flags & RSPAMD_SPF_FLAG_IPV6) {
-			guint64 t[3];
+			uint64_t t[3];
 
 			/*
 			 * Fill hash entry for ipv6 addr with 2 int64 from ipv6 address,
 			 * the remaining int64 has mech + mask
 			 */
-			memcpy(t, cur_addr->addr6, sizeof(guint64) * 2);
-			t[2] = ((guint64) (cur_addr->mech)) << 48u;
+			memcpy(t, cur_addr->addr6, sizeof(uint64_t) * 2);
+			t[2] = ((uint64_t) (cur_addr->mech)) << 48u;
 			t[2] |= cur_addr->m.dual.mask_v6;
 
-			for (guint j = 0; j < G_N_ELEMENTS(t); j++) {
+			for (unsigned int j = 0; j < G_N_ELEMENTS(t); j++) {
 				rec->digest = mum_hash_step(rec->digest, t[j]);
 			}
 		}
 		else if (cur_addr->flags & RSPAMD_SPF_FLAG_IPV4) {
-			guint64 t = 0;
+			uint64_t t = 0;
 
-			memcpy(&t, cur_addr->addr4, sizeof(guint32));
-			t |= ((guint64) (cur_addr->mech)) << 48u;
-			t |= ((guint64) cur_addr->m.dual.mask_v4) << 32u;
+			memcpy(&t, cur_addr->addr4, sizeof(uint32_t));
+			t |= ((uint64_t) (cur_addr->mech)) << 48u;
+			t |= ((uint64_t) cur_addr->m.dual.mask_v4) << 32u;
 
 			rec->digest = mum_hash_step(rec->digest, t);
 		}
@@ -654,8 +668,8 @@ spf_check_ptr_host(struct spf_dns_cb *cb, const char *name)
 	const char *dend, *nend, *dstart, *nstart;
 	struct spf_record *rec = cb->rec;
 
-	if (cb->ptr_host != NULL) {
-		dstart = cb->ptr_host;
+	if (cb->initiated_dns_name != NULL) {
+		dstart = cb->initiated_dns_name;
 	}
 	else {
 		dstart = cb->resolved->cur_domain;
@@ -769,7 +783,7 @@ spf_record_process_addr(struct spf_record *rec, struct spf_addr *addr, struct rd
 static void
 spf_record_addr_set(struct spf_addr *addr, gboolean allow_any)
 {
-	guchar fill;
+	unsigned char fill;
 
 	if (!(addr->flags & RSPAMD_SPF_FLAG_PROCESSED)) {
 		if (allow_any) {
@@ -877,48 +891,62 @@ spf_record_dns_callback(struct rdns_reply *reply, gpointer arg)
 
 	if (reply->code == RDNS_RC_NOERROR && !truncated) {
 
+		msg_debug_spf("in dns callback initiated by %s for %s: resolved",
+					  rspamd_spf_dns_action_to_str(cb->initiated_by),
+					  req_name ? req_name->name : "???");
+
 		LL_FOREACH(reply->entries, elt_data)
 		{
-			/* Adjust ttl if a resolved record has lower ttl than spf record itself */
-			if ((guint) elt_data->ttl < rec->ttl) {
-				msg_debug_spf("reducing ttl from %d to %d after DNS resolving",
-							  rec->ttl, elt_data->ttl);
-				rec->ttl = elt_data->ttl;
-			}
-
 			if (elt_data->type == RDNS_REQUEST_CNAME) {
 				/* Skip cname aliases - it must be handled by a recursor */
 				continue;
 			}
 
-			switch (cb->cur_action) {
+			/* Adjust ttl if a resolved record has lower ttl than spf record itself */
+			if ((unsigned int) elt_data->ttl < rec->ttl) {
+				msg_debug_spf("reducing ttl from %d to %d after DNS resolving",
+							  rec->ttl, elt_data->ttl);
+				rec->ttl = elt_data->ttl;
+			}
+
+			switch (cb->initiated_by) {
 			case SPF_RESOLVE_MX:
 				if (elt_data->type == RDNS_REQUEST_MX) {
 					/* Now resolve A record for this MX */
 					msg_debug_spf("resolve %s after resolving of MX",
 								  elt_data->content.mx.name);
-					if (rspamd_dns_resolver_request_task_forced(task,
-																spf_record_dns_callback, (void *) cb,
-																RDNS_REQUEST_A,
-																elt_data->content.mx.name)) {
-						cb->rec->requests_inflight++;
-					}
-
-					if (!spf_lib_ctx->disable_ipv6) {
+					if (spf_record_can_dns(rec)) {
 						if (rspamd_dns_resolver_request_task_forced(task,
 																	spf_record_dns_callback, (void *) cb,
-																	RDNS_REQUEST_AAAA,
+																	RDNS_REQUEST_A,
 																	elt_data->content.mx.name)) {
 							cb->rec->requests_inflight++;
+							cb->eyeballs_sent++;
+						}
+
+						if (!spf_lib_ctx->disable_ipv6) {
+							if (rspamd_dns_resolver_request_task_forced(task,
+																		spf_record_dns_callback, (void *) cb,
+																		RDNS_REQUEST_AAAA,
+																		elt_data->content.mx.name)) {
+								cb->rec->requests_inflight++;
+								cb->eyeballs_sent++;
+							}
+						}
+						else {
+							msg_debug_spf("skip AAAA request for MX resolution");
 						}
 					}
 					else {
-						msg_debug_spf("skip AAAA request for MX resolution");
+						/* Max DNS requests reached */
+						cb->addr->flags |= RSPAMD_SPF_FLAG_PERMFAIL;
 					}
 				}
 				else {
+					/* If any of the eyeballs requests success we should consider this as resolved */
+					cb->eyeballs_received++;
 					cb->addr->flags |= RSPAMD_SPF_FLAG_RESOLVED;
-					cb->addr->flags &= ~RSPAMD_SPF_FLAG_PERMFAIL;
+					cb->addr->flags &= ~(RSPAMD_SPF_FLAG_PERMFAIL | RSPAMD_SPF_FLAG_TEMPFAIL);
 					msg_debug_spf("resolved MX addr");
 					spf_record_process_addr(rec, addr, elt_data);
 				}
@@ -936,23 +964,31 @@ spf_record_dns_callback(struct rdns_reply *reply, gpointer arg)
 										   elt_data->content.ptr.name)) {
 						msg_debug_spf("resolve PTR %s after resolving of PTR",
 									  elt_data->content.ptr.name);
-						if (rspamd_dns_resolver_request_task_forced(task,
-																	spf_record_dns_callback, (void *) cb,
-																	RDNS_REQUEST_A,
-																	elt_data->content.ptr.name)) {
-							cb->rec->requests_inflight++;
-						}
-
-						if (!spf_lib_ctx->disable_ipv6) {
+						if (spf_record_can_dns(rec)) {
 							if (rspamd_dns_resolver_request_task_forced(task,
 																		spf_record_dns_callback, (void *) cb,
-																		RDNS_REQUEST_AAAA,
+																		RDNS_REQUEST_A,
 																		elt_data->content.ptr.name)) {
 								cb->rec->requests_inflight++;
+								cb->eyeballs_sent++;
+							}
+
+							if (!spf_lib_ctx->disable_ipv6) {
+								if (rspamd_dns_resolver_request_task_forced(task,
+																			spf_record_dns_callback, (void *) cb,
+																			RDNS_REQUEST_AAAA,
+																			elt_data->content.ptr.name)) {
+									cb->rec->requests_inflight++;
+									cb->eyeballs_sent++;
+								}
+							}
+							else {
+								msg_debug_spf("skip AAAA request for PTR resolution");
 							}
 						}
 						else {
-							msg_debug_spf("skip AAAA request for PTR resolution");
+							/* Max DNS requests reached */
+							cb->addr->flags |= RSPAMD_SPF_FLAG_PERMFAIL;
 						}
 					}
 					else {
@@ -961,6 +997,8 @@ spf_record_dns_callback(struct rdns_reply *reply, gpointer arg)
 					}
 				}
 				else {
+					/* If any of the eyeballs requests success we should consider this as resolved */
+					cb->eyeballs_received++;
 					cb->addr->flags |= RSPAMD_SPF_FLAG_RESOLVED;
 					cb->addr->flags &= ~RSPAMD_SPF_FLAG_PERMFAIL;
 					spf_record_process_addr(rec, addr, elt_data);
@@ -1007,9 +1045,9 @@ spf_record_dns_callback(struct rdns_reply *reply, gpointer arg)
 				if (elt_data->type == RDNS_REQUEST_A ||
 					elt_data->type == RDNS_REQUEST_AAAA) {
 					/*
-						 * If specified address resolves, we can accept
-						 * connection from every IP
-						 */
+					 * If specified address resolves, we can accept
+					 * connection from every IP
+					 */
 					addr->flags |= RSPAMD_SPF_FLAG_RESOLVED;
 					spf_record_addr_set(addr, TRUE);
 				}
@@ -1018,9 +1056,20 @@ spf_record_dns_callback(struct rdns_reply *reply, gpointer arg)
 		}
 	}
 	else if (reply->code == RDNS_RC_NXDOMAIN || reply->code == RDNS_RC_NOREC) {
-		switch (cb->cur_action) {
+
+		msg_debug_spf("in dns callback initiated by %s for %s: NXDOMAIN/NOREC; "
+					  "eyeballs_status: %d sent/%d received, %d errors",
+					  rspamd_spf_dns_action_to_str(cb->initiated_by),
+					  req_name ? req_name->name : "???",
+					  cb->eyeballs_sent, cb->eyeballs_received, cb->eyeballs_errors);
+
+		switch (cb->initiated_by) {
 		case SPF_RESOLVE_MX:
-			if (!(cb->addr->flags & RSPAMD_SPF_FLAG_RESOLVED)) {
+			cb->eyeballs_received++;
+			cb->eyeballs_errors++;
+
+			if (cb->eyeballs_received == cb->eyeballs_sent && cb->eyeballs_errors == cb->eyeballs_sent) {
+				/* We only set error if all eyeball requests have failed */
 				cb->addr->flags |= RSPAMD_SPF_FLAG_PERMFAIL;
 				msg_info_spf(
 					"spf error for domain %s: cannot find MX"
@@ -1061,7 +1110,10 @@ spf_record_dns_callback(struct rdns_reply *reply, gpointer arg)
 			}
 			break;
 		case SPF_RESOLVE_PTR:
-			if (!(cb->addr->flags & RSPAMD_SPF_FLAG_RESOLVED)) {
+			cb->eyeballs_received++;
+			cb->eyeballs_errors++;
+
+			if (cb->eyeballs_received == cb->eyeballs_sent && cb->eyeballs_errors == cb->eyeballs_sent) {
 				msg_info_spf(
 					"spf error for domain %s: cannot resolve PTR"
 					" record for %s: %s",
@@ -1118,8 +1170,8 @@ spf_record_dns_callback(struct rdns_reply *reply, gpointer arg)
 			"spf error for domain %s: cannot resolve %s DNS record for"
 			" %s: %s",
 			cb->rec->sender_domain,
-			rspamd_spf_dns_action_to_str(cb->cur_action),
-			cb->ptr_host,
+			rspamd_spf_dns_action_to_str(cb->initiated_by),
+			cb->initiated_dns_name,
 			rdns_strerror(reply->code));
 	}
 
@@ -1134,7 +1186,7 @@ end:
  * ip6-cidr-length  = "/" 1*DIGIT
  * dual-cidr-length = [ ip4-cidr-length ] [ "/" ip6-cidr-length ]
  */
-static const gchar *
+static const char *
 parse_spf_domain_mask(struct spf_record *rec, struct spf_addr *addr,
 					  struct spf_resolved_element *resolved,
 					  gboolean allow_mask)
@@ -1150,10 +1202,10 @@ parse_spf_domain_mask(struct spf_record *rec, struct spf_addr *addr,
 		parse_ipv6_mask,
 		skip_garbage
 	} state = 0;
-	const gchar *p = addr->spf_string, *host, *c;
-	gchar *hostbuf;
-	gchar t;
-	guint16 cur_mask = 0;
+	const char *p = addr->spf_string, *host, *c;
+	char *hostbuf;
+	char t;
+	uint16_t cur_mask = 0;
 
 	host = resolved->cur_domain;
 	c = p;
@@ -1272,7 +1324,7 @@ parse_spf_a(struct spf_record *rec,
 			struct spf_resolved_element *resolved, struct spf_addr *addr)
 {
 	struct spf_dns_cb *cb;
-	const gchar *host = NULL;
+	const char *host = NULL;
 	struct rspamd_task *task = rec->task;
 
 	CHECK_REC(rec);
@@ -1284,11 +1336,11 @@ parse_spf_a(struct spf_record *rec,
 	}
 
 	rec->dns_requests++;
-	cb = rspamd_mempool_alloc(task->task_pool, sizeof(struct spf_dns_cb));
+	cb = rspamd_mempool_alloc0(task->task_pool, sizeof(struct spf_dns_cb));
 	cb->rec = rec;
-	cb->ptr_host = host;
+	cb->initiated_dns_name = host;
 	cb->addr = addr;
-	cb->cur_action = SPF_RESOLVE_A;
+	cb->initiated_by = SPF_RESOLVE_A;
 	cb->resolved = resolved;
 	msg_debug_spf("resolve a %s", host);
 
@@ -1296,14 +1348,14 @@ parse_spf_a(struct spf_record *rec,
 												spf_record_dns_callback, (void *) cb, RDNS_REQUEST_A, host)) {
 		rec->requests_inflight++;
 
-		cb = rspamd_mempool_alloc(task->task_pool, sizeof(struct spf_dns_cb));
-		cb->rec = rec;
-		cb->ptr_host = host;
-		cb->addr = addr;
-		cb->cur_action = SPF_RESOLVE_AAA;
-		cb->resolved = resolved;
-
 		if (!spf_lib_ctx->disable_ipv6) {
+			cb = rspamd_mempool_alloc0(task->task_pool, sizeof(struct spf_dns_cb));
+			cb->rec = rec;
+			cb->initiated_dns_name = host;
+			cb->addr = addr;
+			cb->initiated_by = SPF_RESOLVE_AAA;
+			cb->resolved = resolved;
+
 			if (rspamd_dns_resolver_request_task_forced(task,
 														spf_record_dns_callback, (void *) cb, RDNS_REQUEST_AAAA, host)) {
 				rec->requests_inflight++;
@@ -1328,8 +1380,8 @@ parse_spf_ptr(struct spf_record *rec,
 			  struct spf_resolved_element *resolved, struct spf_addr *addr)
 {
 	struct spf_dns_cb *cb;
-	const gchar *host;
-	gchar *ptr;
+	const char *host;
+	char *ptr;
 	struct rspamd_task *task = rec->task;
 
 	CHECK_REC(rec);
@@ -1337,12 +1389,12 @@ parse_spf_ptr(struct spf_record *rec,
 	host = parse_spf_domain_mask(rec, addr, resolved, FALSE);
 
 	rec->dns_requests++;
-	cb = rspamd_mempool_alloc(task->task_pool, sizeof(struct spf_dns_cb));
+	cb = rspamd_mempool_alloc0(task->task_pool, sizeof(struct spf_dns_cb));
 	cb->rec = rec;
 	cb->addr = addr;
-	cb->cur_action = SPF_RESOLVE_PTR;
+	cb->initiated_by = SPF_RESOLVE_PTR;
 	cb->resolved = resolved;
-	cb->ptr_host = rspamd_mempool_strdup(task->task_pool, host);
+	cb->initiated_dns_name = rspamd_mempool_strdup(task->task_pool, host);
 	ptr =
 		rdns_generate_ptr_from_str(rspamd_inet_address_to_string(
 			task->from_addr));
@@ -1375,7 +1427,7 @@ parse_spf_mx(struct spf_record *rec,
 			 struct spf_resolved_element *resolved, struct spf_addr *addr)
 {
 	struct spf_dns_cb *cb;
-	const gchar *host;
+	const char *host;
 	struct rspamd_task *task = rec->task;
 
 	CHECK_REC(rec);
@@ -1387,11 +1439,11 @@ parse_spf_mx(struct spf_record *rec,
 	}
 
 	rec->dns_requests++;
-	cb = rspamd_mempool_alloc(task->task_pool, sizeof(struct spf_dns_cb));
+	cb = rspamd_mempool_alloc0(task->task_pool, sizeof(struct spf_dns_cb));
 	cb->rec = rec;
 	cb->addr = addr;
-	cb->cur_action = SPF_RESOLVE_MX;
-	cb->ptr_host = host;
+	cb->initiated_by = SPF_RESOLVE_MX;
+	cb->initiated_dns_name = host;
 	cb->resolved = resolved;
 
 	msg_debug_spf("resolve mx for %s", host);
@@ -1418,7 +1470,7 @@ parse_spf_all(struct spf_record *rec, struct spf_addr *addr)
 
 	/* Disallow +all */
 	if (addr->mech == SPF_PASS) {
-		addr->flags |= RSPAMD_SPF_FLAG_INVALID;
+		addr->flags |= RSPAMD_SPF_FLAG_PLUSALL;
 		msg_notice_spf("domain %s allows any SPF (+all), ignore SPF record completely",
 					   rec->sender_domain);
 	}
@@ -1430,11 +1482,11 @@ static gboolean
 parse_spf_ip4(struct spf_record *rec, struct spf_addr *addr)
 {
 	/* ip4:addr[/mask] */
-	const gchar *semicolon, *slash;
+	const char *semicolon, *slash;
 	gsize len;
-	gchar ipbuf[INET_ADDRSTRLEN + 1];
-	guint32 mask;
-	static const guint32 min_valid_mask = 8;
+	char ipbuf[INET_ADDRSTRLEN + 1];
+	uint32_t mask;
+	static const uint32_t min_valid_mask = 8;
 
 	semicolon = strchr(addr->spf_string, ':');
 
@@ -1467,7 +1519,7 @@ parse_spf_ip4(struct spf_record *rec, struct spf_addr *addr)
 	}
 
 	if (slash) {
-		gchar *end = NULL;
+		char *end = NULL;
 
 		mask = strtoul(slash + 1, &end, 10);
 		if (mask > 32) {
@@ -1506,11 +1558,11 @@ static gboolean
 parse_spf_ip6(struct spf_record *rec, struct spf_addr *addr)
 {
 	/* ip6:addr[/mask] */
-	const gchar *semicolon, *slash;
+	const char *semicolon, *slash;
 	gsize len;
-	gchar ipbuf[INET6_ADDRSTRLEN + 1];
-	guint32 mask;
-	static const guint32 min_valid_mask = 8;
+	char ipbuf[INET6_ADDRSTRLEN + 1];
+	uint32_t mask;
+	static const uint32_t min_valid_mask = 8;
 
 	semicolon = strchr(addr->spf_string, ':');
 
@@ -1543,7 +1595,7 @@ parse_spf_ip6(struct spf_record *rec, struct spf_addr *addr)
 	}
 
 	if (slash) {
-		gchar *end = NULL;
+		char *end = NULL;
 		mask = strtoul(slash + 1, &end, 10);
 		if (mask > 128) {
 			msg_notice_spf("invalid mask for ip6 element for %s: %s", addr->spf_string,
@@ -1582,7 +1634,7 @@ static gboolean
 parse_spf_include(struct spf_record *rec, struct spf_addr *addr)
 {
 	struct spf_dns_cb *cb;
-	const gchar *domain;
+	const char *domain;
 	struct rspamd_task *task = rec->task;
 
 	CHECK_REC(rec);
@@ -1603,13 +1655,13 @@ parse_spf_include(struct spf_record *rec, struct spf_addr *addr)
 
 	rec->dns_requests++;
 
-	cb = rspamd_mempool_alloc(task->task_pool, sizeof(struct spf_dns_cb));
+	cb = rspamd_mempool_alloc0(task->task_pool, sizeof(struct spf_dns_cb));
 	cb->rec = rec;
 	cb->addr = addr;
-	cb->cur_action = SPF_RESOLVE_INCLUDE;
+	cb->initiated_by = SPF_RESOLVE_INCLUDE;
 	addr->m.idx = rec->resolved->len;
 	cb->resolved = rspamd_spf_new_addr_list(rec, domain);
-	cb->ptr_host = domain;
+	cb->initiated_dns_name = domain;
 	/* Set reference */
 	addr->flags |= RSPAMD_SPF_FLAG_REFERENCE;
 	msg_debug_spf("resolve include %s", domain);
@@ -1641,7 +1693,7 @@ parse_spf_redirect(struct spf_record *rec,
 				   struct spf_resolved_element *resolved, struct spf_addr *addr)
 {
 	struct spf_dns_cb *cb;
-	const gchar *domain;
+	const char *domain;
 	struct rspamd_task *task = rec->task;
 
 	CHECK_REC(rec);
@@ -1664,16 +1716,16 @@ parse_spf_redirect(struct spf_record *rec,
 	rec->dns_requests++;
 	resolved->redirected = TRUE;
 
-	cb = rspamd_mempool_alloc(task->task_pool, sizeof(struct spf_dns_cb));
+	cb = rspamd_mempool_alloc0(task->task_pool, sizeof(struct spf_dns_cb));
 	/* Set reference */
 	addr->flags |= RSPAMD_SPF_FLAG_REFERENCE | RSPAMD_SPF_FLAG_REDIRECT;
 	addr->m.idx = rec->resolved->len;
 
 	cb->rec = rec;
 	cb->addr = addr;
-	cb->cur_action = SPF_RESOLVE_REDIRECT;
+	cb->initiated_by = SPF_RESOLVE_REDIRECT;
 	cb->resolved = rspamd_spf_new_addr_list(rec, domain);
-	cb->ptr_host = domain;
+	cb->initiated_dns_name = domain;
 	msg_debug_spf("resolve redirect %s", domain);
 
 	if (rspamd_dns_resolver_request_task_forced(task,
@@ -1694,7 +1746,7 @@ static gboolean
 parse_spf_exists(struct spf_record *rec, struct spf_addr *addr)
 {
 	struct spf_dns_cb *cb;
-	const gchar *host;
+	const char *host;
 	struct rspamd_task *task = rec->task;
 	struct spf_resolved_element *resolved;
 
@@ -1715,12 +1767,12 @@ parse_spf_exists(struct spf_record *rec, struct spf_addr *addr)
 	host++;
 	rec->dns_requests++;
 
-	cb = rspamd_mempool_alloc(task->task_pool, sizeof(struct spf_dns_cb));
+	cb = rspamd_mempool_alloc0(task->task_pool, sizeof(struct spf_dns_cb));
 	cb->rec = rec;
 	cb->addr = addr;
-	cb->cur_action = SPF_RESOLVE_EXISTS;
+	cb->initiated_by = SPF_RESOLVE_EXISTS;
 	cb->resolved = resolved;
-	cb->ptr_host = host;
+	cb->initiated_dns_name = host;
 
 	msg_debug_spf("resolve exists %s", host);
 	if (rspamd_dns_resolver_request_task_forced(task,
@@ -1738,11 +1790,11 @@ parse_spf_exists(struct spf_record *rec, struct spf_addr *addr)
 }
 
 static gsize
-rspamd_spf_split_elt(const gchar *val, gsize len, gint *pos,
-					 gsize poslen, gchar delim)
+rspamd_spf_split_elt(const char *val, gsize len, int *pos,
+					 gsize poslen, char delim)
 {
-	const gchar *p, *end;
-	guint cur_pos = 0, cur_st = 0, nsub = 0;
+	const char *p, *end;
+	unsigned int cur_pos = 0, cur_st = 0, nsub = 0;
 
 	p = val;
 	end = val + len;
@@ -1781,14 +1833,14 @@ rspamd_spf_split_elt(const gchar *val, gsize len, gint *pos,
 }
 
 static gsize
-rspamd_spf_process_substitution(const gchar *macro_value,
-								gsize macro_len, guint ndelim, gchar delim, gboolean reversed,
-								gchar *dest)
+rspamd_spf_process_substitution(const char *macro_value,
+								gsize macro_len, unsigned int ndelim, char delim, gboolean reversed,
+								char *dest)
 {
-	gchar *d = dest;
-	const gchar canon_delim = '.';
-	guint vlen, i;
-	gint pos[49 * 2], tlen;
+	char *d = dest;
+	const char canon_delim = '.';
+	unsigned int vlen, i;
+	int pos[49 * 2], tlen;
 
 	if (!reversed && ndelim == 0 && delim == canon_delim) {
 		/* Trivial case */
@@ -1843,15 +1895,15 @@ rspamd_spf_process_substitution(const gchar *macro_value,
 	return (d - dest);
 }
 
-static const gchar *
+static const char *
 expand_spf_macro(struct spf_record *rec, struct spf_resolved_element *resolved,
-				 const gchar *begin)
+				 const char *begin)
 {
-	const gchar *p, *macro_value = NULL;
-	gchar *c, *new, *tmp, delim = '.';
+	const char *p, *macro_value = NULL;
+	char *c, *new, *tmp, delim = '.';
 	gsize len = 0, macro_len = 0;
-	gint state = 0, ndelim = 0;
-	gchar ip_buf[64 + 1]; /* cannot use INET6_ADDRSTRLEN as we use ptr lookup */
+	int state = 0, ndelim = 0;
+	char ip_buf[64 + 1]; /* cannot use INET6_ADDRSTRLEN as we use ptr lookup */
 	gboolean need_expand = FALSE, reversed;
 	struct rspamd_task *task;
 
@@ -2222,13 +2274,13 @@ expand_spf_macro(struct spf_record *rec, struct spf_resolved_element *resolved,
 static gboolean
 spf_process_element(struct spf_record *rec,
 					struct spf_resolved_element *resolved,
-					const gchar *elt,
-					const gchar **elts)
+					const char *elt,
+					const char **elts)
 {
 	struct spf_addr *addr = NULL;
 	gboolean res = FALSE;
-	const gchar *begin;
-	gchar t;
+	const char *begin;
+	char t;
 
 	g_assert(elt != NULL);
 	g_assert(rec != NULL);
@@ -2329,7 +2381,7 @@ spf_process_element(struct spf_record *rec,
 				 */
 			gboolean ignore_redirect = FALSE;
 
-			for (const gchar **tmp = elts; *tmp != NULL; tmp++) {
+			for (const char **tmp = elts; *tmp != NULL; tmp++) {
 				if (g_ascii_strcasecmp((*tmp) + 1, "all") == 0) {
 					ignore_redirect = TRUE;
 					break;
@@ -2378,7 +2430,7 @@ spf_process_element(struct spf_record *rec,
 }
 
 static void
-parse_spf_scopes(struct spf_record *rec, gchar **begin)
+parse_spf_scopes(struct spf_record *rec, char **begin)
 {
 	for (;;) {
 		if (g_ascii_strncasecmp(*begin, SPF_SCOPE_PRA, sizeof(SPF_SCOPE_PRA) - 1) == 0) {
@@ -2402,9 +2454,9 @@ parse_spf_scopes(struct spf_record *rec, gchar **begin)
 
 static gboolean
 start_spf_parse(struct spf_record *rec, struct spf_resolved_element *resolved,
-				gchar *begin)
+				char *begin)
 {
-	gchar **elts, **cur_elt;
+	char **elts, **cur_elt;
 	gsize len;
 
 	/* Skip spaces */
@@ -2439,7 +2491,7 @@ start_spf_parse(struct spf_record *rec, struct spf_resolved_element *resolved,
 		msg_debug_spf(
 			"spf error for domain %s: bad spf record start: %*s",
 			rec->sender_domain,
-			(gint) len,
+			(int) len,
 			begin);
 
 		return FALSE;
@@ -2455,7 +2507,7 @@ start_spf_parse(struct spf_record *rec, struct spf_resolved_element *resolved,
 		cur_elt = elts;
 
 		while (*cur_elt) {
-			spf_process_element(rec, resolved, *cur_elt, (const gchar **) elts);
+			spf_process_element(rec, resolved, *cur_elt, (const char **) elts);
 			cur_elt++;
 		}
 
@@ -2609,10 +2661,10 @@ rspamd_spf_get_cred(struct rspamd_task *task)
 	return cred;
 }
 
-const gchar *
+const char *
 rspamd_spf_get_domain(struct rspamd_task *task)
 {
-	gchar *domain = NULL;
+	char *domain = NULL;
 	struct rspamd_spf_cred *cred;
 
 	cred = rspamd_spf_get_cred(task);
@@ -2686,22 +2738,22 @@ rspamd_spf_resolve(struct rspamd_task *task, spf_cb_t callback,
 }
 
 struct spf_resolved *
-_spf_record_ref(struct spf_resolved *flat, const gchar *loc)
+spf_record_ref_internal(struct spf_resolved *flat, const char *loc)
 {
 	REF_RETAIN(flat);
 	return flat;
 }
 
-void _spf_record_unref(struct spf_resolved *flat, const gchar *loc)
+void spf_record_unref_internal(struct spf_resolved *flat, const char *loc)
 {
 	REF_RELEASE(flat);
 }
 
-gchar *
+char *
 spf_addr_mask_to_string(struct spf_addr *addr)
 {
 	GString *res;
-	gchar *s, ipstr[INET6_ADDRSTRLEN + 1];
+	char *s, ipstr[INET6_ADDRSTRLEN + 1];
 
 	if (addr->flags & RSPAMD_SPF_FLAG_ANY) {
 		res = g_string_new("any");
@@ -2731,10 +2783,10 @@ spf_addr_mask_to_string(struct spf_addr *addr)
 struct spf_addr *
 spf_addr_match_task(struct rspamd_task *task, struct spf_resolved *rec)
 {
-	const guint8 *s, *d;
-	guint af, mask, bmask, addrlen;
+	const uint8_t *s, *d;
+	unsigned int af, mask, bmask, addrlen;
 	struct spf_addr *selected = NULL, *addr, *any_addr = NULL;
-	guint i;
+	unsigned int i;
 
 	if (task->from_addr == NULL) {
 		return FALSE;
@@ -2753,11 +2805,11 @@ spf_addr_match_task(struct rspamd_task *task, struct spf_resolved *rec)
 			d = rspamd_inet_address_get_hash_key(task->from_addr, &addrlen);
 
 			if (af == AF_INET6) {
-				s = (const guint8 *) addr->addr6;
+				s = (const uint8_t *) addr->addr6;
 				mask = addr->m.dual.mask_v6;
 			}
 			else {
-				s = (const guint8 *) addr->addr4;
+				s = (const uint8_t *) addr->addr4;
 				mask = addr->m.dual.mask_v4;
 			}
 
