@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -28,23 +28,23 @@
 
 struct rspamd_stat_sqlite3_db {
 	sqlite3 *sqlite;
-	gchar *fname;
+	char *fname;
 	GArray *prstmt;
 	lua_State *L;
 	rspamd_mempool_t *pool;
 	gboolean in_transaction;
 	gboolean enable_users;
 	gboolean enable_languages;
-	gint cbref_user;
-	gint cbref_language;
+	int cbref_user;
+	int cbref_language;
 };
 
 struct rspamd_stat_sqlite3_rt {
 	struct rspamd_task *task;
 	struct rspamd_stat_sqlite3_db *db;
 	struct rspamd_statfile_config *cf;
-	gint64 user_id;
-	gint64 lang_id;
+	int64_t user_id;
+	int64_t lang_id;
 };
 
 static const char *create_tables_sql =
@@ -156,13 +156,13 @@ rspamd_sqlite3_backend_quark(void)
 	return g_quark_from_static_string("sqlite3-stat-backend");
 }
 
-static gint64
+static int64_t
 rspamd_sqlite3_get_user(struct rspamd_stat_sqlite3_db *db,
 						struct rspamd_task *task, gboolean learn)
 {
-	gint64 id = 0; /* Default user is 0 */
-	gint rc, err_idx;
-	const gchar *user = NULL;
+	int64_t id = 0; /* Default user is 0 */
+	int rc, err_idx;
+	const char *user = NULL;
 	struct rspamd_task **ptask;
 	lua_State *L = db->L;
 
@@ -177,7 +177,7 @@ rspamd_sqlite3_get_user(struct rspamd_stat_sqlite3_db *db,
 		lua_rawgeti(L, LUA_REGISTRYINDEX, db->cbref_user);
 		ptask = lua_newuserdata(L, sizeof(struct rspamd_task *));
 		*ptask = task;
-		rspamd_lua_setclass(L, "rspamd{task}", -1);
+		rspamd_lua_setclass(L, rspamd_task_classname, -1);
 
 		if (lua_pcall(L, 1, 1, err_idx) != 0) {
 			msg_err_task("call to user extraction script failed: %s",
@@ -215,14 +215,14 @@ rspamd_sqlite3_get_user(struct rspamd_stat_sqlite3_db *db,
 	return id;
 }
 
-static gint64
+static int64_t
 rspamd_sqlite3_get_language(struct rspamd_stat_sqlite3_db *db,
 							struct rspamd_task *task, gboolean learn)
 {
-	gint64 id = 0; /* Default language is 0 */
-	gint rc, err_idx;
-	guint i;
-	const gchar *language = NULL;
+	int64_t id = 0; /* Default language is 0 */
+	int rc, err_idx;
+	unsigned int i;
+	const char *language = NULL;
 	struct rspamd_mime_text_part *tp;
 	struct rspamd_task **ptask;
 	lua_State *L = db->L;
@@ -246,7 +246,7 @@ rspamd_sqlite3_get_language(struct rspamd_stat_sqlite3_db *db,
 		lua_rawgeti(L, LUA_REGISTRYINDEX, db->cbref_language);
 		ptask = lua_newuserdata(L, sizeof(struct rspamd_task *));
 		*ptask = task;
-		rspamd_lua_setclass(L, "rspamd{task}", -1);
+		rspamd_lua_setclass(L, rspamd_task_classname, -1);
 
 		if (lua_pcall(L, 1, 1, err_idx) != 0) {
 			msg_err_task("call to language extraction script failed: %s",
@@ -286,17 +286,17 @@ rspamd_sqlite3_get_language(struct rspamd_stat_sqlite3_db *db,
 static struct rspamd_stat_sqlite3_db *
 rspamd_sqlite3_opendb(rspamd_mempool_t *pool,
 					  struct rspamd_statfile_config *stcf,
-					  const gchar *path, const ucl_object_t *opts,
+					  const char *path, const ucl_object_t *opts,
 					  gboolean create, GError **err)
 {
 	struct rspamd_stat_sqlite3_db *bk;
 	struct rspamd_stat_tokenizer *tokenizer;
 	gpointer tk_conf;
 	gsize sz = 0;
-	gint64 sz64 = 0;
-	gchar *tok_conf_encoded;
-	gint ret, ntries = 0;
-	const gint max_tries = 100;
+	int64_t sz64 = 0;
+	char *tok_conf_encoded;
+	int ret, ntries = 0;
+	const int max_tries = 100;
 	struct timespec sleep_ts = {
 		.tv_sec = 0,
 		.tv_nsec = 1000000};
@@ -347,7 +347,7 @@ rspamd_sqlite3_opendb(rspamd_mempool_t *pool,
 
 		if (rspamd_sqlite3_run_prstmt(pool, bk->sqlite, bk->prstmt,
 									  RSPAMD_STAT_BACKEND_SAVE_TOKENIZER,
-									  (gint64) strlen(tok_conf_encoded),
+									  (int64_t) strlen(tok_conf_encoded),
 									  tok_conf_encoded) != SQLITE_OK) {
 			sqlite3_close(bk->sqlite);
 			g_free(bk);
@@ -375,7 +375,7 @@ rspamd_sqlite3_init(struct rspamd_stat_ctx *ctx,
 	struct rspamd_classifier_config *clf = st->classifier->cfg;
 	struct rspamd_statfile_config *stf = st->stcf;
 	const ucl_object_t *filenameo, *lang_enabled, *users_enabled;
-	const gchar *filename, *lua_script;
+	const char *filename, *lua_script;
 	struct rspamd_stat_sqlite3_db *bk;
 	GError *err = NULL;
 
@@ -502,7 +502,7 @@ void rspamd_sqlite3_close(gpointer p)
 
 gpointer
 rspamd_sqlite3_runtime(struct rspamd_task *task,
-					   struct rspamd_statfile_config *stcf, gboolean learn, gpointer p, gint _id)
+					   struct rspamd_statfile_config *stcf, gboolean learn, gpointer p, int _id)
 {
 	struct rspamd_stat_sqlite3_rt *rt = NULL;
 	struct rspamd_stat_sqlite3_db *bk = p;
@@ -522,12 +522,12 @@ rspamd_sqlite3_runtime(struct rspamd_task *task,
 gboolean
 rspamd_sqlite3_process_tokens(struct rspamd_task *task,
 							  GPtrArray *tokens,
-							  gint id, gpointer p)
+							  int id, gpointer p)
 {
 	struct rspamd_stat_sqlite3_db *bk;
 	struct rspamd_stat_sqlite3_rt *rt = p;
-	gint64 iv = 0;
-	guint i;
+	int64_t iv = 0;
+	unsigned int i;
 	rspamd_token_t *tok;
 
 	g_assert(p != NULL);
@@ -625,12 +625,12 @@ rspamd_sqlite3_finalize_process(struct rspamd_task *task, gpointer runtime,
 
 gboolean
 rspamd_sqlite3_learn_tokens(struct rspamd_task *task, GPtrArray *tokens,
-							gint id, gpointer p)
+							int id, gpointer p)
 {
 	struct rspamd_stat_sqlite3_db *bk;
 	struct rspamd_stat_sqlite3_rt *rt = p;
-	gint64 iv = 0;
-	guint i;
+	int64_t iv = 0;
+	unsigned int i;
 	rspamd_token_t *tok;
 
 	g_assert(tokens != NULL);
@@ -691,7 +691,7 @@ rspamd_sqlite3_finalize_learn(struct rspamd_task *task, gpointer runtime,
 {
 	struct rspamd_stat_sqlite3_rt *rt = runtime;
 	struct rspamd_stat_sqlite3_db *bk;
-	gint wal_frames, wal_checkpointed, mode;
+	int wal_frames, wal_checkpointed, mode;
 
 	g_assert(rt != NULL);
 	bk = rt->db;
@@ -735,7 +735,7 @@ rspamd_sqlite3_total_learns(struct rspamd_task *task, gpointer runtime,
 {
 	struct rspamd_stat_sqlite3_rt *rt = runtime;
 	struct rspamd_stat_sqlite3_db *bk;
-	guint64 res;
+	uint64_t res;
 
 	g_assert(rt != NULL);
 	bk = rt->db;
@@ -751,7 +751,7 @@ rspamd_sqlite3_inc_learns(struct rspamd_task *task, gpointer runtime,
 {
 	struct rspamd_stat_sqlite3_rt *rt = runtime;
 	struct rspamd_stat_sqlite3_db *bk;
-	guint64 res;
+	uint64_t res;
 
 	g_assert(rt != NULL);
 	bk = rt->db;
@@ -780,7 +780,7 @@ rspamd_sqlite3_dec_learns(struct rspamd_task *task, gpointer runtime,
 {
 	struct rspamd_stat_sqlite3_rt *rt = runtime;
 	struct rspamd_stat_sqlite3_db *bk;
-	guint64 res;
+	uint64_t res;
 
 	g_assert(rt != NULL);
 	bk = rt->db;
@@ -809,7 +809,7 @@ rspamd_sqlite3_learns(struct rspamd_task *task, gpointer runtime,
 {
 	struct rspamd_stat_sqlite3_rt *rt = runtime;
 	struct rspamd_stat_sqlite3_db *bk;
-	guint64 res;
+	uint64_t res;
 
 	g_assert(rt != NULL);
 	bk = rt->db;
@@ -828,7 +828,7 @@ rspamd_sqlite3_get_stat(gpointer runtime,
 	struct rspamd_stat_sqlite3_db *bk;
 	rspamd_mempool_t *pool;
 	struct stat st;
-	gint64 rev;
+	int64_t rev;
 
 	g_assert(rt != NULL);
 	bk = rt->db;
@@ -873,7 +873,7 @@ rspamd_sqlite3_load_tokenizer_config(gpointer runtime,
 									 gsize *len)
 {
 	gpointer tk_conf, copied_conf;
-	guint64 sz;
+	uint64_t sz;
 	struct rspamd_stat_sqlite3_rt *rt = runtime;
 	struct rspamd_stat_sqlite3_db *bk;
 
