@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -50,16 +50,17 @@ typedef struct rspamd_stat_token_s {
 	rspamd_ftok_unicode_t unicode; /* array of unicode characters, normalized, lowercased */
 	rspamd_ftok_t normalized;      /* normalized and lowercased utf8 */
 	rspamd_ftok_t stemmed;         /* stemmed utf8 */
-	guint flags;
+	unsigned int flags;
 } rspamd_stat_token_t;
 
+#define RSPAMD_TOKEN_VALUE_TYPE float
 typedef struct token_node_s {
-	guint64 data;
-	guint window_idx;
-	guint flags;
+	uint64_t data;
+	unsigned int window_idx;
+	unsigned int flags;
 	rspamd_stat_token_t *t1;
 	rspamd_stat_token_t *t2;
-	float values[];
+	RSPAMD_TOKEN_VALUE_TYPE values[0];
 } rspamd_token_t;
 
 struct rspamd_stat_ctx;
@@ -103,7 +104,7 @@ void rspamd_stat_process_tokenize(struct rspamd_stat_ctx *st_ctx,
  * @return TRUE if task has been classified
  */
 rspamd_stat_result_t rspamd_stat_classify(struct rspamd_task *task,
-										  lua_State *L, guint stage, GError **err);
+										  lua_State *L, unsigned int stage, GError **err);
 
 
 /**
@@ -123,8 +124,8 @@ gboolean rspamd_stat_check_autolearn(struct rspamd_task *task);
  * @return TRUE if task has been learned
  */
 rspamd_stat_result_t rspamd_stat_learn(struct rspamd_task *task,
-									   gboolean spam, lua_State *L, const gchar *classifier,
-									   guint stage,
+									   gboolean spam, lua_State *L, const char *classifier,
+									   unsigned int stage,
 									   GError **err);
 
 /**
@@ -135,7 +136,7 @@ rspamd_stat_result_t rspamd_stat_learn(struct rspamd_task *task,
  */
 rspamd_stat_result_t rspamd_stat_statistics(struct rspamd_task *task,
 											struct rspamd_config *cfg,
-											guint64 *total_learns,
+											uint64_t *total_learns,
 											ucl_object_t **res);
 
 void rspamd_stat_unload(void);
