@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2024 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -23,7 +23,7 @@
 #include "libmime/lang_detection.h"
 
 /* Size for features pipe */
-#define DEFAULT_FEATURE_WINDOW_SIZE 5
+#define DEFAULT_FEATURE_WINDOW_SIZE 2
 #define DEFAULT_OSB_VERSION 2
 
 static const int primes[] = {
@@ -49,7 +49,7 @@ static const int primes[] = {
 	3277,
 };
 
-static const guchar osb_tokenizer_magic[] = {'o', 's', 'b', 't', 'o', 'k', 'v', '2'};
+static const unsigned char osb_tokenizer_magic[] = {'o', 's', 'b', 't', 'o', 'k', 'v', '2'};
 
 enum rspamd_osb_hash_type {
 	RSPAMD_OSB_HASH_COMPAT = 0,
@@ -58,11 +58,11 @@ enum rspamd_osb_hash_type {
 };
 
 struct rspamd_osb_tokenizer_config {
-	guchar magic[8];
+	unsigned char magic[8];
 	gshort version;
 	gshort window_size;
 	enum rspamd_osb_hash_type ht;
-	guint64 seed;
+	uint64_t seed;
 	rspamd_sipkey_t sk;
 };
 
@@ -92,7 +92,7 @@ rspamd_tokenizer_osb_config_from_ucl(rspamd_mempool_t *pool,
 {
 	const ucl_object_t *elt;
 	struct rspamd_osb_tokenizer_config *cf, *def;
-	guchar *key = NULL;
+	unsigned char *key = NULL;
 	gsize keylen;
 
 
@@ -262,25 +262,25 @@ rspamd_tokenizer_osb_is_compat (struct rspamd_tokenizer_runtime *rt)
 #endif
 
 struct token_pipe_entry {
-	guint64 h;
+	uint64_t h;
 	rspamd_stat_token_t *t;
 };
 
-gint rspamd_tokenizer_osb(struct rspamd_stat_ctx *ctx,
-						  struct rspamd_task *task,
-						  GArray *words,
-						  gboolean is_utf,
-						  const gchar *prefix,
-						  GPtrArray *result)
+int rspamd_tokenizer_osb(struct rspamd_stat_ctx *ctx,
+						 struct rspamd_task *task,
+						 GArray *words,
+						 gboolean is_utf,
+						 const char *prefix,
+						 GPtrArray *result)
 {
 	rspamd_token_t *new_tok = NULL;
 	rspamd_stat_token_t *token;
 	struct rspamd_osb_tokenizer_config *osb_cf;
-	guint64 cur, seed;
+	uint64_t cur, seed;
 	struct token_pipe_entry *hashpipe;
-	guint32 h1, h2;
+	uint32_t h1, h2;
 	gsize token_size;
-	guint processed = 0, i, w, window_size, token_flags = 0;
+	unsigned int processed = 0, i, w, window_size, token_flags = 0;
 
 	if (words == NULL) {
 		return FALSE;
@@ -302,15 +302,14 @@ gint rspamd_tokenizer_osb(struct rspamd_stat_ctx *ctx,
 		hashpipe[i].h = 0xfe;
 		hashpipe[i].t = NULL;
 	}
-
 	token_size = sizeof(rspamd_token_t) +
-				 sizeof(gdouble) * ctx->statfiles->len;
+				 sizeof(RSPAMD_TOKEN_VALUE_TYPE) * ctx->statfiles->len;
 	g_assert(token_size > 0);
 
 	for (w = 0; w < words->len; w++) {
 		token = &g_array_index(words, rspamd_stat_token_t, w);
 		token_flags = token->flags;
-		const gchar *begin;
+		const char *begin;
 		gsize len;
 
 		if (token->flags &
@@ -342,7 +341,7 @@ gint rspamd_tokenizer_osb(struct rspamd_stat_ctx *ctx,
 														  begin, len, osb_cf->seed);
 			}
 			else {
-				rspamd_cryptobox_siphash((guchar *) &cur, begin,
+				rspamd_cryptobox_siphash((unsigned char *) &cur, begin,
 										 len, osb_cf->sk);
 
 				if (prefix) {
@@ -370,12 +369,12 @@ gint rspamd_tokenizer_osb(struct rspamd_stat_ctx *ctx,
 		new_tok->t1 = hashpipe[0].t;                                                    \
 		new_tok->t2 = hashpipe[i].t;                                                    \
 		if (osb_cf->ht == RSPAMD_OSB_HASH_COMPAT) {                                     \
-			h1 = ((guint32) hashpipe[0].h) * primes[0] +                                \
-				 ((guint32) hashpipe[i].h) * primes[i << 1];                            \
-			h2 = ((guint32) hashpipe[0].h) * primes[1] +                                \
-				 ((guint32) hashpipe[i].h) * primes[(i << 1) - 1];                      \
-			memcpy((guchar *) &new_tok->data, &h1, sizeof(h1));                         \
-			memcpy(((guchar *) &new_tok->data) + sizeof(h1), &h2, sizeof(h2));          \
+			h1 = ((uint32_t) hashpipe[0].h) * primes[0] +                               \
+				 ((uint32_t) hashpipe[i].h) * primes[i << 1];                           \
+			h2 = ((uint32_t) hashpipe[0].h) * primes[1] +                               \
+				 ((uint32_t) hashpipe[i].h) * primes[(i << 1) - 1];                     \
+			memcpy((unsigned char *) &new_tok->data, &h1, sizeof(h1));                  \
+			memcpy(((unsigned char *) &new_tok->data) + sizeof(h1), &h2, sizeof(h2));   \
 		}                                                                               \
 		else {                                                                          \
 			new_tok->data = hashpipe[0].h * primes[0] + hashpipe[i].h * primes[i << 1]; \
