@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 #include "unix-std.h"
 #include "zlib.h"
 #include "utlist.h"
+#include "libserver/protocol_internal.h"
 
 /***
  * @module rspamd_http
@@ -28,27 +29,128 @@
  * This module hides all complexity: DNS resolving, sessions management, zero-copy
  * text transfers and so on under the hood.
  * @example
+-- Basic GET request with callback
 local rspamd_http = require "rspamd_http"
 
 local function symbol_callback(task)
 	local function http_callback(err_message, code, body, headers)
 		task:insert_result('SYMBOL', 1) -- task is available via closure
+
+		if err_message then
+			-- Handle error
+			return
+		end
+
+		-- Process response
+		if code == 200 then
+			-- Process body and headers
+			for name, value in pairs(headers) do
+				-- Headers are lowercase
+			end
+		end
 	end
 
- 	rspamd_http.request({
- 		task=task,
- 		url='http://example.com/data',
- 		body=task:get_content(),
- 		callback=http_callback,
- 		headers={Header='Value', OtherHeader='Value'},
- 		mime_type='text/plain',
- 		})
- end
+	rspamd_http.request({
+		task=task,
+		url='http://example.com/data',
+		body=task:get_content(),
+		callback=http_callback,
+		headers={Header='Value', OtherHeader='Value', DuplicatedHeader={'Multiple', 'Values'}},
+		mime_type='text/plain',
+	})
+end
+
+-- POST request with JSON body
+local function post_json_example(task)
+	local ucl = require "ucl"
+	local data = {
+		id = task:get_queue_id(),
+		sender = task:get_from()[1].addr
+	}
+
+	local json_data = ucl.to_json(data)
+
+	rspamd_http.request({
+		task = task,
+		url = "http://example.com/api/submit",
+		method = "POST",
+		body = json_data,
+		headers = {['Content-Type'] = 'application/json'},
+		callback = function(err, code, body, headers)
+			if not err and code == 200 then
+				-- Success
+			end
+		end
+	})
+end
+
+-- Synchronous HTTP request (using coroutines)
+local function sync_http_example(task)
+	-- No callback makes this a synchronous call
+	local err, response = rspamd_http.request({
+		task = task,
+		url = "http://example.com/api/data",
+		method = "GET",
+		timeout = 10.0
+	})
+
+	if not err then
+		-- Response is a table with code, content, and headers
+		if response.code == 200 then
+			-- Process response.content
+			return true
+		end
+	end
+	return false
+end
+
+-- Using authentication
+local function auth_example(task)
+	rspamd_http.request({
+		task = task,
+		url = "https://example.com/api/protected",
+		method = "GET",
+		user = "username",
+		password = "secret",
+		callback = function(err, code, body, headers)
+			-- Process authenticated response
+		end
+	})
+end
+
+-- Using HTTPS with SSL options
+local function https_example(task)
+	rspamd_http.request({
+		task = task,
+		url = "https://example.com/api/secure",
+		method = "GET",
+		no_ssl_verify = false, -- Verify SSL (default)
+		callback = function(err, code, body, headers)
+			-- Process secure response
+		end
+	})
+end
+
+-- Using keep-alive and gzip
+local function advanced_example(task)
+	rspamd_http.request({
+		task = task,
+		url = "http://example.com/api/data",
+		method = "POST",
+		body = task:get_content(),
+		gzip = true, -- Compress request body
+		keepalive = true, -- Use keep-alive connection
+		max_size = 1024 * 1024, -- Limit response to 1MB
+		callback = function(err, code, body, headers)
+			-- Process response
+		end
+	})
+end
  */
 
 #define MAX_HEADERS_SIZE 8192
 
-static const gchar *M = "rspamd lua http";
+static const char *M = "rspamd lua http";
 
 LUA_FUNCTION_DEF(http, request);
 
@@ -75,20 +177,20 @@ struct lua_http_cbdata {
 	struct rspamd_cryptobox_keypair *local_kp;
 	struct rspamd_cryptobox_pubkey *peer_pk;
 	rspamd_inet_addr_t *addr;
-	gchar *mime_type;
-	gchar *host;
-	gchar *auth;
+	char *mime_type;
+	char *host;
+	char *auth;
 	struct upstream *up;
-	const gchar *url;
+	const char *url;
 	gsize max_size;
-	gint flags;
-	gint fd;
-	gint cbref;
+	int flags;
+	int fd;
+	int cbref;
 	struct thread_entry *thread;
 	ref_entry_t ref;
 };
 
-static const gdouble default_http_timeout = 5.0;
+static const double default_http_timeout = 5.0;
 
 static struct rspamd_dns_resolver *
 lua_http_global_resolver(struct ev_loop *ev_base)
@@ -237,7 +339,7 @@ lua_http_finish_handler(struct rspamd_http_connection *conn,
 {
 	struct lua_http_cbdata *cbd = (struct lua_http_cbdata *) conn->ud;
 	struct rspamd_http_header *h;
-	const gchar *body;
+	const char *body;
 	gsize body_len;
 
 	struct lua_callback_state lcbd;
@@ -278,7 +380,7 @@ lua_http_finish_handler(struct rspamd_http_connection *conn,
 		struct rspamd_lua_text *t;
 
 		t = lua_newuserdata(L, sizeof(*t));
-		rspamd_lua_setclass(L, "rspamd{text}", -1);
+		rspamd_lua_setclass(L, rspamd_text_classname, -1);
 		t->start = body;
 		t->len = body_len;
 		t->flags = 0;
@@ -330,7 +432,7 @@ lua_http_resume_handler(struct rspamd_http_connection *conn,
 {
 	struct lua_http_cbdata *cbd = (struct lua_http_cbdata *) conn->ud;
 	lua_State *L = cbd->thread->lua_state;
-	const gchar *body;
+	const char *body;
 	gsize body_len;
 	struct rspamd_http_header *h;
 
@@ -363,7 +465,7 @@ lua_http_resume_handler(struct rspamd_http_connection *conn,
 			struct rspamd_lua_text *t;
 
 			t = lua_newuserdata(L, sizeof(*t));
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
+			rspamd_lua_setclass(L, rspamd_text_classname, -1);
 			t->start = body;
 			t->len = body_len;
 			t->flags = 0;
@@ -557,7 +659,7 @@ static void
 lua_http_push_headers(lua_State *L, struct rspamd_http_message *msg)
 {
 	const char *name, *value;
-	gint i, sz;
+	int i, sz;
 
 	lua_pushnil(L);
 	while (lua_next(L, -2) != 0) {
@@ -599,9 +701,9 @@ lua_http_push_headers(lua_State *L, struct rspamd_http_message *msg)
  * - `config`
  *
  * @param {string} url specifies URL for a request in the standard URI form (e.g. 'http://example.com/path')
- * @param {function} callback specifies callback function in format  `function (err_message, code, body, headers)` that is called on HTTP request completion. if this parameter is missing, the function performs "pseudo-synchronous" call (see [Synchronous and Asynchronous API overview](/doc/lua/sync_async.html#API-example-http-module)
+ * @param {function} callback specifies callback function in format  `function (err_message, code, body, headers)` that is called on HTTP request completion. if this parameter is missing, the function performs "pseudo-synchronous" call (see [Synchronous and Asynchronous API overview](/doc/developers/sync_async.html#API-example-http-module)
  * @param {task} task if called from symbol handler it is generally a good idea to use the common task objects: event base, DNS resolver and events session
- * @param {table} headers optional headers in form `[name='value', name='value']`
+ * @param {table} headers optional headers in form `[name='value']` or `[name=['value1', 'value2']]` to duplicate a header with multiple values
  * @param {string} mime_type MIME type of the HTTP content (for example, `text/html`)
  * @param {string/text} body full body content, can be opaque `rspamd{text}` to avoid data copying
  * @param {number} timeout floating point request timeout value in seconds (default is 5.0 seconds)
@@ -614,7 +716,7 @@ lua_http_push_headers(lua_State *L, struct rspamd_http_message *msg)
  * @return {boolean} `true`, in **async** mode, if a request has been successfully scheduled. If this value is `false` then some error occurred, the callback thus will not be called.
  * @return In **sync** mode `string|nil, nil|table` In sync mode  error message if any and response as table: `int` _code_, `string` _content_ and `table` _headers_ (header -> value)
  */
-static gint
+static int
 lua_http_request(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -629,14 +731,14 @@ lua_http_request(lua_State *L)
 	struct rspamd_cryptobox_pubkey *peer_key = NULL;
 	struct rspamd_cryptobox_keypair *local_kp = NULL;
 	struct upstream *up = NULL;
-	const gchar *url, *lua_body;
+	const char *url, *lua_body;
 	rspamd_fstring_t *body = NULL;
-	gint cbref = -1;
+	int cbref = -1;
 	gsize bodylen;
-	gdouble timeout = default_http_timeout;
-	gint flags = 0;
-	gchar *mime_type = NULL;
-	gchar *auth = NULL;
+	double timeout = default_http_timeout;
+	int flags = 0;
+	char *mime_type = NULL;
+	char *auth = NULL;
 	gsize max_size = 0;
 	gboolean gzip = FALSE;
 
@@ -653,21 +755,21 @@ lua_http_request(lua_State *L)
 		lua_pushvalue(L, 2);
 		cbref = luaL_ref(L, LUA_REGISTRYINDEX);
 
-		if (lua_gettop(L) >= 3 && rspamd_lua_check_udata_maybe(L, 3, "rspamd{ev_base}")) {
+		if (lua_gettop(L) >= 3 && rspamd_lua_check_udata_maybe(L, 3, rspamd_ev_base_classname)) {
 			ev_base = *(struct ev_loop **) lua_touserdata(L, 3);
 		}
 		else {
 			ev_base = NULL;
 		}
 
-		if (lua_gettop(L) >= 4 && rspamd_lua_check_udata_maybe(L, 4, "rspamd{resolver}")) {
+		if (lua_gettop(L) >= 4 && rspamd_lua_check_udata_maybe(L, 4, rspamd_resolver_classname)) {
 			resolver = *(struct rspamd_dns_resolver **) lua_touserdata(L, 4);
 		}
 		else {
 			resolver = lua_http_global_resolver(ev_base);
 		}
 
-		if (lua_gettop(L) >= 5 && rspamd_lua_check_udata_maybe(L, 5, "rspamd{session}")) {
+		if (lua_gettop(L) >= 5 && rspamd_lua_check_udata_maybe(L, 5, rspamd_session_classname)) {
 			session = *(struct rspamd_async_session **) lua_touserdata(L, 5);
 		}
 		else {
@@ -722,7 +824,7 @@ lua_http_request(lua_State *L)
 		if (task == NULL) {
 			lua_pushstring(L, "ev_base");
 			lua_gettable(L, 1);
-			if (rspamd_lua_check_udata_maybe(L, -1, "rspamd{ev_base}")) {
+			if (rspamd_lua_check_udata_maybe(L, -1, rspamd_ev_base_classname)) {
 				ev_base = *(struct ev_loop **) lua_touserdata(L, -1);
 			}
 			else {
@@ -733,7 +835,7 @@ lua_http_request(lua_State *L)
 
 			lua_pushstring(L, "session");
 			lua_gettable(L, 1);
-			if (rspamd_lua_check_udata_maybe(L, -1, "rspamd{session}")) {
+			if (rspamd_lua_check_udata_maybe(L, -1, rspamd_session_classname)) {
 				session = *(struct rspamd_async_session **) lua_touserdata(L, -1);
 			}
 			else {
@@ -743,7 +845,7 @@ lua_http_request(lua_State *L)
 
 			lua_pushstring(L, "config");
 			lua_gettable(L, 1);
-			if (rspamd_lua_check_udata_maybe(L, -1, "rspamd{config}")) {
+			if (rspamd_lua_check_udata_maybe(L, -1, rspamd_config_classname)) {
 				cfg = *(struct rspamd_config **) lua_touserdata(L, -1);
 			}
 			else {
@@ -755,7 +857,7 @@ lua_http_request(lua_State *L)
 			lua_pushstring(L, "resolver");
 			lua_gettable(L, 1);
 
-			if (rspamd_lua_check_udata_maybe(L, -1, "rspamd{resolver}")) {
+			if (rspamd_lua_check_udata_maybe(L, -1, rspamd_resolver_classname)) {
 				resolver = *(struct rspamd_dns_resolver **) lua_touserdata(L, -1);
 			}
 			else {
@@ -903,12 +1005,11 @@ lua_http_request(lua_State *L)
 		lua_gettable(L, 1);
 
 		if (lua_type(L, -1) == LUA_TSTRING) {
-			const gchar *in;
+			const char *in;
 			gsize inlen;
 
 			in = lua_tolstring(L, -1, &inlen);
-			peer_key = rspamd_pubkey_from_base32(in, inlen,
-												 RSPAMD_KEYPAIR_KEX, RSPAMD_CRYPTOBOX_MODE_25519);
+			peer_key = rspamd_pubkey_from_base32(in, inlen, RSPAMD_KEYPAIR_KEX);
 		}
 
 		lua_pop(L, 1);
@@ -997,14 +1098,14 @@ lua_http_request(lua_State *L)
 		lua_gettable(L, 1);
 
 		if (lua_type(L, -1) == LUA_TSTRING) {
-			const gchar *user = lua_tostring(L, -1);
+			const char *user = lua_tostring(L, -1);
 
 			lua_pushstring(L, "password");
 			lua_gettable(L, 1);
 
 			if (lua_type(L, -1) == LUA_TSTRING) {
-				const gchar *password = lua_tostring(L, -1);
-				gchar *tmpbuf;
+				const char *password = lua_tostring(L, -1);
+				char *tmpbuf;
 				gsize tlen;
 
 				tlen = strlen(user) + strlen(password) + 1;
@@ -1108,7 +1209,7 @@ lua_http_request(lua_State *L)
 	if (body) {
 		if (gzip) {
 			if (rspamd_fstring_gzip(&body)) {
-				rspamd_http_message_add_header(msg, "Content-Encoding", "gzip");
+				rspamd_http_message_add_header(msg, CONTENT_ENCODING_HEADER, "gzip");
 			}
 		}
 
@@ -1124,7 +1225,7 @@ lua_http_request(lua_State *L)
 	/* Check if we can skip resolving */
 
 	gsize hostlen = 0;
-	const gchar *host = rspamd_http_message_get_http_host(msg, &hostlen);
+	const char *host = rspamd_http_message_get_http_host(msg, &hostlen);
 
 	if (host) {
 		cbd->host = g_malloc(hostlen + 1);
@@ -1255,7 +1356,7 @@ lua_http_request(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_load_http(lua_State *L)
 {
 	lua_newtable(L);
