@@ -1,11 +1,11 @@
-/*-
- * Copyright 2016 Vsevolod Stakhov
+/*
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- *   http://www.apache.org/licenses/LICENSE-2.0
+ *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -22,7 +22,7 @@
 
 #define REDIS_DEFAULT_TIMEOUT 1.0
 
-static const gchar *M = "rspamd lua redis";
+static const char *M = "rspamd lua redis";
 static void *redis_null;
 
 /***
@@ -101,18 +101,18 @@ struct lua_redis_userdata {
 	struct ev_loop *event_loop;
 	struct rspamd_config *cfg;
 	struct rspamd_redis_pool *pool;
-	gchar *server;
-	gchar log_tag[RSPAMD_LOG_ID_LEN + 1];
+	char *server;
+	char log_tag[RSPAMD_LOG_ID_LEN + 1];
 	struct lua_redis_request_specific_userdata *specific;
-	gdouble timeout;
-	guint16 port;
-	guint16 terminated;
+	ev_tstamp timeout;
+	uint16_t port;
+	uint16_t terminated;
 };
 
 #define msg_debug_lua_redis(...) rspamd_conditional_debug_fast(NULL, NULL,                                        \
-																 rspamd_lua_redis_log_id, "lua_redis", ud->log_tag, \
-																 G_STRFUNC,                                         \
-																 __VA_ARGS__)
+															   rspamd_lua_redis_log_id, "lua_redis", ud->log_tag, \
+															   G_STRFUNC,                                         \
+															   __VA_ARGS__)
 INIT_LOG_MODULE(lua_redis)
 
 #define LUA_REDIS_SPECIFIC_REPLIED (1 << 0)
@@ -126,21 +126,21 @@ INIT_LOG_MODULE(lua_redis)
 #define IS_ASYNC(ctx) ((ctx)->flags & LUA_REDIS_ASYNC)
 
 struct lua_redis_request_specific_userdata {
-	gint cbref;
-	guint nargs;
-	gchar **args;
+	int cbref;
+	unsigned int nargs;
+	char **args;
 	gsize *arglens;
-	struct lua_redis_userdata *c;
+	struct lua_redis_userdata *common_ud;
 	struct lua_redis_ctx *ctx;
 	struct lua_redis_request_specific_userdata *next;
 	ev_timer timeout_ev;
-	guint flags;
+	unsigned int flags;
 };
 
 struct lua_redis_ctx {
-	guint flags;
+	unsigned int flags;
 	struct lua_redis_userdata async;
-	guint cmds_pending;
+	unsigned int cmds_pending;
 	ref_entry_t ref;
 	GQueue *replies;             /* for sync connection only */
 	GQueue *events_cleanup;      /* for sync connection only */
@@ -149,7 +149,7 @@ struct lua_redis_ctx {
 
 struct lua_redis_result {
 	gboolean is_error;
-	gint result_ref;
+	int result_ref;
 	struct rspamd_symcache_dynamic_item *item;
 	struct rspamd_async_session *s;
 	struct rspamd_task *task;
@@ -157,17 +157,17 @@ struct lua_redis_result {
 };
 
 static struct lua_redis_ctx *
-lua_check_redis(lua_State *L, gint pos)
+lua_check_redis(lua_State *L, int pos)
 {
-	void *ud = rspamd_lua_check_udata(L, pos, "rspamd{redis}");
+	void *ud = rspamd_lua_check_udata(L, pos, rspamd_redis_classname);
 	luaL_argcheck(L, ud != NULL, pos, "'redis' expected");
 	return ud ? *((struct lua_redis_ctx **) ud) : NULL;
 }
 
 static void
-lua_redis_free_args(char **args, gsize *arglens, guint nargs)
+lua_redis_free_args(char **args, gsize *arglens, unsigned int nargs)
 {
-	guint i;
+	unsigned int i;
 
 	if (args) {
 		for (i = 0; i < nargs; i++) {
@@ -242,7 +242,7 @@ lua_redis_dtor(struct lua_redis_ctx *ctx)
 	g_free(ctx);
 }
 
-static gint
+static int
 lua_redis_gc(lua_State *L)
 {
 	struct lua_redis_ctx *ctx = lua_check_redis(L, 1);
@@ -262,7 +262,7 @@ lua_redis_fin(void *arg)
 	struct lua_redis_ctx *ctx;
 
 	ctx = sp_ud->ctx;
-	ud = sp_ud->c;
+	ud = sp_ud->common_ud;
 
 	if (ev_can_stop(&sp_ud->timeout_ev)) {
 		ev_timer_stop(sp_ud->ctx->async.event_loop, &sp_ud->timeout_ev);
@@ -280,16 +280,23 @@ lua_redis_fin(void *arg)
  * @param code
  * @param ud
  */
+#ifdef __GNUC__
+__attribute__((format(printf, 1, 5)))
+#endif
 static void
-lua_redis_push_error(const gchar *err,
+lua_redis_push_error(const char *err,
 					 struct lua_redis_ctx *ctx,
 					 struct lua_redis_request_specific_userdata *sp_ud,
-					 gboolean connected)
+					 gboolean connected,
+					 ...)
 {
-	struct lua_redis_userdata *ud = sp_ud->c;
+	struct lua_redis_userdata *ud = sp_ud->common_ud;
 	struct lua_callback_state cbs;
 	lua_State *L;
 
+	va_list ap;
+	va_start(ap, connected);
+
 	if (!(sp_ud->flags & (LUA_REDIS_SPECIFIC_REPLIED | LUA_REDIS_SPECIFIC_FINISHED))) {
 		if (sp_ud->cbref != -1) {
 
@@ -302,7 +309,7 @@ lua_redis_push_error(const gchar *err,
 			lua_rawgeti(cbs.L, LUA_REGISTRYINDEX, sp_ud->cbref);
 
 			/* String of error */
-			lua_pushstring(cbs.L, err);
+			lua_pushvfstring(cbs.L, err, ap);
 			/* Data is nil */
 			lua_pushnil(cbs.L);
 
@@ -331,12 +338,14 @@ lua_redis_push_error(const gchar *err,
 			lua_redis_fin(sp_ud);
 		}
 	}
+
+	va_end(ap);
 }
 
 static void
 lua_redis_push_reply(lua_State *L, const redisReply *r, gboolean text_data)
 {
-	guint i;
+	unsigned int i;
 	struct rspamd_lua_text *t;
 
 	switch (r->type) {
@@ -350,7 +359,7 @@ lua_redis_push_reply(lua_State *L, const redisReply *r, gboolean text_data)
 	case REDIS_REPLY_STATUS:
 		if (text_data) {
 			t = lua_newuserdata(L, sizeof(*t));
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
+			rspamd_lua_setclass(L, rspamd_text_classname, -1);
 			t->flags = 0;
 			t->start = r->str;
 			t->len = r->len;
@@ -381,7 +390,7 @@ static void
 lua_redis_push_data(const redisReply *r, struct lua_redis_ctx *ctx,
 					struct lua_redis_request_specific_userdata *sp_ud)
 {
-	struct lua_redis_userdata *ud = sp_ud->c;
+	struct lua_redis_userdata *ud = sp_ud->common_ud;
 	struct lua_callback_state cbs;
 	lua_State *L;
 
@@ -404,7 +413,7 @@ lua_redis_push_data(const redisReply *r, struct lua_redis_ctx *ctx,
 				rspamd_symcache_set_cur_item(ud->task, ud->item);
 			}
 
-			gint ret = lua_pcall(cbs.L, 2, 0, err_idx);
+			int ret = lua_pcall(cbs.L, 2, 0, err_idx);
 
 			if (ret != 0) {
 				msg_info("call to lua_redis callback failed (%d): %s",
@@ -458,14 +467,14 @@ lua_redis_callback(redisAsyncContext *c, gpointer r, gpointer priv)
 	redisAsyncContext *ac;
 
 	ctx = sp_ud->ctx;
-	ud = sp_ud->c;
+	ud = sp_ud->common_ud;
 
 	if (ud->terminated || !rspamd_lua_is_initialised()) {
 		/* We are already at the termination stage, just go out */
 		return;
 	}
 
-	msg_debug_lua_redis("got reply from redis %p for query %p", sp_ud->c->ctx,
+	msg_debug_lua_redis("got async reply from redis %p for query %p", sp_ud->common_ud->ctx,
 						sp_ud);
 
 	REDIS_RETAIN(ctx);
@@ -479,7 +488,7 @@ lua_redis_callback(redisAsyncContext *c, gpointer r, gpointer priv)
 					lua_redis_push_data(reply, ctx, sp_ud);
 				}
 				else {
-					lua_redis_push_error(reply->str, ctx, sp_ud, TRUE);
+					lua_redis_push_error("%s", ctx, sp_ud, TRUE, reply->str);
 				}
 			}
 			else {
@@ -488,10 +497,10 @@ lua_redis_callback(redisAsyncContext *c, gpointer r, gpointer priv)
 		}
 		else {
 			if (c->err == REDIS_ERR_IO) {
-				lua_redis_push_error(strerror(errno), ctx, sp_ud, TRUE);
+				lua_redis_push_error("%s", ctx, sp_ud, TRUE, strerror(errno));
 			}
 			else {
-				lua_redis_push_error(c->errstr, ctx, sp_ud, TRUE);
+				lua_redis_push_error("%s", ctx, sp_ud, TRUE, c->errstr);
 			}
 		}
 	}
@@ -517,11 +526,11 @@ lua_redis_callback(redisAsyncContext *c, gpointer r, gpointer priv)
 	REDIS_RELEASE(ctx);
 }
 
-static gint
+static int
 lua_redis_push_results(struct lua_redis_ctx *ctx, lua_State *L)
 {
-	gint results = g_queue_get_length(ctx->replies);
-	gint i;
+	int results = g_queue_get_length(ctx->replies);
+	int i;
 	gboolean can_use_lua = TRUE;
 
 	results = g_queue_get_length(ctx->replies);
@@ -589,10 +598,10 @@ lua_redis_callback_sync(redisAsyncContext *ac, gpointer r, gpointer priv)
 	struct lua_redis_ctx *ctx;
 	struct lua_redis_userdata *ud;
 	struct thread_entry *thread;
-	gint results;
+	int results;
 
 	ctx = sp_ud->ctx;
-	ud = sp_ud->c;
+	ud = sp_ud->common_ud;
 	lua_State *L = ctx->async.cfg->lua_state;
 
 	sp_ud->flags |= LUA_REDIS_SPECIFIC_REPLIED;
@@ -611,7 +620,7 @@ lua_redis_callback_sync(redisAsyncContext *ac, gpointer r, gpointer priv)
 	}
 
 	if (!(sp_ud->flags & LUA_REDIS_SPECIFIC_FINISHED)) {
-		msg_debug_lua_redis("got reply from redis: %p for query %p", ac, sp_ud);
+		msg_debug_lua_redis("got sync reply from redis: %p for query %p", ac, sp_ud);
 
 		struct lua_redis_result *result = g_malloc0(sizeof *result);
 
@@ -644,17 +653,17 @@ lua_redis_callback_sync(redisAsyncContext *ac, gpointer r, gpointer priv)
 		/* if error happened, we should terminate the connection,
 		   and release it */
 
-		if (result->is_error && sp_ud->c->ctx) {
-			ac = sp_ud->c->ctx;
+		if (result->is_error && sp_ud->common_ud->ctx) {
+			ac = sp_ud->common_ud->ctx;
 			/* Set to NULL to avoid double free in dtor */
-			sp_ud->c->ctx = NULL;
+			sp_ud->common_ud->ctx = NULL;
 			ctx->flags |= LUA_REDIS_TERMINATED;
 
 			/*
 			 * This will call all callbacks pending so the entire context
 			 * will be destructed
 			 */
-			rspamd_redis_pool_release_connection(sp_ud->c->pool, ac,
+			rspamd_redis_pool_release_connection(sp_ud->common_ud->pool, ac,
 												 RSPAMD_REDIS_RELEASE_FATAL);
 		}
 
@@ -670,6 +679,8 @@ lua_redis_callback_sync(redisAsyncContext *ac, gpointer r, gpointer priv)
 	ctx->cmds_pending--;
 
 	if (ctx->cmds_pending == 0) {
+		msg_debug_lua_redis("no more commands left for: %p for query %p", ac, sp_ud);
+
 		if (ctx->thread) {
 			if (!(sp_ud->flags & LUA_REDIS_SPECIFIC_FINISHED)) {
 				/* somebody yielded and waits for results */
@@ -708,16 +719,16 @@ lua_redis_timeout_sync(EV_P_ ev_timer *w, int revents)
 		return;
 	}
 
-	ud = sp_ud->c;
+	ud = sp_ud->common_ud;
 	ctx = sp_ud->ctx;
 	msg_debug_lua_redis("timeout while querying redis server: %p, redis: %p", sp_ud,
-						sp_ud->c->ctx);
+						sp_ud->common_ud->ctx);
 
-	if (sp_ud->c->ctx) {
-		ac = sp_ud->c->ctx;
+	if (sp_ud->common_ud->ctx) {
+		ac = sp_ud->common_ud->ctx;
 
 		/* Set to NULL to avoid double free in dtor */
-		sp_ud->c->ctx = NULL;
+		sp_ud->common_ud->ctx = NULL;
 		ac->err = REDIS_ERR_IO;
 		errno = ETIMEDOUT;
 		ctx->flags |= LUA_REDIS_TERMINATED;
@@ -726,7 +737,7 @@ lua_redis_timeout_sync(EV_P_ ev_timer *w, int revents)
 		 * This will call all callbacks pending so the entire context
 		 * will be destructed
 		 */
-		rspamd_redis_pool_release_connection(sp_ud->c->pool, ac,
+		rspamd_redis_pool_release_connection(sp_ud->common_ud->pool, ac,
 											 RSPAMD_REDIS_RELEASE_FATAL);
 	}
 }
@@ -745,24 +756,24 @@ lua_redis_timeout(EV_P_ ev_timer *w, int revents)
 	}
 
 	ctx = sp_ud->ctx;
-	ud = sp_ud->c;
+	ud = sp_ud->common_ud;
 
 	REDIS_RETAIN(ctx);
 	msg_debug_lua_redis("timeout while querying redis server: %p, redis: %p", sp_ud,
-						sp_ud->c->ctx);
-	lua_redis_push_error("timeout while connecting the server", ctx, sp_ud, TRUE);
+						sp_ud->common_ud->ctx);
+	lua_redis_push_error("timeout while connecting the server (%.2f sec)", ctx, sp_ud, TRUE, ud->timeout);
 
-	if (sp_ud->c->ctx) {
-		ac = sp_ud->c->ctx;
+	if (sp_ud->common_ud->ctx) {
+		ac = sp_ud->common_ud->ctx;
 		/* Set to NULL to avoid double free in dtor */
-		sp_ud->c->ctx = NULL;
+		sp_ud->common_ud->ctx = NULL;
 		ac->err = REDIS_ERR_IO;
 		errno = ETIMEDOUT;
 		/*
 		 * This will call all callbacks pending so the entire context
 		 * will be destructed
 		 */
-		rspamd_redis_pool_release_connection(sp_ud->c->pool, ac,
+		rspamd_redis_pool_release_connection(sp_ud->common_ud->pool, ac,
 											 RSPAMD_REDIS_RELEASE_FATAL);
 	}
 
@@ -771,12 +782,12 @@ lua_redis_timeout(EV_P_ ev_timer *w, int revents)
 
 
 static void
-lua_redis_parse_args(lua_State *L, gint idx, const gchar *cmd,
-					 gchar ***pargs, gsize **parglens, guint *nargs)
+lua_redis_parse_args(lua_State *L, int idx, const char *cmd,
+					 char ***pargs, gsize **parglens, unsigned int *nargs)
 {
-	gchar **args = NULL;
+	char **args = NULL;
 	gsize *arglens;
-	gint top;
+	int top;
 
 	if (idx != 0 && lua_type(L, idx) == LUA_TTABLE) {
 		/* Get all arguments */
@@ -785,7 +796,7 @@ lua_redis_parse_args(lua_State *L, gint idx, const gchar *cmd,
 		top = 0;
 
 		while (lua_next(L, -2) != 0) {
-			gint type = lua_type(L, -1);
+			int type = lua_type(L, -1);
 
 			if (type == LUA_TNUMBER || type == LUA_TSTRING ||
 				type == LUA_TUSERDATA) {
@@ -794,7 +805,7 @@ lua_redis_parse_args(lua_State *L, gint idx, const gchar *cmd,
 			lua_pop(L, 1);
 		}
 
-		args = g_malloc((top + 1) * sizeof(gchar *));
+		args = g_malloc((top + 1) * sizeof(char *));
 		arglens = g_malloc((top + 1) * sizeof(gsize));
 		arglens[0] = strlen(cmd);
 		args[0] = g_malloc(arglens[0]);
@@ -803,10 +814,10 @@ lua_redis_parse_args(lua_State *L, gint idx, const gchar *cmd,
 		lua_pushnil(L);
 
 		while (lua_next(L, -2) != 0) {
-			gint type = lua_type(L, -1);
+			int type = lua_type(L, -1);
 
 			if (type == LUA_TSTRING) {
-				const gchar *s;
+				const char *s;
 
 				s = lua_tolstring(L, -1, &arglens[top]);
 				args[top] = g_malloc(arglens[top]);
@@ -826,13 +837,13 @@ lua_redis_parse_args(lua_State *L, gint idx, const gchar *cmd,
 				}
 			}
 			else if (type == LUA_TNUMBER) {
-				gdouble val = lua_tonumber(L, -1);
-				gint r;
-				gchar numbuf[64];
+				double val = lua_tonumber(L, -1);
+				int r;
+				char numbuf[64];
 
-				if (val == (gdouble) ((gint64) val)) {
+				if (val == (double) ((int64_t) val)) {
 					r = rspamd_snprintf(numbuf, sizeof(numbuf), "%L",
-										(gint64) val);
+										(int64_t) val);
 				}
 				else {
 					r = rspamd_snprintf(numbuf, sizeof(numbuf), "%f",
@@ -853,7 +864,7 @@ lua_redis_parse_args(lua_State *L, gint idx, const gchar *cmd,
 	else {
 		/* Use merely cmd */
 
-		args = g_malloc(sizeof(gchar *));
+		args = g_malloc(sizeof(char *));
 		arglens = g_malloc(sizeof(gsize));
 		arglens[0] = strlen(cmd);
 		args[0] = g_malloc(arglens[0]);
@@ -867,21 +878,21 @@ lua_redis_parse_args(lua_State *L, gint idx, const gchar *cmd,
 }
 
 static struct lua_redis_ctx *
-rspamd_lua_redis_prepare_connection(lua_State *L, gint *pcbref, gboolean is_async)
+rspamd_lua_redis_prepare_connection(lua_State *L, int *pcbref, gboolean is_async)
 {
 	struct lua_redis_ctx *ctx = NULL;
 	rspamd_inet_addr_t *ip = NULL;
 	struct lua_redis_userdata *ud = NULL;
 	struct rspamd_lua_ip *addr = NULL;
 	struct rspamd_task *task = NULL;
-	const gchar *host = NULL;
-	const gchar *username = NULL, *password = NULL, *dbname = NULL, *log_tag = NULL;
-	gint cbref = -1;
+	const char *host = NULL;
+	const char *username = NULL, *password = NULL, *dbname = NULL, *log_tag = NULL;
+	int cbref = -1;
 	struct rspamd_config *cfg = NULL;
 	struct rspamd_async_session *session = NULL;
 	struct ev_loop *ev_base = NULL;
 	gboolean ret = FALSE;
-	guint flags = 0;
+	unsigned int flags = 0;
 
 	if (lua_istable(L, 1)) {
 		/* Table version */
@@ -1086,8 +1097,8 @@ rspamd_lua_redis_prepare_connection(lua_State *L, gint *pcbref, gboolean is_asyn
 			return NULL;
 		}
 
-		msg_debug_lua_redis("opened redis connection host=%s; ctx=%p; ud=%p",
-							host, ctx, ud);
+		msg_debug_lua_redis("opened redis connection host=%s; lua_ctx=%p; redis_ctx=%p; ud=%p",
+							host, ctx, ud->ctx, ud);
 
 		return ctx;
 	}
@@ -1117,9 +1128,9 @@ lua_redis_make_request(lua_State *L)
 	struct lua_redis_request_specific_userdata *sp_ud;
 	struct lua_redis_userdata *ud;
 	struct lua_redis_ctx *ctx, **pctx;
-	const gchar *cmd = NULL;
-	gdouble timeout = REDIS_DEFAULT_TIMEOUT;
-	gint cbref = -1;
+	const char *cmd = NULL;
+	double timeout = REDIS_DEFAULT_TIMEOUT;
+	int cbref = -1;
 	gboolean ret = FALSE;
 
 	ctx = rspamd_lua_redis_prepare_connection(L, &cbref, TRUE);
@@ -1128,7 +1139,7 @@ lua_redis_make_request(lua_State *L)
 		ud = &ctx->async;
 		sp_ud = g_malloc0(sizeof(*sp_ud));
 		sp_ud->cbref = cbref;
-		sp_ud->c = ud;
+		sp_ud->common_ud = ud;
 		sp_ud->ctx = ctx;
 
 		lua_pushstring(L, "cmd");
@@ -1156,7 +1167,7 @@ lua_redis_make_request(lua_State *L)
 									lua_redis_callback,
 									sp_ud,
 									sp_ud->nargs,
-									(const gchar **) sp_ud->args,
+									(const char **) sp_ud->args,
 									sp_ud->arglens);
 
 		if (ret == REDIS_OK) {
@@ -1206,7 +1217,7 @@ lua_redis_make_request(lua_State *L)
 	if (ret) {
 		pctx = lua_newuserdata(L, sizeof(ctx));
 		*pctx = ctx;
-		rspamd_lua_setclass(L, "rspamd{redis}", -1);
+		rspamd_lua_setclass(L, rspamd_redis_classname, -1);
 	}
 	else {
 		lua_pushnil(L);
@@ -1230,13 +1241,13 @@ lua_redis_make_request_sync(lua_State *L)
 	LUA_TRACE_POINT;
 	struct rspamd_lua_ip *addr = NULL;
 	rspamd_inet_addr_t *ip = NULL;
-	const gchar *cmd = NULL, *host;
+	const char *cmd = NULL, *host;
 	struct timeval tv;
 	gboolean ret = FALSE;
-	gdouble timeout = REDIS_DEFAULT_TIMEOUT;
-	gchar **args = NULL;
+	double timeout = REDIS_DEFAULT_TIMEOUT;
+	char **args = NULL;
 	gsize *arglens = NULL;
-	guint nargs = 0, flags = 0;
+	unsigned int nargs = 0, flags = 0;
 	redisContext *ctx;
 	redisReply *r;
 
@@ -1323,7 +1334,7 @@ lua_redis_make_request_sync(lua_State *L)
 
 		r = redisCommandArgv(ctx,
 							 nargs,
-							 (const gchar **) args,
+							 (const char **) args,
 							 arglens);
 
 		if (r != NULL) {
@@ -1376,7 +1387,7 @@ lua_redis_connect(lua_State *L)
 	LUA_TRACE_POINT;
 	struct lua_redis_userdata *ud;
 	struct lua_redis_ctx *ctx, **pctx;
-	gdouble timeout = REDIS_DEFAULT_TIMEOUT;
+	double timeout = REDIS_DEFAULT_TIMEOUT;
 
 	ctx = rspamd_lua_redis_prepare_connection(L, NULL, TRUE);
 
@@ -1402,7 +1413,7 @@ lua_redis_connect(lua_State *L)
 	lua_pushboolean(L, TRUE);
 	pctx = lua_newuserdata(L, sizeof(ctx));
 	*pctx = ctx;
-	rspamd_lua_setclass(L, "rspamd{redis}", -1);
+	rspamd_lua_setclass(L, rspamd_redis_classname, -1);
 
 	return 2;
 }
@@ -1418,7 +1429,7 @@ static int
 lua_redis_connect_sync(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble timeout = REDIS_DEFAULT_TIMEOUT;
+	double timeout = REDIS_DEFAULT_TIMEOUT;
 	struct lua_redis_ctx *ctx, **pctx;
 
 	ctx = rspamd_lua_redis_prepare_connection(L, NULL, FALSE);
@@ -1438,7 +1449,7 @@ lua_redis_connect_sync(lua_State *L)
 		lua_pushboolean(L, TRUE);
 		pctx = lua_newuserdata(L, sizeof(ctx));
 		*pctx = ctx;
-		rspamd_lua_setclass(L, "rspamd{redis}", -1);
+		rspamd_lua_setclass(L, rspamd_redis_classname, -1);
 	}
 	else {
 		lua_pushboolean(L, FALSE);
@@ -1463,9 +1474,9 @@ lua_redis_add_cmd(lua_State *L)
 	struct lua_redis_ctx *ctx = lua_check_redis(L, 1);
 	struct lua_redis_request_specific_userdata *sp_ud;
 	struct lua_redis_userdata *ud;
-	const gchar *cmd = NULL;
-	gint args_pos = 2;
-	gint cbref = -1, ret;
+	const char *cmd = NULL;
+	int args_pos = 2;
+	int cbref = -1, ret;
 
 	if (ctx) {
 		if (ctx->flags & LUA_REDIS_TERMINATED) {
@@ -1492,21 +1503,18 @@ lua_redis_add_cmd(lua_State *L)
 		}
 
 		sp_ud = g_malloc0(sizeof(*sp_ud));
+		sp_ud->common_ud = &ctx->async;
+		ud = &ctx->async;
 		if (IS_ASYNC(ctx)) {
-			sp_ud->c = &ctx->async;
-			ud = &ctx->async;
 			sp_ud->cbref = cbref;
 		}
-		else {
-			sp_ud->c = &ctx->async;
-			ud = &ctx->async;
-		}
+
 		sp_ud->ctx = ctx;
 
 		lua_redis_parse_args(L, args_pos, cmd, &sp_ud->args,
 							 &sp_ud->arglens, &sp_ud->nargs);
 
-		LL_PREPEND(sp_ud->c->specific, sp_ud);
+		LL_PREPEND(sp_ud->common_ud->specific, sp_ud);
 
 		if (ud->s && rspamd_session_blocked(ud->s)) {
 			lua_pushboolean(L, 0);
@@ -1516,19 +1524,19 @@ lua_redis_add_cmd(lua_State *L)
 		}
 
 		if (IS_ASYNC(ctx)) {
-			ret = redisAsyncCommandArgv(sp_ud->c->ctx,
+			ret = redisAsyncCommandArgv(sp_ud->common_ud->ctx,
 										lua_redis_callback,
 										sp_ud,
 										sp_ud->nargs,
-										(const gchar **) sp_ud->args,
+										(const char **) sp_ud->args,
 										sp_ud->arglens);
 		}
 		else {
-			ret = redisAsyncCommandArgv(sp_ud->c->ctx,
+			ret = redisAsyncCommandArgv(sp_ud->common_ud->ctx,
 										lua_redis_callback_sync,
 										sp_ud,
 										sp_ud->nargs,
-										(const gchar **) sp_ud->args,
+										(const char **) sp_ud->args,
 										sp_ud->arglens);
 		}
 
@@ -1545,25 +1553,28 @@ lua_redis_add_cmd(lua_State *L)
 			}
 
 			sp_ud->timeout_ev.data = sp_ud;
+			ev_now_update_if_cheap(ud->event_loop);
 
 			if (IS_ASYNC(ctx)) {
 				ev_timer_init(&sp_ud->timeout_ev, lua_redis_timeout,
-							  sp_ud->c->timeout, 0.0);
+							  sp_ud->common_ud->timeout, 0.0);
 			}
 			else {
 				ev_timer_init(&sp_ud->timeout_ev, lua_redis_timeout_sync,
-							  sp_ud->c->timeout, 0.0);
+							  sp_ud->common_ud->timeout, 0.0);
 			}
 
 			ev_timer_start(ud->event_loop, &sp_ud->timeout_ev);
+			msg_debug_lua_redis("added timeout %f for %p", sp_ud->common_ud->timeout, sp_ud);
+
 			REDIS_RETAIN(ctx);
 			ctx->cmds_pending++;
 		}
 		else {
 			msg_info("call to redis failed: %s",
-					 sp_ud->c->ctx->errstr);
+					 sp_ud->common_ud->ctx->errstr);
 			lua_pushboolean(L, 0);
-			lua_pushstring(L, sp_ud->c->ctx->errstr);
+			lua_pushstring(L, sp_ud->common_ud->ctx->errstr);
 
 			return 2;
 		}
@@ -1597,12 +1608,21 @@ lua_redis_exec(lua_State *L)
 		return 0;
 	}
 	else {
-		if (ctx->cmds_pending == 0 && g_queue_get_length(ctx->replies) == 0) {
+		struct lua_redis_userdata *ud = &ctx->async;
+		int replies_pending = g_queue_get_length(ctx->replies);
+
+		msg_debug_lua_redis("execute pending commands for %p; commands pending = %d; replies pending = %d",
+			ctx,
+			ctx->cmds_pending,
+			replies_pending);
+
+		if (ctx->cmds_pending == 0 && replies_pending == 0) {
 			lua_pushstring(L, "No pending commands to execute");
 			lua_error(L);
 		}
-		if (ctx->cmds_pending == 0 && g_queue_get_length(ctx->replies) > 0) {
-			gint results = lua_redis_push_results(ctx, L);
+
+		if (ctx->cmds_pending == 0 && replies_pending > 0) {
+			int results = lua_redis_push_results(ctx, L);
 			return results;
 		}
 		else {
@@ -1612,7 +1632,7 @@ lua_redis_exec(lua_State *L)
 	}
 }
 
-static gint
+static int
 lua_load_redis(lua_State *L)
 {
 	lua_newtable(L);
@@ -1621,7 +1641,7 @@ lua_load_redis(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_redis_null_idx(lua_State *L)
 {
 	lua_pushnil(L);
@@ -1649,7 +1669,7 @@ lua_redis_null_mt(lua_State *L)
  */
 void luaopen_redis(lua_State *L)
 {
-	rspamd_lua_new_class(L, "rspamd{redis}", redislib_m);
+	rspamd_lua_new_class(L, rspamd_redis_classname, redislib_m);
 	lua_pop(L, 1);
 	rspamd_lua_add_preload(L, "rspamd_redis", lua_load_redis);
 
