@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -23,16 +23,14 @@
 
 #include "lua_parsers.h"
 
-#ifdef WITH_LUA_REPL
-#include "replxx.h"
-#endif
 
+#include "replxx.h"
 #include <math.h>
 #include <glob.h>
 
 #include "unicode/uspoof.h"
 #include "unicode/uscript.h"
-#include "contrib/fastutf8/fastutf8.h"
+#include "rspamd_simdutf.h"
 
 /***
  * @module rspamd_util
@@ -644,9 +642,10 @@ LUA_FUNCTION_DEF(util, get_hostname);
 LUA_FUNCTION_DEF(util, parse_content_type);
 
 /***
- *  @function util.mime_header_encode(hdr)
+ *  @function util.mime_header_encode(hdr[, is_structured])
  * Encodes header if needed
  * @param {string} hdr input header
+ * @param {boolean} is_structured if true, then we encode as structured header (e.g. encode all non alpha-numeric characters)
  * @return encoded header
  */
 LUA_FUNCTION_DEF(util, mime_header_encode);
@@ -754,40 +753,48 @@ static const struct luaL_reg int64lib_m[] = {
 	{NULL, NULL}};
 
 LUA_FUNCTION_DEF(ev_base, loop);
+LUA_FUNCTION_DEF(ev_base, update_time);
+LUA_FUNCTION_DEF(ev_base, timestamp);
+LUA_FUNCTION_DEF(ev_base, pending_events);
+LUA_FUNCTION_DEF(ev_base, add_timer);
 
 static const struct luaL_reg ev_baselib_m[] = {
 	LUA_INTERFACE_DEF(ev_base, loop),
+	LUA_INTERFACE_DEF(ev_base, update_time),
+	LUA_INTERFACE_DEF(ev_base, timestamp),
+	LUA_INTERFACE_DEF(ev_base, pending_events),
+	LUA_INTERFACE_DEF(ev_base, add_timer),
 	{"__tostring", rspamd_lua_class_tostring},
 	{NULL, NULL}};
 
-static gint64
-lua_check_int64(lua_State *L, gint pos)
+static int64_t
+lua_check_int64(lua_State *L, int pos)
 {
-	void *ud = rspamd_lua_check_udata(L, pos, "rspamd{int64}");
+	void *ud = rspamd_lua_check_udata(L, pos, rspamd_int64_classname);
 	luaL_argcheck(L, ud != NULL, pos, "'int64' expected");
-	return ud ? *((gint64 *) ud) : 0LL;
+	return ud ? *((int64_t *) ud) : 0LL;
 }
 
 
-static gint
+static int
 lua_util_create_event_base(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct ev_loop **pev_base;
 
 	pev_base = lua_newuserdata(L, sizeof(struct ev_loop *));
-	rspamd_lua_setclass(L, "rspamd{ev_base}", -1);
+	rspamd_lua_setclass(L, rspamd_ev_base_classname, -1);
 	*pev_base = ev_loop_new(EVFLAG_SIGNALFD | EVBACKEND_ALL);
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_load_rspamd_config(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_config *cfg, **pcfg;
-	const gchar *cfg_name;
+	const char *cfg_name;
 
 	cfg_name = luaL_checkstring(L, 1);
 
@@ -802,7 +809,7 @@ lua_util_load_rspamd_config(lua_State *L)
 		else {
 			rspamd_config_post_load(cfg, 0);
 			pcfg = lua_newuserdata(L, sizeof(struct rspamd_config *));
-			rspamd_lua_setclass(L, "rspamd{config}", -1);
+			rspamd_lua_setclass(L, rspamd_config_classname, -1);
 			*pcfg = cfg;
 		}
 	}
@@ -810,13 +817,13 @@ lua_util_load_rspamd_config(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 parse_config_options(const char *str_options)
 {
-	gint ret = 0;
-	gchar **vec;
-	const gchar *str;
-	guint i, l;
+	int ret = 0;
+	char **vec;
+	const char *str;
+	unsigned int i, l;
 
 	vec = g_strsplit_set(str_options, ",;", -1);
 	if (vec) {
@@ -853,7 +860,7 @@ parse_config_options(const char *str_options)
 	return ret;
 }
 
-static gint
+static int
 lua_util_config_from_ucl(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -862,7 +869,7 @@ lua_util_config_from_ucl(lua_State *L)
 	GError *err = NULL;
 	ucl_object_t *obj;
 	const char *str_options = NULL;
-	gint int_options = 0;
+	int int_options = 0;
 
 
 	obj = ucl_object_lua_import(L, 1);
@@ -898,7 +905,7 @@ lua_util_config_from_ucl(lua_State *L)
 
 			rspamd_config_post_load(cfg, int_options);
 			pcfg = lua_newuserdata(L, sizeof(struct rspamd_config *));
-			rspamd_lua_setclass(L, "rspamd{config}", -1);
+			rspamd_lua_setclass(L, rspamd_config_classname, -1);
 			*pcfg = cfg;
 		}
 
@@ -919,12 +926,12 @@ lua_util_task_fin(struct rspamd_task *task, void *ud)
 	return TRUE;
 }
 
-static gint
+static int
 lua_util_process_message(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_config *cfg = lua_check_config(L, 1);
-	const gchar *message;
+	const char *message;
 	gsize mlen;
 	struct rspamd_task *task;
 	struct ev_loop *base;
@@ -979,12 +986,12 @@ lua_util_process_message(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_encode_base64(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	gchar *out;
+	char *out;
 	gsize outlen;
 	long str_lim = 0;
 	gboolean fold = FALSE;
@@ -1008,7 +1015,7 @@ lua_util_encode_base64(lua_State *L)
 			enum rspamd_newlines_type how = RSPAMD_TASK_NEWLINES_CRLF;
 
 			if (lua_type(L, 3) == LUA_TSTRING) {
-				const gchar *how_str = lua_tostring(L, 3);
+				const char *how_str = lua_tostring(L, 3);
 
 				if (g_ascii_strcasecmp(how_str, "cr") == 0) {
 					how = RSPAMD_TASK_NEWLINES_CR;
@@ -1025,7 +1032,12 @@ lua_util_encode_base64(lua_State *L)
 		}
 
 		if (out != NULL) {
-			lua_new_text(L, out, outlen, TRUE);
+			/*
+			 * Manually set OWN flag, as `lua_new_text` will allocate another chunk of memory,
+			 * and we will have memory leak of the memory allocated by `rspamd_encode_base64_fold`
+			 */
+			t = lua_new_text(L, out, outlen, FALSE);
+			t->flags = RSPAMD_TEXT_FLAG_OWN;
 		}
 		else {
 			lua_pushnil(L);
@@ -1035,15 +1047,15 @@ lua_util_encode_base64(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_encode_qp(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
-	gchar *out;
+	const char *s = NULL;
+	char *out;
 	gsize inlen, outlen;
-	guint str_lim = 0;
+	unsigned int str_lim = 0;
 
 	if (lua_type(L, 1) == LUA_TSTRING) {
 		s = luaL_checklstring(L, 1, &inlen);
@@ -1068,7 +1080,7 @@ lua_util_encode_qp(lua_State *L)
 		enum rspamd_newlines_type how = RSPAMD_TASK_NEWLINES_CRLF;
 
 		if (lua_type(L, 3) == LUA_TSTRING) {
-			const gchar *how_str = lua_tostring(L, 3);
+			const char *how_str = lua_tostring(L, 3);
 
 			if (g_ascii_strcasecmp(how_str, "cr") == 0) {
 				how = RSPAMD_TASK_NEWLINES_CR;
@@ -1085,7 +1097,7 @@ lua_util_encode_qp(lua_State *L)
 
 		if (out != NULL) {
 			t = lua_newuserdata(L, sizeof(*t));
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
+			rspamd_lua_setclass(L, rspamd_text_classname, -1);
 			t->start = out;
 			t->len = outlen;
 			/* Need destruction */
@@ -1099,12 +1111,12 @@ lua_util_encode_qp(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_decode_qp(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t, *out;
-	const gchar *s = NULL;
+	const char *s = NULL;
 	gsize inlen = 0;
 	gssize outlen;
 
@@ -1125,10 +1137,10 @@ lua_util_decode_qp(lua_State *L)
 	}
 	else {
 		out = lua_newuserdata(L, sizeof(*t));
-		rspamd_lua_setclass(L, "rspamd{text}", -1);
+		rspamd_lua_setclass(L, rspamd_text_classname, -1);
 		out->start = g_malloc(inlen + 1);
 		out->flags = RSPAMD_TEXT_FLAG_OWN;
-		outlen = rspamd_decode_qp_buf(s, inlen, (gchar *) out->start, inlen + 1);
+		outlen = rspamd_decode_qp_buf(s, inlen, (char *) out->start, inlen + 1);
 
 		if (outlen > 0) {
 			out->len = outlen;
@@ -1145,12 +1157,12 @@ lua_util_decode_qp(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_decode_base64(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
+	const char *s = NULL;
 	gsize inlen = 0, outlen;
 
 	if (lua_type(L, 1) == LUA_TSTRING) {
@@ -1167,11 +1179,11 @@ lua_util_decode_base64(lua_State *L)
 
 	if (s != NULL) {
 		t = lua_newuserdata(L, sizeof(*t));
-		rspamd_lua_setclass(L, "rspamd{text}", -1);
+		rspamd_lua_setclass(L, rspamd_text_classname, -1);
 		t->len = (inlen / 4) * 3 + 3;
 		t->start = g_malloc(t->len);
 
-		rspamd_cryptobox_base64_decode(s, inlen, (guchar *) t->start,
+		rspamd_cryptobox_base64_decode(s, inlen, (unsigned char *) t->start,
 									   &outlen);
 		t->len = outlen;
 		t->flags = RSPAMD_TEXT_FLAG_OWN;
@@ -1183,13 +1195,13 @@ lua_util_decode_base64(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_encode_base32(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
-	gchar *out;
+	const char *s = NULL;
+	char *out;
 	enum rspamd_base32_type btype = RSPAMD_BASE32_DEFAULT;
 	gsize inlen, outlen;
 
@@ -1222,7 +1234,7 @@ lua_util_encode_base32(lua_State *L)
 		if (out != NULL) {
 			t = lua_newuserdata(L, sizeof(*t));
 			outlen = strlen(out);
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
+			rspamd_lua_setclass(L, rspamd_text_classname, -1);
 			t->start = out;
 			t->len = outlen;
 			/* Need destruction */
@@ -1236,12 +1248,12 @@ lua_util_encode_base32(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_decode_base32(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
-	const gchar *s = NULL;
+	const char *s = NULL;
 	gsize inlen, outlen;
 	enum rspamd_base32_type btype = RSPAMD_BASE32_DEFAULT;
 
@@ -1266,14 +1278,14 @@ lua_util_decode_base32(lua_State *L)
 	}
 
 	if (s != NULL) {
-		guchar *decoded;
+		unsigned char *decoded;
 
 		decoded = rspamd_decode_base32(s, inlen, &outlen, btype);
 
 		if (decoded) {
 			t = lua_newuserdata(L, sizeof(*t));
-			rspamd_lua_setclass(L, "rspamd{text}", -1);
-			t->start = (const gchar *) decoded;
+			rspamd_lua_setclass(L, rspamd_text_classname, -1);
+			t->start = (const char *) decoded;
 			t->len = outlen;
 			t->flags = RSPAMD_TEXT_FLAG_OWN;
 		}
@@ -1288,7 +1300,7 @@ lua_util_decode_base32(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_decode_url(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1309,36 +1321,36 @@ lua_util_decode_url(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_tokenize_text(lua_State *L)
 {
 	return lua_parsers_tokenize_text(L);
 }
 
-static gint
+static int
 lua_util_tanh(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble in = luaL_checknumber(L, 1);
+	double in = luaL_checknumber(L, 1);
 
 	lua_pushnumber(L, tanh(in));
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_parse_html(lua_State *L)
 {
 	return lua_parsers_parse_html(L);
 }
 
-static gint
+static int
 lua_util_levenshtein_distance(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t1, *t2;
-	gint dist = 0;
-	guint replace_cost = 1;
+	int dist = 0;
+	unsigned int replace_cost = 1;
 
 	t1 = lua_check_text_or_string(L, 1);
 	t2 = lua_check_text_or_string(L, 2);
@@ -1359,11 +1371,11 @@ lua_util_levenshtein_distance(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_fold_header(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *how, *stop_chars = NULL;
+	const char *how, *stop_chars = NULL;
 	struct rspamd_lua_text *name, *value;
 	GString *folded;
 
@@ -1415,13 +1427,13 @@ lua_util_fold_header(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_is_uppercase(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint32 i = 0;
+	int32_t i = 0;
 	UChar32 uc;
-	guint nlc = 0, nuc = 0;
+	bool is_upper = false, is_lower = false, is_other = false;
 
 	struct rspamd_lua_text *t = lua_check_text_or_string(L, 1);
 	if (t) {
@@ -1433,15 +1445,20 @@ lua_util_is_uppercase(lua_State *L)
 			}
 
 			if (u_isupper(uc)) {
-				nuc++;
+				is_upper = true;
 			}
 			else if (u_islower(uc)) {
-				nlc++;
+				is_lower = true;
+				break;
+			}
+			else if (u_charType(uc) == U_OTHER_LETTER) {
+				is_other = true;
+				break;
 			}
 		}
 	}
 
-	if (nuc > 0 && nlc == 0) {
+	if (is_upper && !is_lower && !is_other) {
 		lua_pushboolean(L, TRUE);
 	}
 	else {
@@ -1451,12 +1468,12 @@ lua_util_is_uppercase(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_humanize_number(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint64 number = luaL_checkinteger(L, 1);
-	gchar numbuf[32];
+	int64_t number = luaL_checkinteger(L, 1);
+	char numbuf[32];
 
 
 	rspamd_snprintf(numbuf, sizeof(numbuf), "%hL", number);
@@ -1465,11 +1482,11 @@ lua_util_humanize_number(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_get_tld(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *host;
+	const char *host;
 	gsize hostlen;
 	rspamd_ftok_t tld;
 
@@ -1491,13 +1508,13 @@ lua_util_get_tld(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_glob(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *pattern;
+	const char *pattern;
 	glob_t gl;
-	gint top, i, flags = 0;
+	int top, i, flags = 0;
 
 	top = lua_gettop(L);
 	memset(&gl, 0, sizeof(gl));
@@ -1518,7 +1535,7 @@ lua_util_glob(lua_State *L)
 
 	lua_createtable(L, gl.gl_pathc, 0);
 	/* Push results */
-	for (i = 0; i < (gint) gl.gl_pathc; i++) {
+	for (i = 0; i < (int) gl.gl_pathc; i++) {
 		lua_pushstring(L, gl.gl_pathv[i]);
 		lua_rawseti(L, -2, i + 1);
 	}
@@ -1528,13 +1545,13 @@ lua_util_glob(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_parse_mail_address(lua_State *L)
 {
 	return lua_parsers_parse_mail_address(L);
 }
 
-static gint
+static int
 lua_util_strlen_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1543,11 +1560,11 @@ lua_util_strlen_utf8(lua_State *L)
 	t = lua_check_text_or_string(L, 1);
 
 	if (t) {
-		gint32 i = 0, nchars = 0;
+		int32_t i = 0, nchars = 0;
 		UChar32 uc;
 
 		while (i < t->len) {
-			U8_NEXT((guint8 *) t->start, i, t->len, uc);
+			U8_NEXT((uint8_t *) t->start, i, t->len, uc);
 			nchars++;
 		}
 
@@ -1560,16 +1577,16 @@ lua_util_strlen_utf8(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_lower_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t;
 
-	gchar *dst;
+	char *dst;
 	UChar32 uc;
 	UBool err = 0;
-	gint32 i = 0, j = 0;
+	int32_t i = 0, j = 0;
 
 	t = lua_check_text_or_string(L, 1);
 
@@ -1577,7 +1594,7 @@ lua_util_lower_utf8(lua_State *L)
 		dst = g_malloc(t->len);
 
 		while (i < t->len && err == 0) {
-			U8_NEXT((guint8 *) t->start, i, t->len, uc);
+			U8_NEXT((uint8_t *) t->start, i, t->len, uc);
 			uc = u_tolower(uc);
 			U8_APPEND(dst, j, t->len, uc, err);
 		}
@@ -1599,7 +1616,7 @@ lua_util_lower_utf8(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_normalize_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1632,7 +1649,7 @@ lua_util_normalize_utf8(lua_State *L)
 	return 2;
 }
 
-static gint
+static int
 lua_util_transliterate(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1645,17 +1662,19 @@ lua_util_transliterate(lua_State *L)
 
 	gsize outlen;
 	char *transliterated = rspamd_utf8_transliterate(t->start, t->len, &outlen);
-	lua_new_text(L, transliterated, outlen, TRUE);
+
+	t = lua_new_text(L, transliterated, outlen, FALSE);
+	t->flags = RSPAMD_TEXT_FLAG_OWN;
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_strequal_caseless(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t1, *t2;
-	gint ret = -1;
+	int ret = -1;
 
 	t1 = lua_check_text_or_string(L, 1);
 	t2 = lua_check_text_or_string(L, 2);
@@ -1677,12 +1696,12 @@ lua_util_strequal_caseless(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_strequal_caseless_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	struct rspamd_lua_text *t1, *t2;
-	gint ret = -1;
+	int ret = -1;
 
 	t1 = lua_check_text_or_string(L, 1);
 	t2 = lua_check_text_or_string(L, 2);
@@ -1699,11 +1718,11 @@ lua_util_strequal_caseless_utf8(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_get_ticks(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble ticks;
+	double ticks;
 	gboolean rdtsc = FALSE;
 
 	if (lua_isboolean(L, 1)) {
@@ -1716,7 +1735,7 @@ lua_util_get_ticks(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_get_time(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -1726,11 +1745,11 @@ lua_util_get_time(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_time_to_string(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble seconds;
+	double seconds;
 	char timebuf[128];
 
 	if (lua_isnumber(L, 1)) {
@@ -1746,11 +1765,11 @@ lua_util_time_to_string(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_stat(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *fpath;
+	const char *fpath;
 	struct stat st;
 
 	fpath = luaL_checkstring(L, 1);
@@ -1792,12 +1811,12 @@ lua_util_stat(lua_State *L)
 	return 2;
 }
 
-static gint
+static int
 lua_util_unlink(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *fpath;
-	gint ret;
+	const char *fpath;
+	int ret;
 
 	fpath = luaL_checkstring(L, 1);
 
@@ -1820,12 +1839,12 @@ lua_util_unlink(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_lock_file(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *fpath;
-	gint fd = -1;
+	const char *fpath;
+	int fd = -1;
 	gboolean own = FALSE;
 
 #if !HAVE_FLOCK
@@ -1878,11 +1897,11 @@ lua_util_lock_file(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_unlock_file(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint fd = -1, ret, serrno;
+	int fd = -1, ret, serrno;
 	gboolean do_close = TRUE;
 
 #if !HAVE_FLOCK
@@ -1928,12 +1947,12 @@ lua_util_unlock_file(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_create_file(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint fd, mode = 00644;
-	const gchar *fpath;
+	int fd, mode = 00644;
+	const char *fpath;
 
 	fpath = luaL_checkstring(L, 1);
 
@@ -1960,11 +1979,11 @@ lua_util_create_file(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_close_file(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint fd = -1;
+	int fd = -1;
 
 	if (lua_isnumber(L, 1)) {
 		fd = lua_tointeger(L, 1);
@@ -1985,12 +2004,12 @@ lua_util_close_file(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_random_hex(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gchar *buf;
-	gint buflen;
+	char *buf;
+	int buflen;
 
 	buflen = lua_tointeger(L, 1);
 
@@ -2006,41 +2025,41 @@ lua_util_random_hex(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_zstd_compress(lua_State *L)
 {
 	return lua_compress_zstd_compress(L);
 }
 
-static gint
+static int
 lua_util_zstd_decompress(lua_State *L)
 {
 	return lua_compress_zstd_decompress(L);
 }
 
-static gint
+static int
 lua_util_gzip_compress(lua_State *L)
 {
 	return lua_compress_zlib_compress(L);
 }
 
-static gint
+static int
 lua_util_gzip_decompress(lua_State *L)
 {
 	return lua_compress_zlib_decompress(L, true);
 }
 
-static gint
+static int
 lua_util_inflate(lua_State *L)
 {
 	return lua_compress_zlib_decompress(L, false);
 }
 
-static gint
+static int
 lua_util_normalize_prob(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gdouble x, bias = 0.5;
+	double x, bias = 0.5;
 
 	x = lua_tonumber(L, 1);
 
@@ -2053,13 +2072,13 @@ lua_util_normalize_prob(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_caseless_hash(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	guint64 seed = 0xdeadbabe, h;
+	uint64_t seed = 0xdeadbabe, h;
 	struct rspamd_lua_text *t = NULL;
-	gint64 *r;
+	int64_t *r;
 
 	t = lua_check_text_or_string(L, 1);
 
@@ -2077,19 +2096,19 @@ lua_util_caseless_hash(lua_State *L)
 	h = rspamd_icase_hash(t->start, t->len, seed);
 	r = lua_newuserdata(L, sizeof(*r));
 	*r = h;
-	rspamd_lua_setclass(L, "rspamd{int64}", -1);
+	rspamd_lua_setclass(L, rspamd_int64_classname, -1);
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_caseless_hash_fast(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	guint64 seed = 0xdeadbabe, h;
+	uint64_t seed = 0xdeadbabe, h;
 	struct rspamd_lua_text *t = NULL;
 	union {
-		guint64 i;
+		uint64_t i;
 		double d;
 	} u;
 
@@ -2118,14 +2137,14 @@ lua_util_caseless_hash_fast(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_is_utf_spoofed(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	gsize l1, l2;
-	gint ret, nres = 2;
-	const gchar *s1 = lua_tolstring(L, 1, &l1),
-				*s2 = lua_tolstring(L, 2, &l2);
+	int ret, nres = 2;
+	const char *s1 = lua_tolstring(L, 1, &l1),
+			   *s2 = lua_tolstring(L, 2, &l2);
 	static USpoofChecker *spc, *spc_sgl;
 	UErrorCode uc_err = U_ZERO_ERROR;
 
@@ -2139,6 +2158,17 @@ lua_util_is_utf_spoofed(lua_State *L)
 
 				return 1;
 			}
+
+			/* Disable single script confusables, as it is not what we want to check */
+			uspoof_setChecks(spc,
+							 USPOOF_CONFUSABLE & ~USPOOF_SINGLE_SCRIPT_CONFUSABLE,
+							 &uc_err);
+			if (uc_err != U_ZERO_ERROR) {
+				msg_err("Cannot set proper checks for uspoof: %s", u_errorName(uc_err));
+				lua_pushboolean(L, false);
+				uspoof_close(spc);
+				return 1;
+			}
 		}
 
 		ret = uspoof_areConfusableUTF8(spc, s1, l1, s2, l2, &uc_err);
@@ -2161,7 +2191,7 @@ lua_util_is_utf_spoofed(lua_State *L)
 			if (uc_err != U_ZERO_ERROR) {
 				msg_err("Cannot set proper checks for uspoof: %s", u_errorName(uc_err));
 				lua_pushboolean(L, false);
-				uspoof_close(spc);
+				uspoof_close(spc_sgl);
 				return 1;
 			}
 		}
@@ -2195,12 +2225,12 @@ lua_util_is_utf_spoofed(lua_State *L)
 	return nres;
 }
 
-static gint
+static int
 lua_util_is_utf_mixed_script(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	gsize len_of_string;
-	const guchar *string_to_check = lua_tolstring(L, 1, &len_of_string);
+	const unsigned char *string_to_check = lua_tolstring(L, 1, &len_of_string);
 	UScriptCode last_script_code = USCRIPT_INVALID_CODE;
 	UErrorCode uc_err = U_ZERO_ERROR;
 
@@ -2250,17 +2280,17 @@ lua_util_is_utf_mixed_script(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_get_string_stats(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint num_of_digits = 0, num_of_letters = 0;
+	int num_of_digits = 0, num_of_letters = 0;
 	struct rspamd_lua_text *t;
 
 	t = lua_check_text_or_string(L, 1);
 
 	if (t) {
-		const gchar *p = t->start, *end = t->start + t->len;
+		const char *p = t->start, *end = t->start + t->len;
 		while (p < end) {
 			if (g_ascii_isdigit(*p)) {
 				num_of_digits++;
@@ -2287,14 +2317,14 @@ lua_util_get_string_stats(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_is_utf_outside_range(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint ret;
+	int ret;
 	struct rspamd_lua_text *t = lua_check_text_or_string(L, 1);
-	guint32 range_start = lua_tointeger(L, 2);
-	guint32 range_end = lua_tointeger(L, 3);
+	uint32_t range_start = lua_tointeger(L, 2);
+	uint32_t range_end = lua_tointeger(L, 3);
 
 	static rspamd_lru_hash_t *validators;
 
@@ -2303,7 +2333,7 @@ lua_util_is_utf_outside_range(lua_State *L)
 	}
 
 	if (t) {
-		guint64 hash_key = (guint64) range_end << 32 || range_start;
+		uint64_t hash_key = (uint64_t) range_end << 32 || range_start;
 
 		USpoofChecker *validator = rspamd_lru_hash_lookup(validators, &hash_key, 0);
 
@@ -2311,7 +2341,7 @@ lua_util_is_utf_outside_range(lua_State *L)
 
 		if (validator == NULL) {
 			USet *allowed_chars;
-			guint64 *creation_hash_key = g_malloc(sizeof(guint64));
+			uint64_t *creation_hash_key = g_malloc(sizeof(uint64_t));
 			*creation_hash_key = hash_key;
 
 			validator = uspoof_open(&uc_err);
@@ -2344,7 +2374,7 @@ lua_util_is_utf_outside_range(lua_State *L)
 								   0, 0);
 		}
 
-		gint32 pos = 0;
+		int32_t pos = 0;
 		ret = uspoof_checkUTF8(validator, t->start, t->len, &pos,
 							   &uc_err);
 	}
@@ -2358,11 +2388,11 @@ lua_util_is_utf_outside_range(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_get_hostname(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gchar *hostbuf;
+	char *hostbuf;
 	gsize hostlen;
 
 	hostlen = sysconf(_SC_HOST_NAME_MAX);
@@ -2383,33 +2413,37 @@ lua_util_get_hostname(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_parse_content_type(lua_State *L)
 {
 	return lua_parsers_parse_content_type(L);
 }
 
 
-static gint
+static int
 lua_util_mime_header_encode(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gsize len;
-	const gchar *hdr = luaL_checklstring(L, 1, &len);
-	gchar *encoded;
+	struct rspamd_lua_text *hdr = lua_check_text_or_string(L, 1);
+	char *encoded;
+	bool is_structured = false;
 
 	if (!hdr) {
 		return luaL_error(L, "invalid arguments");
 	}
 
-	encoded = rspamd_mime_header_encode(hdr, len);
+	if (lua_isboolean(L, 2)) {
+		is_structured = lua_toboolean(L, 2);
+	}
+
+	encoded = rspamd_mime_header_encode(hdr->start, hdr->len, is_structured);
 	lua_pushstring(L, encoded);
 	g_free(encoded);
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_is_valid_utf8(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -2435,11 +2469,11 @@ lua_util_is_valid_utf8(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_has_obscured_unicode(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gint32 i = 0, prev_i;
+	int32_t i = 0, prev_i;
 	UChar32 uc;
 
 	struct rspamd_lua_text *t = lua_check_text_or_string(L, 1);
@@ -2464,17 +2498,17 @@ lua_util_has_obscured_unicode(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_readline(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *prompt = "";
-	gchar *input;
+	const char *prompt = "";
+	char *input = NULL;
 
 	if (lua_type(L, 1) == LUA_TSTRING) {
 		prompt = lua_tostring(L, 1);
 	}
-#ifdef WITH_LUA_REPL
+
 	static Replxx *rx_instance = NULL;
 
 	if (rx_instance == NULL) {
@@ -2483,7 +2517,7 @@ lua_util_readline(lua_State *L)
 		replxx_history_add(rx_instance, "");
 	}
 
-	input = (gchar *) replxx_input(rx_instance, prompt);
+	input = (char *) replxx_input(rx_instance, prompt);
 
 	if (input) {
 		lua_pushstring(L, input);
@@ -2491,35 +2525,15 @@ lua_util_readline(lua_State *L)
 	else {
 		lua_pushnil(L);
 	}
-#else
-	size_t linecap = 0;
-	ssize_t linelen;
-
-	fprintf(stdout, "%s ", prompt);
-
-	linelen = getline(&input, &linecap, stdin);
-
-	if (linelen > 0) {
-		if (input[linelen - 1] == '\n') {
-			linelen--;
-		}
-
-		lua_pushlstring(L, input, linelen);
-		free(input);
-	}
-	else {
-		lua_pushnil(L);
-	}
-#endif
 
 	return 1;
 }
 
-static gint
+static int
 lua_util_readpassphrase(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	gchar test_password[8192];
+	char test_password[8192];
 	gsize r;
 
 	r = rspamd_read_passphrase(test_password, sizeof(test_password), 0, NULL);
@@ -2537,12 +2551,12 @@ lua_util_readpassphrase(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_file_exists(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *fname = luaL_checkstring(L, 1);
-	gint serrno;
+	const char *fname = luaL_checkstring(L, 1);
+	int serrno;
 
 	if (fname) {
 		if (access(fname, R_OK) == -1) {
@@ -2562,13 +2576,13 @@ lua_util_file_exists(lua_State *L)
 	return 2;
 }
 
-static gint
+static int
 lua_util_mkdir(lua_State *L)
 {
 	LUA_TRACE_POINT;
-	const gchar *dname = luaL_checkstring(L, 1);
+	const char *dname = luaL_checkstring(L, 1);
 	gboolean recursive = FALSE;
-	gint r = -1;
+	int r = -1;
 
 	if (dname) {
 		if (lua_isboolean(L, 2)) {
@@ -2626,14 +2640,14 @@ lua_util_mkdir(lua_State *L)
 }
 
 
-static gint
+static int
 lua_util_umask(lua_State *L)
 {
 	LUA_TRACE_POINT;
 	mode_t mask = 0, old;
 
 	if (lua_type(L, 1) == LUA_TSTRING) {
-		const gchar *str = lua_tostring(L, 1);
+		const char *str = lua_tostring(L, 1);
 
 		if (str[0] == '0') {
 			/* e.g. '022' */
@@ -2658,7 +2672,7 @@ lua_util_umask(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_util_isatty(lua_State *L)
 {
 	LUA_TRACE_POINT;
@@ -3411,15 +3425,15 @@ lua_util_unpack(lua_State *L)
 static int
 lua_util_btc_polymod(lua_State *L)
 {
-	guint64 c = 1;
+	uint64_t c = 1;
 
 	if (lua_type(L, 1) != LUA_TTABLE) {
 		return luaL_error(L, "invalid arguments");
 	}
 
 	for (lua_pushnil(L); lua_next(L, 1); lua_pop(L, 1)) {
-		guint8 c0 = c >> 35;
-		guint64 d = lua_tointeger(L, -1);
+		uint8_t c0 = c >> 35;
+		uint64_t d = lua_tointeger(L, -1);
 
 		c = ((c & 0x07ffffffff) << 5) ^ d;
 
@@ -3447,7 +3461,7 @@ lua_util_parse_smtp_date(lua_State *L)
 }
 
 
-static gint
+static int
 lua_load_util(lua_State *L)
 {
 	lua_newtable(L);
@@ -3456,7 +3470,7 @@ lua_load_util(lua_State *L)
 	return 1;
 }
 
-static gint
+static int
 lua_load_int64(lua_State *L)
 {
 	lua_newtable(L);
@@ -3468,9 +3482,9 @@ lua_load_int64(lua_State *L)
 
 void luaopen_util(lua_State *L)
 {
-	rspamd_lua_new_class(L, "rspamd{ev_base}", ev_baselib_m);
+	rspamd_lua_new_class(L, rspamd_ev_base_classname, ev_baselib_m);
 	lua_pop(L, 1);
-	rspamd_lua_new_class(L, "rspamd{int64}", int64lib_m);
+	rspamd_lua_new_class(L, rspamd_int64_classname, int64lib_m);
 	lua_pop(L, 1);
 	rspamd_lua_add_preload(L, "rspamd_util", lua_load_util);
 	rspamd_lua_add_preload(L, "rspamd_int64", lua_load_int64);
@@ -3479,8 +3493,8 @@ void luaopen_util(lua_State *L)
 static int
 lua_int64_tostring(lua_State *L)
 {
-	gint64 n = lua_check_int64(L, 1);
-	gchar buf[32];
+	int64_t n = lua_check_int64(L, 1);
+	char buf[32];
 	bool is_signed = false;
 
 	if (lua_isboolean(L, 2)) {
@@ -3504,7 +3518,7 @@ lua_int64_fromstring(lua_State *L)
 	struct rspamd_lua_text *t = lua_check_text_or_string(L, 1);
 
 	if (t && t->len > 0) {
-		guint64 u64;
+		uint64_t u64;
 		const char *p = t->start;
 		gsize len = t->len;
 		bool neg = false;
@@ -3529,8 +3543,8 @@ lua_int64_fromstring(lua_State *L)
 			return 2;
 		}
 
-		gint64 *i64_p = lua_newuserdata(L, sizeof(gint64));
-		rspamd_lua_setclass(L, "rspamd{int64}", -1);
+		int64_t *i64_p = lua_newuserdata(L, sizeof(int64_t));
+		rspamd_lua_setclass(L, rspamd_int64_classname, -1);
 		memcpy(i64_p, &u64, sizeof(u64));
 
 		if (neg) {
@@ -3546,8 +3560,8 @@ lua_int64_fromstring(lua_State *L)
 static int
 lua_int64_tonumber(lua_State *L)
 {
-	gint64 n = lua_check_int64(L, 1);
-	gdouble d;
+	int64_t n = lua_check_int64(L, 1);
+	double d;
 
 	d = n;
 	lua_pushinteger(L, d);
@@ -3558,8 +3572,8 @@ lua_int64_tonumber(lua_State *L)
 static int
 lua_int64_hex(lua_State *L)
 {
-	gint64 n = lua_check_int64(L, 1);
-	gchar buf[32];
+	int64_t n = lua_check_int64(L, 1);
+	char buf[32];
 
 	rspamd_snprintf(buf, sizeof(buf), "%XL", n);
 	lua_pushstring(L, buf);
@@ -3583,3 +3597,106 @@ lua_ev_base_loop(lua_State *L)
 
 	return 1;
 }
+
+static int
+lua_ev_base_update_time(lua_State *L)
+{
+	struct ev_loop *ev_base;
+
+	ev_base = lua_check_ev_base(L, 1);
+	ev_now_update_if_cheap(ev_base);
+
+	lua_pushnumber(L, ev_time());
+
+	return 1;
+}
+
+static int
+lua_ev_base_timestamp(lua_State *L)
+{
+	struct ev_loop *ev_base;
+
+	ev_base = lua_check_ev_base(L, 1);
+	lua_pushnumber(L, ev_now(ev_base));
+
+	return 1;
+}
+
+static int
+lua_ev_base_pending_events(lua_State *L)
+{
+	struct ev_loop *ev_base;
+
+	ev_base = lua_check_ev_base(L, 1);
+	lua_pushnumber(L, ev_pending_count(ev_base));
+
+	return 1;
+}
+
+struct rspamd_ev_base_cbdata {
+	lua_State *L;
+	int cbref;
+	ev_timer ev;
+};
+
+static void
+lua_ev_base_cb(struct ev_loop *loop, struct ev_timer *t, int events)
+{
+	struct rspamd_ev_base_cbdata *cbdata = (struct rspamd_ev_base_cbdata *) t->data;
+	lua_State *L;
+	bool schedule_more = false;
+
+	L = cbdata->L;
+
+	lua_pushcfunction(L, &rspamd_lua_traceback);
+	int err_idx = lua_gettop(L);
+	lua_rawgeti(L, LUA_REGISTRYINDEX, cbdata->cbref);
+
+	if (lua_pcall(L, 0, 1, err_idx) != 0) {
+		msg_err("call to periodic "
+				"script failed: %s",
+				lua_tostring(L, -1));
+	}
+	else {
+		if (lua_isnumber(L, -1)) {
+			schedule_more = true;
+			ev_timer_set(&cbdata->ev, lua_tonumber(L, -1), 0.0);
+		}
+	}
+
+	if (schedule_more) {
+		ev_timer_again(loop, t);
+	}
+	else {
+		/* Cleanup */
+		ev_timer_stop(loop, t);
+		luaL_unref(L, LUA_REGISTRYINDEX, cbdata->cbref);
+		g_free(cbdata);
+	}
+}
+
+static int
+lua_ev_base_add_timer(lua_State *L)
+{
+	struct ev_loop *ev_base;
+
+	ev_base = lua_check_ev_base(L, 1);
+	if (!lua_isfunction(L, 3)) {
+		return luaL_error(L, "invalid arguments: callback expected");
+	}
+
+	if (!lua_isnumber(L, 2)) {
+		return luaL_error(L, "invalid arguments: timeout expected");
+	}
+
+	struct rspamd_ev_base_cbdata *cbdata = g_malloc(sizeof(*cbdata));
+	cbdata->L = L;
+	lua_pushvalue(L, 3);
+	cbdata->ev.data = cbdata;
+	cbdata->cbref = luaL_ref(L, LUA_REGISTRYINDEX);
+
+	ev_timer_init(&cbdata->ev, lua_ev_base_cb, lua_tonumber(L, 2), 0.0);
+	ev_timer_start(ev_base, &cbdata->ev);
+
+	return 0;
+}
\ No newline at end of file
