@@ -29,6 +29,7 @@ local rspamd_ip = require "rspamd_ip"
 local lua_util = require "lua_util"
 local lua_selectors = require "lua_selectors"
 local lua_maps = require "lua_maps"
+local lua_mime = require "lua_mime"
 local redis_params
 local fun = require "fun"
 local N = 'multimap'
@@ -453,19 +454,19 @@ local function apply_content_filter(task, filter)
     return { task:get_raw_headers() }
   elseif filter == 'text' then
     local ret = {}
-    for _, p in ipairs(task:get_text_parts()) do
+    for _, p in ipairs(lua_mime.get_distinct_text_parts(task)) do
       table.insert(ret, p:get_content())
     end
     return ret
   elseif filter == 'rawtext' then
     local ret = {}
-    for _, p in ipairs(task:get_text_parts()) do
+    for _, p in ipairs(lua_mime.get_distinct_text_parts(task)) do
       table.insert(ret, p:get_content('raw_parsed'))
     end
     return ret
   elseif filter == 'oneline' then
     local ret = {}
-    for _, p in ipairs(task:get_text_parts()) do
+    for _, p in ipairs(lua_mime.get_distinct_text_parts(task)) do
       table.insert(ret, p:get_content_oneline())
     end
     return ret
@@ -878,21 +879,81 @@ local function multimap_callback(task, rule)
       end
     end,
     rcpt = function()
-      if task:has_recipients('smtp') then
-        local rcpts = task:get_recipients('smtp')
-        match_addr(rule, rcpts)
-      elseif task:has_recipients('mime') then
+      local extract_from = rule.extract_from or 'default'
+
+      if extract_from == 'mime' then
         local rcpts = task:get_recipients('mime')
-        match_addr(rule, rcpts)
+        if rcpts then
+          lua_util.debugm(N, task, 'checking mime rcpts against the map')
+          match_addr(rule, rcpts)
+        end
+      elseif extract_from == 'smtp' then
+        local rcpts = task:get_recipients('smtp')
+        if rcpts then
+          lua_util.debugm(N, task, 'checking smtp rcpts against the map')
+          match_addr(rule, rcpts)
+        end
+      elseif extract_from == 'both' then
+        local rcpts = task:get_recipients('smtp')
+        if rcpts then
+          lua_util.debugm(N, task, 'checking smtp rcpts against the map')
+          match_addr(rule, rcpts)
+        end
+        rcpts = task:get_recipients('mime')
+        if rcpts then
+          lua_util.debugm(N, task, 'checking mime rcpts against the map')
+          match_addr(rule, rcpts)
+        end
+      else
+        -- Default algorithm
+        if task:has_recipients('smtp') then
+          local rcpts = task:get_recipients('smtp')
+          lua_util.debugm(N, task, 'checking smtp rcpts against the map')
+          match_addr(rule, rcpts)
+        elseif task:has_recipients('mime') then
+          local rcpts = task:get_recipients('mime')
+          lua_util.debugm(N, task, 'checking mime rcpts against the map')
+          match_addr(rule, rcpts)
+        end
       end
     end,
     from = function()
-      if task:has_from('smtp') then
-        local from = task:get_from('smtp')
-        match_addr(rule, from)
-      elseif task:has_from('mime') then
+      local extract_from = rule.extract_from or 'default'
+
+      if extract_from == 'mime' then
         local from = task:get_from('mime')
-        match_addr(rule, from)
+        if from then
+          lua_util.debugm(N, task, 'checking mime from against the map')
+          match_addr(rule, from)
+        end
+      elseif extract_from == 'smtp' then
+        local from = task:get_from('smtp')
+        if from then
+          lua_util.debugm(N, task, 'checking smtp from against the map')
+          match_addr(rule, from)
+        end
+      elseif extract_from == 'both' then
+        local from = task:get_from('smtp')
+        if from then
+          lua_util.debugm(N, task, 'checking smtp from against the map')
+          match_addr(rule, from)
+        end
+        from = task:get_from('mime')
+        if from then
+          lua_util.debugm(N, task, 'checking mime from against the map')
+          match_addr(rule, from)
+        end
+      else
+        -- Default algorithm
+        if task:has_from('smtp') then
+          local from = task:get_from('smtp')
+          lua_util.debugm(N, task, 'checking smtp from against the map')
+          match_addr(rule, from)
+        elseif task:has_from('mime') then
+          local from = task:get_from('mime')
+          lua_util.debugm(N, task, 'checking mime from against the map')
+          match_addr(rule, from)
+        end
       end
     end,
     helo = function()
@@ -1058,22 +1119,23 @@ local function multimap_on_load_gen(rule)
     lua_util.debugm(N, rspamd_config, "loaded map object for rule %s", rule['symbol'])
     local known_symbols = {}
     rule.map_obj:foreach(function(key, value)
+      local mult = rule.score or 1.0
       local r, symbol, score, _ = parse_multimap_value(rule, value)
 
       if r and symbol and not known_symbols[symbol] then
         lua_util.debugm(N, rspamd_config, "%s: adding new symbol %s (score = %s), triggered by %s",
             rule.symbol, symbol, score, key)
         rspamd_config:register_symbol {
-          name = value,
+          name = symbol,
           parent = rule.callback_id,
           type = 'virtual',
-          score = score,
+          score = score * mult,
         }
         rspamd_config:set_metric_symbol({
           group = N,
-          score = 1.0, -- In future, we will parse score from `get_value` and use it as multiplier
+          score = mult, -- In future, we will parse score from `get_value` and use it as multiplier
           description = 'Automatic symbol generated by rule: ' .. rule.symbol,
-          name = value,
+          name = symbol,
         })
         known_symbols[value] = true
       end
@@ -1205,6 +1267,7 @@ local function add_multimap_rule(key, newrule)
         {
           rules = newrule.rules,
           expression = newrule.expression,
+          description = newrule.description,
           on_load = newrule.dynamic_symbols and multimap_on_load_gen(newrule) or nil,
         }, N, 'Combined map for ' .. newrule.symbol)
     if not newrule.combined then
@@ -1219,7 +1282,7 @@ local function add_multimap_rule(key, newrule)
       if newrule.map_obj then
         ret = true
       else
-        rspamd_logger.warnx(rspamd_config, 'Cannot add rule: map doesn\'t exists: %1',
+        rspamd_logger.warnx(rspamd_config, 'Cannot add rule: map doesn\'t exists: %s',
             newrule['map'])
       end
     elseif newrule['type'] == 'received' then
@@ -1240,7 +1303,7 @@ local function add_multimap_rule(key, newrule)
         if newrule.map_obj then
           ret = true
         else
-          rspamd_logger.warnx(rspamd_config, 'Cannot add rule: map doesn\'t exists: %1',
+          rspamd_logger.warnx(rspamd_config, 'Cannot add rule: map doesn\'t exists: %s',
               newrule['map'])
         end
       else
@@ -1249,7 +1312,7 @@ local function add_multimap_rule(key, newrule)
         if newrule.map_obj then
           ret = true
         else
-          rspamd_logger.warnx(rspamd_config, 'Cannot add rule: map doesn\'t exists: %1',
+          rspamd_logger.warnx(rspamd_config, 'Cannot add rule: map doesn\'t exists: %s',
               newrule['map'])
         end
       end
@@ -1265,11 +1328,14 @@ local function add_multimap_rule(key, newrule)
       if newrule.map_obj then
         ret = true
       else
-        rspamd_logger.warnx(rspamd_config, 'Cannot add rule: map doesn\'t exists: %1',
+        rspamd_logger.warnx(rspamd_config, 'Cannot add rule: map doesn\'t exists: %s',
             newrule['map'])
       end
     elseif newrule['type'] == 'dnsbl' then
       ret = true
+    else
+      rspamd_logger.errx(rspamd_config, 'cannot add rule %s: invalid type %s',
+          key, newrule['type'])
     end
   end
 
