@@ -1,5 +1,5 @@
 /*
- * Copyright 2024 Vsevolod Stakhov
+ * Copyright 2025 Vsevolod Stakhov
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -38,6 +38,7 @@
 #include "libmime/lang_detection.h"
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_NETINET_TCP_H
 #include <netinet/tcp.h> /* for TCP_NODELAY */
@@ -84,36 +85,50 @@ worker_t rspamd_proxy_worker = {
 	RSPAMD_WORKER_SOCKET_TCP, /* TCP socket */
 	RSPAMD_WORKER_VER};
 
+enum rspamd_proxy_log_tag_type {
+	RSPAMD_PROXY_LOG_TAG_SESSION = 0, /* Use session mempool tag (default) */
+	RSPAMD_PROXY_LOG_TAG_QUEUE_ID,    /* Use Queue-ID from client message */
+	RSPAMD_PROXY_LOG_TAG_NONE,        /* Skip log tag passing */
+};
+
 struct rspamd_http_upstream {
-	gchar *name;
-	gchar *settings_id;
+	char *name;
+	char *settings_id;
 	struct upstream_list *u;
 	struct rspamd_cryptobox_pubkey *key;
-	gdouble timeout;
-	gint parser_from_ref;
-	gint parser_to_ref;
+	double timeout;
+	int parser_from_ref;
+	int parser_to_ref;
 	gboolean local;
 	gboolean self_scan;
 	gboolean compress;
+	gboolean ssl;
+	gboolean keepalive; /* Whether to use keepalive for this upstream */
+	enum rspamd_proxy_log_tag_type log_tag_type;
+	ucl_object_t *extra_headers;
 };
 
 struct rspamd_http_mirror {
-	gchar *name;
-	gchar *settings_id;
+	char *name;
+	char *settings_id;
 	struct upstream_list *u;
 	struct rspamd_cryptobox_pubkey *key;
-	gdouble prob;
-	gdouble timeout;
-	gint parser_from_ref;
-	gint parser_to_ref;
+	double prob;
+	double timeout;
+	int parser_from_ref;
+	int parser_to_ref;
 	gboolean local;
 	gboolean compress;
+	gboolean ssl;
+	gboolean keepalive; /* Whether to use keepalive for this mirror */
+	enum rspamd_proxy_log_tag_type log_tag_type;
+	ucl_object_t *extra_headers;
 };
 
-static const guint64 rspamd_rspamd_proxy_magic = 0xcdeb4fd1fc351980ULL;
+static const uint64_t rspamd_rspamd_proxy_magic = 0xcdeb4fd1fc351980ULL;
 
 struct rspamd_proxy_ctx {
-	guint64 magic;
+	uint64_t magic;
 	/* Events base */
 	struct ev_loop *event_loop;
 	/* DNS resolver */
@@ -121,7 +136,7 @@ struct rspamd_proxy_ctx {
 	/* Config */
 	struct rspamd_config *cfg;
 	/* END OF COMMON PART */
-	gdouble timeout;
+	double timeout;
 	/* Encryption key for clients */
 	struct rspamd_cryptobox_keypair *key;
 	/* HTTP context */
@@ -136,7 +151,7 @@ struct rspamd_proxy_ctx {
 	/* Array of callback functions called on end of scan to compare results */
 	GArray *cmp_refs;
 	/* Maximum count for retries */
-	guint max_retries;
+	unsigned int max_retries;
 	gboolean encrypted_only;
 	/* If we have self_scanning backends, we need to work as a normal worker */
 	gboolean has_self_scan;
@@ -147,17 +162,22 @@ struct rspamd_proxy_ctx {
 	/* Quarantine messages instead of rejecting them */
 	gboolean quarantine_on_reject;
 	/* Milter spam header */
-	gchar *spam_header;
+	char *spam_header;
 	/* CA name that can be used for client certificates */
-	gchar *client_ca_name;
-	/* Milter rejection message */
-	gchar *reject_message;
+	char *client_ca_name;
+	/* Milter messages */
+	char *reject_message;
+	char *quarantine_message;
+	char *tempfail_message;
 	/* Sessions cache */
 	void *sessions_cache;
 	struct rspamd_milter_context milter_ctx;
 	/* Language detector */
 	struct rspamd_lang_detector *lang_det;
-	gdouble task_timeout;
+	double task_timeout;
+	/* Default log tag type for worker */
+	enum rspamd_proxy_log_tag_type log_tag_type;
+	struct rspamd_main *srv;
 };
 
 enum rspamd_backend_flags {
@@ -169,19 +189,19 @@ enum rspamd_backend_flags {
 struct rspamd_proxy_session;
 
 struct rspamd_proxy_backend_connection {
-	const gchar *name;
+	const char *name;
 	struct rspamd_cryptobox_keypair *local_key;
 	struct rspamd_cryptobox_pubkey *remote_key;
 	struct upstream *up;
 	struct rspamd_http_connection *backend_conn;
 	ucl_object_t *results;
-	const gchar *err;
+	const char *err;
 	struct rspamd_proxy_session *s;
-	gint backend_sock;
+	int backend_sock;
 	ev_tstamp timeout;
 	enum rspamd_backend_flags flags;
-	gint parser_from_ref;
-	gint parser_to_ref;
+	int parser_from_ref;
+	int parser_to_ref;
 	struct rspamd_task *task;
 };
 
@@ -200,16 +220,17 @@ struct rspamd_proxy_session {
 	struct rspamd_milter_session *client_milter_conn;
 	struct rspamd_http_upstream *backend;
 	gpointer map;
-	gchar *fname;
+	char *fname;
 	gpointer shmem_ref;
 	struct rspamd_proxy_backend_connection *master_conn;
 	struct rspamd_http_message *client_message;
 	GPtrArray *mirror_conns;
 	gsize map_len;
-	gint client_sock;
+	int client_sock;
 	enum rspamd_proxy_legacy_support legacy_support;
-	gint retries;
+	int retries;
 	ref_entry_t ref;
+	gboolean use_keepalive; /* Whether to use keepalive for this session */
 };
 
 static gboolean proxy_send_master_message(struct rspamd_proxy_session *session);
@@ -220,13 +241,84 @@ rspamd_proxy_quark(void)
 	return g_quark_from_static_string("rspamd-proxy");
 }
 
+static enum rspamd_proxy_log_tag_type
+rspamd_proxy_parse_log_tag_type(const char *str)
+{
+	if (str == NULL) {
+		return RSPAMD_PROXY_LOG_TAG_SESSION;
+	}
+
+	if (g_ascii_strcasecmp(str, "session") == 0 ||
+		g_ascii_strcasecmp(str, "session_tag") == 0) {
+		return RSPAMD_PROXY_LOG_TAG_SESSION;
+	}
+	else if (g_ascii_strcasecmp(str, "queue_id") == 0 ||
+			 g_ascii_strcasecmp(str, "queue-id") == 0) {
+		return RSPAMD_PROXY_LOG_TAG_QUEUE_ID;
+	}
+	else if (g_ascii_strcasecmp(str, "none") == 0 ||
+			 g_ascii_strcasecmp(str, "skip") == 0) {
+		return RSPAMD_PROXY_LOG_TAG_NONE;
+	}
+
+	/* Default to session tag for unknown values */
+	return RSPAMD_PROXY_LOG_TAG_SESSION;
+}
+
+static void
+rspamd_proxy_add_log_tag_header(struct rspamd_http_message *msg,
+								struct rspamd_proxy_session *session,
+								enum rspamd_proxy_log_tag_type log_tag_type)
+{
+	const rspamd_ftok_t *queue_id_hdr;
+
+	switch (log_tag_type) {
+	case RSPAMD_PROXY_LOG_TAG_SESSION:
+		/* Use session mempool tag (current behavior) */
+		rspamd_http_message_add_header_len(msg, LOG_TAG_HEADER, session->pool->tag.uid,
+										   strnlen(session->pool->tag.uid, sizeof(session->pool->tag.uid)));
+		break;
+
+	case RSPAMD_PROXY_LOG_TAG_QUEUE_ID:
+		/* Try to extract Queue-ID from client message */
+		if (session->client_message) {
+			queue_id_hdr = rspamd_http_message_find_header(session->client_message, QUEUE_ID_HEADER);
+			if (queue_id_hdr) {
+				rspamd_http_message_add_header_len(msg, LOG_TAG_HEADER,
+												   queue_id_hdr->begin, queue_id_hdr->len);
+			}
+			/* If no Queue-ID found, fall back to session tag */
+			else {
+				rspamd_http_message_add_header_len(msg, LOG_TAG_HEADER, session->pool->tag.uid,
+												   strnlen(session->pool->tag.uid, sizeof(session->pool->tag.uid)));
+			}
+		}
+		else {
+			/* No client message, fall back to session tag */
+			rspamd_http_message_add_header_len(msg, LOG_TAG_HEADER, session->pool->tag.uid,
+											   strnlen(session->pool->tag.uid, sizeof(session->pool->tag.uid)));
+		}
+		break;
+
+	case RSPAMD_PROXY_LOG_TAG_NONE:
+		/* Skip adding log tag header */
+		break;
+
+	default:
+		/* Fall back to session tag for unknown types */
+		rspamd_http_message_add_header_len(msg, LOG_TAG_HEADER, session->pool->tag.uid,
+										   strnlen(session->pool->tag.uid, sizeof(session->pool->tag.uid)));
+		break;
+	}
+}
+
 static gboolean
 rspamd_proxy_parse_lua_parser(lua_State *L, const ucl_object_t *obj,
-							  gint *ref_from, gint *ref_to, GError **err)
+							  int *ref_from, int *ref_to, GError **err)
 {
-	const gchar *lua_script;
+	const char *lua_script;
 	gsize slen;
-	gint err_idx, ref_idx;
+	int err_idx, ref_idx;
 	gboolean has_ref = FALSE;
 
 	g_assert(obj != NULL);
@@ -388,11 +480,12 @@ rspamd_proxy_parse_upstream(rspamd_mempool_t *pool,
 	up->parser_from_ref = -1;
 	up->parser_to_ref = -1;
 	up->timeout = ctx->timeout;
+	up->log_tag_type = ctx->log_tag_type; /* Inherit from worker default */
 
 	elt = ucl_object_lookup(obj, "key");
 	if (elt != NULL) {
 		up->key = rspamd_pubkey_from_base32(ucl_object_tostring(elt), 0,
-											RSPAMD_KEYPAIR_KEX, RSPAMD_CRYPTOBOX_MODE_25519);
+											RSPAMD_KEYPAIR_KEX);
 
 		if (up->key == NULL) {
 			g_set_error(err, rspamd_proxy_quark(), 100,
@@ -416,6 +509,21 @@ rspamd_proxy_parse_upstream(rspamd_mempool_t *pool,
 		up->compress = TRUE;
 	}
 
+	elt = ucl_object_lookup(obj, "ssl");
+	if (elt && ucl_object_toboolean(elt)) {
+		up->ssl = TRUE;
+	}
+
+	elt = ucl_object_lookup_any(obj, "keepalive", "keep_alive", NULL);
+	if (elt && ucl_object_toboolean(elt)) {
+		up->keepalive = TRUE;
+	}
+
+	elt = ucl_object_lookup_any(obj, "keepalive", "keep_alive", NULL);
+	if (elt && ucl_object_toboolean(elt)) {
+		up->keepalive = TRUE;
+	}
+
 	elt = ucl_object_lookup(obj, "hosts");
 
 	if (elt == NULL && !up->self_scan) {
@@ -465,6 +573,27 @@ rspamd_proxy_parse_upstream(rspamd_mempool_t *pool,
 		up->settings_id = rspamd_mempool_strdup(pool, ucl_object_tostring(elt));
 	}
 
+	elt = ucl_object_lookup(obj, "extra_headers");
+	if (elt && ucl_object_type(elt) == UCL_OBJECT) {
+		up->extra_headers = ucl_object_ref(elt);
+		rspamd_mempool_add_destructor(pool,
+									  (rspamd_mempool_destruct_t) ucl_object_unref,
+									  up->extra_headers);
+	}
+
+	elt = ucl_object_lookup(obj, "extra_headers");
+	if (elt && ucl_object_type(elt) == UCL_OBJECT) {
+		up->extra_headers = ucl_object_ref(elt);
+		rspamd_mempool_add_destructor(pool,
+									  (rspamd_mempool_destruct_t) ucl_object_unref,
+									  up->extra_headers);
+	}
+
+	elt = ucl_object_lookup_any(obj, "log_tag", "log_tag_type", NULL);
+	if (elt && ucl_object_type(elt) == UCL_STRING) {
+		up->log_tag_type = rspamd_proxy_parse_log_tag_type(ucl_object_tostring(elt));
+	}
+
 	/*
 	 * Accept lua function here in form
 	 * fun :: String -> UCL
@@ -564,11 +693,12 @@ rspamd_proxy_parse_mirror(rspamd_mempool_t *pool,
 	up->parser_to_ref = -1;
 	up->parser_from_ref = -1;
 	up->timeout = ctx->timeout;
+	up->log_tag_type = ctx->log_tag_type; /* Inherit from worker default */
 
 	elt = ucl_object_lookup(obj, "key");
 	if (elt != NULL) {
 		up->key = rspamd_pubkey_from_base32(ucl_object_tostring(elt), 0,
-											RSPAMD_KEYPAIR_KEX, RSPAMD_CRYPTOBOX_MODE_25519);
+											RSPAMD_KEYPAIR_KEX);
 
 		if (up->key == NULL) {
 			g_set_error(err, rspamd_proxy_quark(), 100,
@@ -644,6 +774,11 @@ rspamd_proxy_parse_mirror(rspamd_mempool_t *pool,
 		up->settings_id = rspamd_mempool_strdup(pool, ucl_object_tostring(elt));
 	}
 
+	elt = ucl_object_lookup_any(obj, "log_tag", "log_tag_type", NULL);
+	if (elt && ucl_object_type(elt) == UCL_STRING) {
+		up->log_tag_type = rspamd_proxy_parse_log_tag_type(ucl_object_tostring(elt));
+	}
+
 	g_ptr_array_add(ctx->mirrors, up);
 
 	return TRUE;
@@ -663,9 +798,9 @@ rspamd_proxy_parse_script(rspamd_mempool_t *pool,
 	struct rspamd_proxy_ctx *ctx;
 	struct rspamd_rcl_struct_parser *pd = ud;
 	lua_State *L;
-	const gchar *lua_script;
+	const char *lua_script;
 	gsize slen;
-	gint err_idx, ref_idx;
+	int err_idx, ref_idx;
 	struct stat st;
 
 	ctx = pd->user_struct;
@@ -743,6 +878,29 @@ err:
 	return FALSE;
 }
 
+static gboolean
+rspamd_proxy_parse_log_tag_worker_option(rspamd_mempool_t *pool,
+										 const ucl_object_t *obj,
+										 gpointer ud,
+										 struct rspamd_rcl_section *section,
+										 GError **err)
+{
+	struct rspamd_proxy_ctx *ctx;
+	struct rspamd_rcl_struct_parser *pd = ud;
+
+	ctx = pd->user_struct;
+
+	if (ucl_object_type(obj) != UCL_STRING) {
+		g_set_error(err, rspamd_proxy_quark(), 100,
+					"log_tag_type option must be a string");
+		return FALSE;
+	}
+
+	ctx->log_tag_type = rspamd_proxy_parse_log_tag_type(ucl_object_tostring(obj));
+
+	return TRUE;
+}
+
 gpointer
 init_rspamd_proxy(struct rspamd_config *cfg)
 {
@@ -763,11 +921,12 @@ init_rspamd_proxy(struct rspamd_config *cfg)
 								  (rspamd_mempool_destruct_t) rspamd_ptr_array_free_hard, ctx->mirrors);
 	ctx->cfg = cfg;
 	ctx->lua_state = cfg->lua_state;
-	ctx->cmp_refs = g_array_new(FALSE, FALSE, sizeof(gint));
+	ctx->cmp_refs = g_array_new(FALSE, FALSE, sizeof(int));
 	rspamd_mempool_add_destructor(cfg->cfg_pool,
 								  (rspamd_mempool_destruct_t) rspamd_array_free_hard, ctx->cmp_refs);
 	ctx->max_retries = DEFAULT_RETRIES;
 	ctx->spam_header = RSPAMD_MILTER_SPAM_HEADER;
+	ctx->log_tag_type = RSPAMD_PROXY_LOG_TAG_SESSION; /* Default to session tag */
 
 	rspamd_rcl_register_worker_option(cfg,
 									  type,
@@ -874,6 +1033,32 @@ init_rspamd_proxy(struct rspamd_config *cfg)
 									  G_STRUCT_OFFSET(struct rspamd_proxy_ctx, reject_message),
 									  0,
 									  "Use custom rejection message");
+	rspamd_rcl_register_worker_option(cfg,
+									  type,
+									  "quarantine_message",
+									  rspamd_rcl_parse_struct_string,
+									  ctx,
+									  G_STRUCT_OFFSET(struct rspamd_proxy_ctx, quarantine_message),
+									  0,
+									  "Use custom quarantine message");
+	rspamd_rcl_register_worker_option(cfg,
+									  type,
+									  "tempfail_message",
+									  rspamd_rcl_parse_struct_string,
+									  ctx,
+									  G_STRUCT_OFFSET(struct rspamd_proxy_ctx, tempfail_message),
+									  0,
+									  "Use custom tempfail message");
+
+	/* We need a custom parser for log_tag_type as it's an enum */
+	rspamd_rcl_register_worker_option(cfg,
+									  type,
+									  "log_tag_type",
+									  rspamd_proxy_parse_log_tag_worker_option,
+									  ctx,
+									  0,
+									  0,
+									  "Log tag type: session (default), queue_id, or none");
 
 	return ctx;
 }
@@ -885,7 +1070,11 @@ proxy_backend_close_connection(struct rspamd_proxy_backend_connection *conn)
 		if (conn->backend_conn) {
 			rspamd_http_connection_reset(conn->backend_conn);
 			rspamd_http_connection_unref(conn->backend_conn);
-			close(conn->backend_sock);
+
+			if (!(conn->s && conn->s->use_keepalive)) {
+				/* Only close socket if we're not using keepalive */
+				close(conn->backend_sock);
+			}
 		}
 
 		conn->flags |= RSPAMD_BACKEND_CLOSED;
@@ -895,14 +1084,14 @@ proxy_backend_close_connection(struct rspamd_proxy_backend_connection *conn)
 static gboolean
 proxy_backend_parse_results(struct rspamd_proxy_session *session,
 							struct rspamd_proxy_backend_connection *conn,
-							lua_State *L, gint parser_ref,
+							lua_State *L, int parser_ref,
 							struct rspamd_http_message *msg,
 							goffset *body_offset,
 							const rspamd_ftok_t *ct)
 {
 	struct ucl_parser *parser;
-	gint err_idx;
-	const gchar *in = msg->body_buf.begin;
+	int err_idx;
+	const char *in = msg->body_buf.begin;
 	gsize inlen = msg->body_buf.len;
 	const rspamd_ftok_t *offset_hdr;
 
@@ -950,10 +1139,10 @@ proxy_backend_parse_results(struct rspamd_proxy_session *session,
 		RSPAMD_FTOK_ASSIGN(&json_ct, "application/json");
 
 		if (ct && rspamd_ftok_casecmp(ct, &json_ct) == 0) {
-			parser = ucl_parser_new(0);
+			parser = ucl_parser_new(UCL_PARSER_SAFE_FLAGS);
 
 			if (!ucl_parser_add_chunk(parser, in, inlen)) {
-				gchar *encoded;
+				char *encoded;
 
 				encoded = rspamd_encode_base64(in, inlen, 0, NULL);
 				msg_err_session("cannot parse input: %s", ucl_parser_get_error(
@@ -974,10 +1163,10 @@ proxy_backend_parse_results(struct rspamd_proxy_session *session,
 }
 
 static void
-proxy_call_cmp_script(struct rspamd_proxy_session *session, gint cbref)
+proxy_call_cmp_script(struct rspamd_proxy_session *session, int cbref)
 {
-	gint err_idx;
-	guint i;
+	int err_idx;
+	unsigned int i;
 	struct rspamd_proxy_backend_connection *conn;
 	lua_State *L;
 
@@ -1015,7 +1204,7 @@ proxy_call_cmp_script(struct rspamd_proxy_session *session, gint cbref)
 		}
 	}
 
-	gchar log_tag[RSPAMD_LOG_ID_LEN + 1];
+	char log_tag[RSPAMD_LOG_ID_LEN + 1];
 	rspamd_strlcpy(log_tag, session->pool->tag.uid, sizeof(log_tag));
 	lua_pushstring(L, log_tag);
 
@@ -1031,13 +1220,13 @@ proxy_call_cmp_script(struct rspamd_proxy_session *session, gint cbref)
 static void
 proxy_session_dtor(struct rspamd_proxy_session *session)
 {
-	guint i;
-	gint cbref;
+	unsigned int i;
+	int cbref;
 	struct rspamd_proxy_backend_connection *conn;
 
 	if (session->master_conn && session->master_conn->results) {
 		for (i = 0; i < session->ctx->cmp_refs->len; i++) {
-			cbref = g_array_index(session->ctx->cmp_refs, gint, i);
+			cbref = g_array_index(session->ctx->cmp_refs, int, i);
 			proxy_call_cmp_script(session, cbref);
 		}
 	}
@@ -1101,16 +1290,18 @@ proxy_session_dtor(struct rspamd_proxy_session *session)
 		rspamd_mempool_delete(session->pool);
 	}
 
+	session->worker->nconns--;
+
 	g_free(session);
 }
 
 static void
 proxy_request_compress(struct rspamd_http_message *msg)
 {
-	guint flags;
+	unsigned int flags;
 	ZSTD_CCtx *zctx;
 	rspamd_fstring_t *body;
-	const gchar *in;
+	const char *in;
 	gsize inlen;
 
 	flags = rspamd_http_message_get_flags(msg);
@@ -1151,7 +1342,7 @@ static void
 proxy_request_decompress(struct rspamd_http_message *msg)
 {
 	rspamd_fstring_t *body;
-	const gchar *in;
+	const char *in;
 	gsize inlen, outlen, r;
 	ZSTD_DStream *zstream;
 	ZSTD_inBuffer zin;
@@ -1242,7 +1433,7 @@ proxy_check_file(struct rspamd_http_message *msg,
 {
 	const rspamd_ftok_t *tok, *key_tok;
 	rspamd_ftok_t srch;
-	gchar *file_str;
+	char *file_str;
 	GHashTable *query_args;
 	GHashTableIter it;
 	gpointer k, v;
@@ -1355,13 +1546,15 @@ proxy_backend_mirror_error_handler(struct rspamd_http_connection *conn, GError *
 	REF_RELEASE(bk_conn->s);
 }
 
-static gint
+static int
 proxy_backend_mirror_finish_handler(struct rspamd_http_connection *conn,
 									struct rspamd_http_message *msg)
 {
 	struct rspamd_proxy_backend_connection *bk_conn = conn->ud;
 	struct rspamd_proxy_session *session;
 	const rspamd_ftok_t *orig_ct;
+	const rspamd_ftok_t *conn_hdr;
+	gboolean is_keepalive = FALSE;
 
 	session = bk_conn->s;
 
@@ -1377,9 +1570,40 @@ proxy_backend_mirror_finish_handler(struct rspamd_http_connection *conn,
 		bk_conn->err = "cannot parse ucl";
 	}
 
-	msg_info_session("finished mirror connection to %s", bk_conn->name);
+	msg_info_session("finished mirror connection to %s; HTTP code: %d",
+					 bk_conn->name, msg->code);
 	rspamd_upstream_ok(bk_conn->up);
 
+	/* Check if we can use keepalive */
+	conn_hdr = rspamd_http_message_find_header(msg, "Connection");
+	if (conn_hdr) {
+		if (rspamd_substring_search_caseless(conn_hdr->begin, conn_hdr->len,
+											 "keep-alive", 10) != -1) {
+			is_keepalive = TRUE;
+		}
+	}
+
+	if (is_keepalive && session->use_keepalive &&
+		bk_conn->up && session->ctx->http_ctx) {
+		/* Store connection in keepalive pool */
+		const char *up_name = rspamd_upstream_name(bk_conn->up);
+		if (up_name) {
+			rspamd_http_context_prepare_keepalive(session->ctx->http_ctx,
+												  conn, rspamd_upstream_addr_cur(bk_conn->up),
+												  up_name, FALSE);
+			rspamd_http_context_push_keepalive(session->ctx->http_ctx,
+											   conn, msg, session->ctx->event_loop);
+
+			msg_debug_session("pushed mirror connection to %s to keepalive pool",
+							  bk_conn->name);
+
+			/* Mark connection as closed without actually closing it */
+			bk_conn->flags |= RSPAMD_BACKEND_CLOSED;
+			REF_RELEASE(bk_conn->s);
+			return 0;
+		}
+	}
+
 	proxy_backend_close_connection(bk_conn);
 	REF_RELEASE(bk_conn->s);
 
@@ -1389,12 +1613,13 @@ proxy_backend_mirror_finish_handler(struct rspamd_http_connection *conn,
 static void
 proxy_open_mirror_connections(struct rspamd_proxy_session *session)
 {
-	gdouble coin;
+	double coin;
 	struct rspamd_http_mirror *m;
-	guint i;
+	unsigned int i;
 	struct rspamd_proxy_backend_connection *bk_conn;
 	struct rspamd_http_message *msg;
 	GError *err = NULL;
+	const rspamd_inet_addr_t *keepalive_addr;
 
 	coin = rspamd_random_double();
 
@@ -1406,6 +1631,157 @@ proxy_open_mirror_connections(struct rspamd_proxy_session *session)
 			continue;
 		}
 
+		/* Check if we can use keepalive for this mirror */
+		if (m->keepalive && session->ctx->http_ctx) {
+			const char *up_name = NULL;
+			unsigned int port = 0;
+
+			/* Try to find a keepalive connection */
+			if (m->u) {
+				struct upstream *up = rspamd_upstream_get(m->u,
+														  RSPAMD_UPSTREAM_ROUND_ROBIN, NULL, 0);
+				if (up) {
+					up_name = rspamd_upstream_name(up);
+					port = rspamd_inet_address_get_port(rspamd_upstream_addr_cur(up));
+				}
+			}
+
+			if (up_name) {
+				keepalive_addr = rspamd_http_context_has_keepalive(
+					session->ctx->http_ctx, up_name, port, m->ssl);
+
+				if (keepalive_addr) {
+					/* We found a keepalive connection, use it */
+					struct rspamd_http_connection *conn;
+
+					conn = rspamd_http_context_check_keepalive(
+						session->ctx->http_ctx,
+						(rspamd_inet_addr_t *) keepalive_addr,
+						up_name,
+						m->ssl);
+
+					if (conn) {
+						/* We have a keepalive connection, set it up */
+						bk_conn = rspamd_mempool_alloc0(session->pool, sizeof(*bk_conn));
+						bk_conn->s = session;
+						bk_conn->name = m->name;
+						bk_conn->timeout = m->timeout;
+						bk_conn->parser_from_ref = m->parser_from_ref;
+						bk_conn->parser_to_ref = m->parser_to_ref;
+						bk_conn->backend_conn = conn;
+						bk_conn->backend_sock = conn->fd;
+
+						msg = rspamd_http_connection_copy_msg(session->client_message, &err);
+
+						if (msg == NULL) {
+							msg_err_session("cannot copy message to send to a mirror %s: %e",
+											m->name, err);
+							if (err) {
+								g_error_free(err);
+							}
+							continue;
+						}
+
+						if (up_name) {
+							rspamd_http_message_remove_header(msg, "Host");
+							rspamd_http_message_add_header(msg, "Host", up_name);
+						}
+						rspamd_http_message_remove_header(msg, "Connection");
+						rspamd_http_message_add_header(msg, "Connection", "keep-alive");
+
+						if (msg->url->len == 0) {
+							msg->url = rspamd_fstring_append(msg->url, "/check", strlen("/check"));
+						}
+
+						if (m->settings_id != NULL) {
+							rspamd_http_message_remove_header(msg, "Settings-ID");
+							rspamd_http_message_add_header(msg, "Settings-ID", m->settings_id);
+						}
+
+						/* Add extra headers if specified */
+						if (m->extra_headers != NULL) {
+							ucl_object_iter_t it = NULL;
+							const ucl_object_t *cur;
+							const char *key, *value;
+
+							while ((cur = ucl_object_iterate(m->extra_headers, &it, true)) != NULL) {
+								key = ucl_object_key(cur);
+								value = ucl_object_tostring(cur);
+
+								if (key != NULL && value != NULL) {
+									rspamd_http_message_remove_header(msg, key);
+									rspamd_http_message_add_header(msg, key, value);
+								}
+							}
+						}
+
+						/* Add log tag header based on mirror's configuration */
+						rspamd_proxy_add_log_tag_header(msg, session, m->log_tag_type);
+
+						/* Set handlers for the connection */
+						conn->error_handler = proxy_backend_mirror_error_handler;
+						conn->finish_handler = proxy_backend_mirror_finish_handler;
+						conn->ud = bk_conn;
+
+						if (m->key) {
+							msg->peer_key = rspamd_pubkey_ref(m->key);
+						}
+
+						if (m->local || rspamd_inet_address_is_local(keepalive_addr)) {
+							if (session->fname) {
+								rspamd_http_message_add_header(msg, "File", session->fname);
+							}
+
+							msg->method = HTTP_GET;
+							rspamd_http_connection_write_message_shared(conn,
+																		msg, up_name,
+																		NULL, bk_conn,
+																		bk_conn->timeout);
+						}
+						else {
+							if (session->fname) {
+								msg->flags &= ~RSPAMD_HTTP_FLAG_SHMEM;
+								rspamd_http_message_set_body(msg, session->map, session->map_len);
+							}
+
+							msg->method = HTTP_POST;
+
+							if (m->compress) {
+								proxy_request_compress(msg);
+
+								if (session->client_milter_conn) {
+									rspamd_http_message_add_header(msg, "Content-Type",
+																   "application/octet-stream");
+								}
+							}
+							else {
+								if (session->client_milter_conn) {
+									rspamd_http_message_add_header(msg, "Content-Type",
+																   "text/plain");
+								}
+							}
+
+							rspamd_http_connection_write_message(conn,
+																 msg, up_name, NULL, bk_conn,
+																 bk_conn->timeout);
+						}
+
+						g_ptr_array_add(session->mirror_conns, bk_conn);
+						REF_RETAIN(session);
+						msg_info_session("send request to %s (using keepalive)", m->name);
+
+						/*
+						 * We have found the existing keepalive connection, so we can
+						 * process another mirror
+						 */
+						continue;
+					}
+				}
+			}
+		}
+
+		/* Non-keepalive connection */
+
 		bk_conn = rspamd_mempool_alloc0(session->pool,
 										sizeof(*bk_conn));
 		bk_conn->s = session;
@@ -1449,7 +1825,9 @@ proxy_open_mirror_connections(struct rspamd_proxy_session *session)
 			rspamd_http_message_remove_header(msg, "Host");
 			rspamd_http_message_add_header(msg, "Host", up_name);
 		}
-		rspamd_http_message_add_header(msg, "Connection", "close");
+		rspamd_http_message_remove_header(msg, "Connection");
+		rspamd_http_message_add_header(msg, "Connection",
+									   m->keepalive ? "keep-alive" : "close");
 
 		if (msg->url->len == 0) {
 			msg->url = rspamd_fstring_append(msg->url, "/check", strlen("/check"));
@@ -1460,12 +1838,38 @@ proxy_open_mirror_connections(struct rspamd_proxy_session *session)
 			rspamd_http_message_add_header(msg, "Settings-ID", m->settings_id);
 		}
 
+		/* Add extra headers if specified */
+		if (m->extra_headers != NULL) {
+			ucl_object_iter_t it = NULL;
+			const ucl_object_t *cur;
+			const char *key, *value;
+
+			while ((cur = ucl_object_iterate(m->extra_headers, &it, true)) != NULL) {
+				key = ucl_object_key(cur);
+				value = ucl_object_tostring(cur);
+
+				if (key != NULL && value != NULL) {
+					rspamd_http_message_remove_header(msg, key);
+					rspamd_http_message_add_header(msg, key, value);
+				}
+			}
+		}
+
+		/* Add log tag header based on mirror's configuration */
+		rspamd_proxy_add_log_tag_header(msg, session, m->log_tag_type);
+
+		unsigned int http_opts = RSPAMD_HTTP_CLIENT_SIMPLE;
+
+		if (m->ssl) {
+			http_opts |= RSPAMD_HTTP_CLIENT_SSL;
+		}
+
 		bk_conn->backend_conn = rspamd_http_connection_new_client_socket(
 			session->ctx->http_ctx,
 			NULL,
 			proxy_backend_mirror_error_handler,
 			proxy_backend_mirror_finish_handler,
-			RSPAMD_HTTP_CLIENT_SIMPLE,
+			http_opts,
 			bk_conn->backend_sock);
 
 		if (m->key) {
@@ -1519,8 +1923,8 @@ proxy_open_mirror_connections(struct rspamd_proxy_session *session)
 }
 
 static void
-proxy_client_write_error(struct rspamd_proxy_session *session, gint code,
-						 const gchar *status)
+proxy_client_write_error(struct rspamd_proxy_session *session, int code,
+						 const char *status)
 {
 	struct rspamd_http_message *reply;
 
@@ -1577,8 +1981,9 @@ proxy_backend_master_error_handler(struct rspamd_http_connection *conn, GError *
 	session->retries++;
 	msg_info_session("abnormally closing connection from backend: %s, error: %e,"
 					 " retries left: %d",
-					 rspamd_inet_address_to_string_pretty(
-						 rspamd_upstream_addr_cur(session->master_conn->up)),
+					 session->master_conn->up ? rspamd_inet_address_to_string_pretty(
+													rspamd_upstream_addr_cur(session->master_conn->up))
+											  : "self-scan",
 					 err,
 					 session->ctx->max_retries - session->retries);
 	rspamd_upstream_fail(bk_conn->up, FALSE, err ? err->message : "unknown");
@@ -1609,14 +2014,15 @@ proxy_backend_master_error_handler(struct rspamd_http_connection *conn, GError *
 		else {
 			msg_info_session("retry connection to: %s"
 							 " retries left: %d",
-							 rspamd_inet_address_to_string(
-								 rspamd_upstream_addr_cur(session->master_conn->up)),
+							 session->master_conn->up ? rspamd_inet_address_to_string(
+															rspamd_upstream_addr_cur(session->master_conn->up))
+													  : "self-scan",
 							 session->ctx->max_retries - session->retries);
 		}
 	}
 }
 
-static gint
+static int
 proxy_backend_master_finish_handler(struct rspamd_http_connection *conn,
 									struct rspamd_http_message *msg)
 {
@@ -1624,7 +2030,9 @@ proxy_backend_master_finish_handler(struct rspamd_http_connection *conn,
 	struct rspamd_proxy_session *session, *nsession;
 	rspamd_fstring_t *reply;
 	const rspamd_ftok_t *orig_ct;
+	const rspamd_ftok_t *conn_hdr;
 	goffset body_offset = -1;
+	gboolean is_keepalive = FALSE;
 
 	session = bk_conn->s;
 	rspamd_http_connection_steal_msg(session->master_conn->backend_conn);
@@ -1640,6 +2048,16 @@ proxy_backend_master_finish_handler(struct rspamd_http_connection *conn,
 	rspamd_http_message_remove_header(msg, "Server");
 	rspamd_http_message_remove_header(msg, "Key");
 	orig_ct = rspamd_http_message_find_header(msg, "Content-Type");
+
+	/* Check if we can use keepalive */
+	conn_hdr = rspamd_http_message_find_header(msg, "Connection");
+	if (conn_hdr) {
+		if (rspamd_substring_search_caseless(conn_hdr->begin, conn_hdr->len,
+											 "keep-alive", 10) != -1) {
+			is_keepalive = TRUE;
+		}
+	}
+
 	rspamd_http_connection_reset(session->master_conn->backend_conn);
 
 	if (!proxy_backend_parse_results(session, bk_conn, session->ctx->lua_state,
@@ -1672,6 +2090,22 @@ proxy_backend_master_finish_handler(struct rspamd_http_connection *conn,
 
 	rspamd_upstream_ok(bk_conn->up);
 
+	/* Handle keepalive for master connection */
+	if (is_keepalive && session->use_keepalive &&
+		bk_conn->up && session->ctx->http_ctx) {
+		/* Store connection in keepalive pool */
+		const char *up_name = rspamd_upstream_name(bk_conn->up);
+		if (up_name) {
+			rspamd_http_context_prepare_keepalive(session->ctx->http_ctx,
+												  conn, rspamd_upstream_addr_cur(bk_conn->up),
+												  up_name, FALSE);
+
+			/* We'll push to keepalive pool after we're done with the response */
+			msg_debug_session("will push master connection to %s to keepalive pool",
+							  up_name);
+		}
+	}
+
 	if (session->client_milter_conn) {
 		nsession = proxy_session_refresh(session);
 
@@ -1685,11 +2119,25 @@ proxy_backend_master_finish_handler(struct rspamd_http_connection *conn,
 			rspamd_milter_send_task_results(nsession->client_milter_conn,
 											session->master_conn->results, NULL, 0);
 		}
+
+		/* Push to keepalive if needed */
+		if (is_keepalive && session->use_keepalive &&
+			bk_conn->up && session->ctx->http_ctx) {
+			const char *up_name = rspamd_upstream_name(bk_conn->up);
+			if (up_name) {
+				rspamd_http_context_push_keepalive(session->ctx->http_ctx,
+												   conn, msg, session->ctx->event_loop);
+
+				/* Mark connection as closed without actually closing it */
+				bk_conn->flags |= RSPAMD_BACKEND_CLOSED;
+			}
+		}
+
 		REF_RELEASE(session);
 		rspamd_http_message_free(msg);
 	}
 	else {
-		const gchar *passed_ct = NULL;
+		const char *passed_ct = NULL;
 
 		if (orig_ct) {
 			passed_ct = rspamd_mempool_ftokdup(session->pool, orig_ct);
@@ -1700,6 +2148,19 @@ proxy_backend_master_finish_handler(struct rspamd_http_connection *conn,
 		rspamd_http_connection_write_message(session->client_conn,
 											 msg, NULL, passed_ct, session,
 											 bk_conn->timeout);
+
+		/* Push to keepalive if needed */
+		if (is_keepalive && session->use_keepalive &&
+			bk_conn->up && session->ctx->http_ctx) {
+			const char *up_name = rspamd_upstream_name(bk_conn->up);
+			if (up_name) {
+				rspamd_http_context_push_keepalive(session->ctx->http_ctx,
+												   conn, msg, session->ctx->event_loop);
+
+				/* Mark connection as closed without actually closing it */
+				bk_conn->flags |= RSPAMD_BACKEND_CLOSED;
+			}
+		}
 	}
 
 	return 0;
@@ -1711,7 +2172,17 @@ rspamd_proxy_scan_self_reply(struct rspamd_task *task)
 	struct rspamd_http_message *msg;
 	struct rspamd_proxy_session *session = task->fin_arg, *nsession;
 	ucl_object_t *rep = NULL;
+	int out_type = UCL_EMIT_JSON_COMPACT;
 	const char *ctype = "application/json";
+	const rspamd_ftok_t *accept_hdr = rspamd_task_get_request_header(task, "Accept");
+	rspamd_fstring_t *output;
+	struct rspamd_stat stat_copy;
+
+	if (accept_hdr && rspamd_substring_search(accept_hdr->begin, accept_hdr->len,
+											  "application/msgpack", sizeof("application/msgpack") - 1) != -1) {
+		ctype = "application/msgpack";
+		out_type = UCL_EMIT_MSGPACK;
+	}
 
 	msg = rspamd_http_new_message(HTTP_RESPONSE);
 	msg->date = time(NULL);
@@ -1724,13 +2195,21 @@ rspamd_proxy_scan_self_reply(struct rspamd_task *task)
 	case CMD_CHECK_SPAMC:
 	case CMD_CHECK_V2:
 		rspamd_task_set_finish_time(task);
-		rspamd_protocol_http_reply(msg, task, &rep);
+		rspamd_protocol_http_reply(msg, task, &rep, out_type);
 		rspamd_protocol_write_log_pipe(task);
 		break;
 	case CMD_PING:
 		rspamd_http_message_set_body(msg, "pong" CRLF, 6);
 		ctype = "text/plain";
 		break;
+	case CMD_METRICS:
+		memcpy(&stat_copy, session->ctx->srv->stat, sizeof(stat_copy));
+		output = rspamd_metrics_to_prometheus_string(
+			rspamd_worker_metrics_object(task->cfg, &stat_copy, ev_time() - session->ctx->srv->start_time));
+		rspamd_printf_fstring(&output, "# EOF\n");
+		rspamd_http_message_set_body_from_fstring_steal(msg, output);
+		ctype = "application/openmetrics-text; version=1.0.0; charset=utf-8";
+		break;
 	default:
 		msg_err_task("BROKEN");
 		break;
@@ -1746,7 +2225,7 @@ rspamd_proxy_scan_self_reply(struct rspamd_task *task)
 		nsession = proxy_session_refresh(session);
 
 		if (task->flags & RSPAMD_TASK_FLAG_MESSAGE_REWRITE) {
-			const gchar *start;
+			const char *start;
 			goffset len, hdr_off;
 
 			start = task->msg.begin;
@@ -1833,7 +2312,7 @@ rspamd_proxy_self_scan(struct rspamd_proxy_session *session)
 {
 	struct rspamd_task *task;
 	struct rspamd_http_message *msg;
-	const gchar *data;
+	const char *data;
 	gsize len;
 
 	msg = session->client_message;
@@ -1859,8 +2338,6 @@ rspamd_proxy_self_scan(struct rspamd_proxy_session *session)
 
 	task->fin_arg = session;
 	task->resolver = session->ctx->resolver;
-	/* TODO: allow to disable autolearn in protocol */
-	task->flags |= RSPAMD_TASK_FLAG_LEARN_AUTO;
 	task->s = rspamd_session_create(task->task_pool, rspamd_proxy_task_fin,
 									NULL, (event_finalizer_t) rspamd_task_free, task);
 	data = rspamd_http_message_get_body(msg, &len);
@@ -1877,7 +2354,7 @@ rspamd_proxy_self_scan(struct rspamd_proxy_session *session)
 		task->flags |= RSPAMD_TASK_FLAG_SKIP;
 	}
 	else {
-		if (task->cmd == CMD_PING) {
+		if (task->cmd == CMD_PING || task->cmd == CMD_METRICS) {
 			task->flags |= RSPAMD_TASK_FLAG_SKIP;
 		}
 		else {
@@ -1921,7 +2398,7 @@ proxy_send_master_message(struct rspamd_proxy_session *session)
 	struct rspamd_http_upstream *backend = NULL;
 	const rspamd_ftok_t *host;
 	GError *err = NULL;
-	gchar hostbuf[512];
+	char hostbuf[512];
 
 	host = rspamd_http_message_find_header(session->client_message, "Host");
 
@@ -1943,6 +2420,9 @@ proxy_send_master_message(struct rspamd_proxy_session *session)
 	/* Remove the original `Connection` header */
 	rspamd_http_message_remove_header(session->client_message, "Connection");
 
+	/* Set keepalive flag based on backend configuration */
+	session->use_keepalive = backend ? backend->keepalive : FALSE;
+
 	if (backend == NULL) {
 		/* No backend */
 		msg_err_session("cannot find upstream for %s", host ? hostbuf : "default");
@@ -1964,7 +2444,7 @@ proxy_send_master_message(struct rspamd_proxy_session *session)
 		}
 
 		/* Provide hash key if hashing based on source address is desired */
-		guint hash_len;
+		unsigned int hash_len;
 		gpointer hash_key = rspamd_inet_address_get_hash_key(session->client_addr,
 															 &hash_len);
 
@@ -2024,14 +2504,21 @@ proxy_send_master_message(struct rspamd_proxy_session *session)
 		if (up_name) {
 			rspamd_http_message_add_header(msg, "Host", up_name);
 		}
-		rspamd_http_message_add_header(msg, "Connection", "close");
+		rspamd_http_message_add_header(msg, "Connection",
+									   backend->keepalive ? "keep-alive" : "close");
+
+		unsigned int http_opts = RSPAMD_HTTP_CLIENT_SIMPLE;
+
+		if (backend->ssl) {
+			http_opts |= RSPAMD_HTTP_CLIENT_SSL;
+		}
 
 		session->master_conn->backend_conn = rspamd_http_connection_new_client_socket(
 			session->ctx->http_ctx,
 			NULL,
 			proxy_backend_master_error_handler,
 			proxy_backend_master_finish_handler,
-			RSPAMD_HTTP_CLIENT_SIMPLE,
+			http_opts,
 			session->master_conn->backend_sock);
 		session->master_conn->flags &= ~RSPAMD_BACKEND_CLOSED;
 		session->master_conn->parser_from_ref = backend->parser_from_ref;
@@ -2047,6 +2534,26 @@ proxy_send_master_message(struct rspamd_proxy_session *session)
 										   backend->settings_id);
 		}
 
+		/* Add extra headers if specified */
+		if (backend->extra_headers != NULL) {
+			ucl_object_iter_t it = NULL;
+			const ucl_object_t *cur;
+			const char *key, *value;
+
+			while ((cur = ucl_object_iterate(backend->extra_headers, &it, true)) != NULL) {
+				key = ucl_object_key(cur);
+				value = ucl_object_tostring(cur);
+
+				if (key != NULL && value != NULL) {
+					rspamd_http_message_remove_header(msg, key);
+					rspamd_http_message_add_header(msg, key, value);
+				}
+			}
+		}
+
+		/* Add log tag header based on backend's configuration */
+		rspamd_proxy_add_log_tag_header(msg, session, backend->log_tag_type);
+
 		if (backend->local ||
 			rspamd_inet_address_is_local(
 				rspamd_upstream_addr_cur(
@@ -2124,7 +2631,7 @@ proxy_client_error_handler(struct rspamd_http_connection *conn, GError *err)
 	REF_RELEASE(session);
 }
 
-static gint
+static int
 proxy_client_finish_handler(struct rspamd_http_connection *conn,
 							struct rspamd_http_message *msg)
 {
@@ -2174,7 +2681,11 @@ proxy_client_finish_handler(struct rspamd_http_connection *conn,
 		proxy_send_master_message(session);
 	}
 	else {
-		msg_info_session("finished master connection");
+		msg_info_session("finished master connection to %s; HTTP code: %d",
+						 session->master_conn->up ? rspamd_inet_address_to_string_pretty(
+														rspamd_upstream_addr_cur(session->master_conn->up))
+												  : "self-scan",
+						 msg->code);
 		proxy_backend_close_connection(session->master_conn);
 		REF_RELEASE(session);
 	}
@@ -2195,7 +2706,7 @@ err:
 }
 
 static void
-proxy_milter_finish_handler(gint fd,
+proxy_milter_finish_handler(int fd,
 							struct rspamd_milter_session *rms,
 							void *ud)
 {
@@ -2226,7 +2737,7 @@ proxy_milter_finish_handler(gint fd,
 }
 
 static void
-proxy_milter_error_handler(gint fd,
+proxy_milter_error_handler(int fd,
 						   struct rspamd_milter_session *rms, /* unused */
 						   void *ud, GError *err)
 {
@@ -2259,7 +2770,7 @@ proxy_accept_socket(EV_P_ ev_io *w, int revents)
 	struct rspamd_proxy_ctx *ctx;
 	rspamd_inet_addr_t *addr = NULL;
 	struct rspamd_proxy_session *session;
-	gint nfd;
+	int nfd;
 
 	ctx = worker->ctx;
 
@@ -2275,6 +2786,8 @@ proxy_accept_socket(EV_P_ ev_io *w, int revents)
 		return;
 	}
 
+	worker->nconns++;
+
 	session = g_malloc0(sizeof(*session));
 	REF_INIT_RETAIN(session, proxy_session_dtor);
 	session->client_sock = nfd;
@@ -2329,7 +2842,7 @@ proxy_accept_socket(EV_P_ ev_io *w, int revents)
 #endif
 
 		if (rspamd_inet_address_get_af(addr) != AF_UNIX) {
-			gint sopt = 1;
+			int sopt = 1;
 
 			if (setsockopt(nfd, SOL_TCP, TCP_NODELAY, &sopt, sizeof(sopt)) ==
 				-1) {
@@ -2384,6 +2897,7 @@ start_rspamd_proxy(struct rspamd_worker *worker)
 
 	g_assert(rspamd_worker_check_context(worker->ctx, rspamd_rspamd_proxy_magic));
 	ctx->cfg = worker->srv->cfg;
+	ctx->srv = worker->srv;
 	ctx->event_loop = rspamd_prepare_worker(worker, "rspamd_proxy",
 											proxy_accept_socket);
 
@@ -2423,6 +2937,8 @@ start_rspamd_proxy(struct rspamd_worker *worker)
 	ctx->milter_ctx.sessions_cache = ctx->sessions_cache;
 	ctx->milter_ctx.client_ca_name = ctx->client_ca_name;
 	ctx->milter_ctx.reject_message = ctx->reject_message;
+	ctx->milter_ctx.quarantine_message = ctx->quarantine_message;
+	ctx->milter_ctx.tempfail_message = ctx->tempfail_message;
 	ctx->milter_ctx.cfg = ctx->cfg;
 	rspamd_milter_init_library(&ctx->milter_ctx);
 
